/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * This class is used to organize global SpaceNet preferences to apply to all
 * scenarios.
 * 
 * @author Paul Grogan
 */
public class Preferences {
	private String userName;
	private String passwordHash;
	private boolean loginSaved;
	
	private String defaultDirectory;
	private boolean autoRefresh;
	
	/**
	 * The default constructor.
	 */
	public Preferences() {
		userName = "";
		passwordHash = "";
		loginSaved = false;
		
		defaultDirectory = null;
		autoRefresh = true;
	}
	
	/**
	 * Gets the default SpaceNetOnline user name.
	 * 
	 * @return the user name
	 */
	public String getUserName() {
		return userName;
	}
	
	/**
	 * Sets the default SpaceNetOnline user name.
	 * 
	 * @param userName the user name
	 */
	public void setUserName(String userName) {
		this.userName = userName;
	}
	
	/**
	 * Gets the saved SpaceNetOnline password hash.
	 * 
	 * @return the password hash
	 */
	public String getPasswordHash() {
		return passwordHash;
	}
	
	/**
	 * Sets the saved SpaceNetOnline password hash.
	 * 
	 * @param passwordHash the password hash
	 */
	public void setPasswordHash(String passwordHash) {
		this.passwordHash = passwordHash;
	}
	
	/**
	 * Hashes a password using the MD5 algorithm.
	 * 
	 * @param password the password to hash
	 * 
	 * @return the hashed password
	 */
	public static String hashPassword(String password) {
		MessageDigest m = null;
		try {
			m = MessageDigest.getInstance("MD5");
			m.update(password.getBytes(),0,password.length());
		} catch (NoSuchAlgorithmException e) {
			System.err.println("Hashing error");
		}
		return new BigInteger(1,m.digest()).toString(16);
	}
	
	/**
	 * Gets whether the SpaceNetOnline login information is saved.
	 * 
	 * @return whether the SpaceNetOnline login information is saved
	 */
	public boolean isLoginSaved() {
		return loginSaved;
	}
	
	/**
	 * Sets whether the SpaceNetOnline login information is saved.
	 * 
	 * @param loginSaved whether the SpaceNetOnline login information is
	 * saved
	 */
	public void setLoginSaved(boolean loginSaved) {
		this.loginSaved = loginSaved;
	}
	
	/**
	 * Gets the default file path directory.
	 * 
	 * @return the default directory
	 */
	public String getDefaultDirectory() {
		return defaultDirectory==null?System.getProperty("user.dir"):defaultDirectory;
	}
	
	/**
	 * Sets the default file directory.
	 * 
	 * @param defaultDirectory the default directory
	 */
	public void setDefaultDirectory(String defaultDirectory) {
		this.defaultDirectory = defaultDirectory;
	}
	
	/**
	 * Gets whether charts should automatically refresh when inputs are changed.
	 * 
	 * @return whether charts should auto-refresh
	 */
	public boolean isAutoRefresh() {
		return autoRefresh;
	}
	
	/**
	 * Sets whether charts should automatically refresh when inputs are changed.
	 * 
	 * @param autoRefresh whether charts should auto-refresh
	 */
	public void setAutoRefresh(boolean autoRefresh) {
		this.autoRefresh = autoRefresh;
	}
}
