/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.domain.element;

import java.util.SortedMap;
import java.util.TreeMap;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.Environment;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.domain.resource.DemandSet;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.simulator.I_Simulator;
import edu.mit.spacenet.util.GlobalParameters;

/**
 * An element that can hold many types of continuous resources up to set
 * mass, volume, and environment constraints (volume and environment can be
 * disabled via global parameters).
 * 
 * @author Paul Grogan
 */
public class ResourceContainer extends Element implements I_ResourceContainer {
	private double maxCargoMass, maxCargoVolume;
	private Environment cargoEnvironment;
	@Deprecated private boolean isCollapsible;
	protected SortedMap<I_Resource, Double> contents;
	
	/**
	 * Instantiates a new resource container.
	 */
	public ResourceContainer() {
		super();
		setCargoEnvironment(Environment.UNPRESSURIZED);
		setClassOfSupply(ClassOfSupply.COS501);
		contents = new TreeMap<I_Resource, Double>();
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#canAdd(edu.mit.spacenet.domain.resource.I_Resource, double)
	 */
	public boolean canAdd(I_Resource resource, double amount) {
		if(getCargoMass() + resource.getUnitMass()*amount - maxCargoMass > GlobalParameters.getMassPrecision()/2d) {
			return false; // mass constrained
		} else if(GlobalParameters.isVolumeConstrained()
					&& getCargoVolume() + resource.getUnitVolume()*amount - maxCargoVolume > GlobalParameters.getVolumePrecision()/2d) {
			return false; // volume constrained
		} else if(GlobalParameters.isEnvironmentConstrained()
					&& resource.getEnvironment().equals(Environment.PRESSURIZED)
					&& getCargoEnvironment().equals(Environment.UNPRESSURIZED)) {
			return false; // environment constrained
		} else return true;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#add(edu.mit.spacenet.domain.resource.I_Resource, double)
	 */
	public boolean add(I_Resource resource, double amount) {
		if(canAdd(resource, amount)) {
			if(contents.keySet().contains(resource)) {
				double q = contents.get(resource);
				contents.put(resource, q+amount);
			} else {
				contents.put(resource, amount);
			}
			return true;
		} else return false;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#remove(edu.mit.spacenet.domain.resource.I_Resource, double)
	 */
	public boolean remove(I_Resource resource, double amount) {
		if(contents.keySet().contains(resource)
				&& contents.get(resource) > amount) {
			double q = contents.get(resource);
			contents.put(resource, q-amount);
			return true;
		} else if(contents.keySet().contains(resource)
				&& contents.get(resource) == amount) {
			contents.remove(resource);
			return true;
		} else return false;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#getCargoEnvironment()
	 */
	public Environment getCargoEnvironment() {
		return cargoEnvironment;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#setCargoEnvironment(edu.mit.spacenet.domain.Environment)
	 */
	public void setCargoEnvironment(Environment cargoEnvironment) {
		this.cargoEnvironment = cargoEnvironment;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#getMaxCargoMass()
	 */
	public double getMaxCargoMass() {
		return GlobalParameters.getRoundedMass(maxCargoMass);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#setMaxCargoMass(double)
	 */
	public void setMaxCargoMass(double maxCargoMass) {
		this.maxCargoMass = maxCargoMass;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#getMaxCargoVolume()
	 */
	public double getMaxCargoVolume() {
		return GlobalParameters.getRoundedVolume(maxCargoVolume);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#setMaxCargoVolume(double)
	 */
	public void setMaxCargoVolume(double maxCargoVolume) {
		this.maxCargoVolume = maxCargoVolume;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#getCargoVolume()
	 */
	public double getCargoVolume() {
		double volume = 0;
		for(I_Resource r : contents.keySet()) {
			volume += r.getUnitVolume()*contents.get(r);
		}
		return GlobalParameters.getRoundedVolume(volume);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#getCargoMass()
	 */
	public double getCargoMass() {
		double mass = 0;
		for(I_Resource r : contents.keySet()) {
			mass += r.getUnitMass()*contents.get(r);
		}
		return GlobalParameters.getRoundedMass(mass);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.I_ResourceContainer#getContents()
	 */
	public SortedMap<I_Resource, Double> getContents() {
		return contents;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.Element#getTotalMass()
	 */
	@Override
	public double getTotalMass() {
		return GlobalParameters.getRoundedMass(super.getTotalMass() + getCargoMass());
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.Element#getTotalMass(edu.mit.spacenet.domain.ClassOfSupply)
	 */
	@Override
	public double getTotalMass(ClassOfSupply cos) {
		double amount = super.getTotalMass(cos);
		
		for(I_Resource resource : getContents().keySet()) {
			if(resource.getClassOfSupply().isInstanceOf(cos)) {
				amount += getContents().get(resource)*resource.getUnitMass();
			}
		}
		
		return GlobalParameters.getRoundedMass(amount);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.Element#getTotalVolume()
	 */
	@Override
	public double getTotalVolume() {
		if(isCollapsible) {
			return GlobalParameters.getRoundedVolume(getCargoVolume());
		} else return GlobalParameters.getRoundedVolume(super.getVolume());
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.DomainType#print(int)
	 */
	@Override
	public void print(int tabOrder) {
		super.print(tabOrder);
		for(I_Resource r : contents.keySet()) {
			r.print(tabOrder + 1);
		}
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.Element#satisfyDemands(edu.mit.spacenet.domain.resource.DemandSet, edu.mit.spacenet.simulator.I_Simulator)
	 */
	@Override
	public void satisfyDemands(DemandSet demands, I_Simulator simulator) {
		for(Demand demand : demands) {
			if(contents.containsKey(demand.getResource())) {
				if(demand.getAmount() > 0) { // consumption
					if(demand.getAmount() >= contents.get(demand.getResource())) {
						demand.setAmount(demand.getAmount() -  contents.get(demand.getResource()));
						contents.remove(demand.getResource());
					} else {
						contents.put(demand.getResource(), contents.get(demand.getResource())-demand.getAmount());
						demands.remove(demand);
						break;
					}
				} else { // production
					if(demand.getAmount() > getMaxCargoMass() - getCargoMass()) {
						demand.setAmount(demand.getAmount() - (getMaxCargoMass() - getCargoMass()));
						contents.put(demand.getResource(), contents.get(demand.getResource()) + getMaxCargoMass() - getCargoMass()); 
					} else {
						contents.put(demand.getResource(), contents.get(demand.getResource()) + demand.getAmount()); 
						demands.remove(demand);
						break;
					}
				}
			}
		}
	}
	
	/**
	 * Gets whether the container's volume can shrink if it is not filled completely.
	 * 
	 * @return if the container is collapsible
	 */
	@Deprecated public boolean isCollapsible() {
		return isCollapsible;
	}
	
	/**
	 * Sets whether the container's volume can shrink if it is not filled completely.
	 * 
	 * @param isCollapsible if the container is collapsible
	 */
	@Deprecated public void setCollapsible(boolean isCollapsible) {
		this.isCollapsible = isCollapsible;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.element.Element#getElementType()
	 */
	@Override
	public ElementType getElementType() {
		return ElementType.RESOURCE_CONTAINER;
	}
}