/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.domain.model;

import edu.mit.spacenet.domain.element.ResourceTank;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.domain.resource.DemandSet;
import edu.mit.spacenet.simulator.I_Simulator;

/**
 * Demand model that calculates the boil-off of fuel from (typically cryogenic)
 * fuel tanks. Demands at each time step are calculated based on the current
 * amount of remaining fuel, the boil-off rate, and the duration.
 * 
 * @author Paul Grogan
 */
public class BoilOffDemandModel extends AbstractDemandModel {
	private ResourceTank tank;
	private double rate;
	
	/**
	 * The default constructor sets the boil-off rate to value from SpaceNet
	 * v1.3 file: "demand/boiloffcalc.m"
	 */
	public BoilOffDemandModel() {
		// Data from SpaceNet v1.3 file: "demand/boiloffcalc.m"
		double lh2Fraction = 3586d/(3586d+21519d); 		
		// ESAS Appendix 4E: using 3586kg of LH2, 21519kg of LOX
		double lo2Fraction = 21519d/(3586d+21519d);
		double lh2Rate = 221d/3586d/25d; 				
		// ESAS Appendix 4E: 221kg of boil-off from 3586kg of LH2 for 25 days
		double lo2Rate = 163d/21519d/25d; 				
		// ESAS Appendix 4E: 163kg of boil-off from 21519kg of LOX for 25 days
		this.rate = lh2Rate*lh2Fraction + lo2Rate*lo2Fraction;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#generateDemands(double, edu.mit.spacenet.simulator.I_Simulator)
	 */
	public DemandSet generateDemands(double duration, I_Simulator simulator) {
		DemandSet demands = new DemandSet();
		Demand demand = new Demand();
		demand.setResource(tank.getResource());
		demand.setAmount(tank.getAmount()*rate*duration);
		demands.add(demand);
		return demands;
	}
	
	/**
	 * Gets the targeted resource container.
	 * 
	 * @return the container
	 */
	public ResourceTank getTank() {
		return tank;
	}
	
	/**
	 * Sets the targeted resource tank.
	 * 
	 * @param tank the tank
	 */
	public void setTank(ResourceTank tank) {
		this.tank = tank;
	}
	
	/**
	 * Gets the boil-off rate.
	 * 
	 * @return the boil-off rate (percent of remaining amount per day)
	 */
	public double getRate() {
		return rate;
	}
	
	/**
	 * Sets the boil-off rate.
	 * 
	 * @param rate the boil-off rate (percent of remaining amount per day)
	 */
	public void setRate(double rate) {
		this.rate = rate;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#getDemandModelType()
	 */
	public DemandModelType getDemandModelType() {
		return DemandModelType.BOIL_OFF;
	}
}