/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.domain.model;

import edu.mit.spacenet.domain.element.ResourceTank;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.domain.resource.DemandSet;
import edu.mit.spacenet.simulator.I_Simulator;

/**
 * Demand model that demands all of the remaining resources in the container
 * after the specified shelf life.
 * 
 * @author Paul Grogan
 */
public class ExponentialDecayDemandModel extends AbstractDemandModel {
	private ResourceTank container;
	private double halfLife;
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#generateDemands(double, edu.mit.spacenet.simulator.I_Simulator)
	 */
	public DemandSet generateDemands(double duration, I_Simulator simulator) {
		DemandSet demands = new DemandSet();
		Demand demand = new Demand();
		demand.setResource(container.getResource());
		demand.setAmount(container.getAmount()*Math.pow(2, -duration/halfLife));
		demands.add(demand);
		return demands;
	}
	
	/**
	 * Gets the targeted resource container.
	 * 
	 * @return the container
	 */
	public ResourceTank getContainer() {
		return container;
	}
	
	/**
	 * Sets the targeted resource container.
	 * 
	 * @param container the container
	 */
	public void setContainer(ResourceTank container) {
		this.container = container;
	}
	
	/**
	 * Gets the half life.
	 * 
	 * @return the half life (days)
	 */
	public double getHalfLife() {
		return halfLife;
	}
	
	/**
	 * Sets the half life.
	 * 
	 * @param halfLife the half life (days)
	 */
	public void setHalfLife(double halfLife) {
		this.halfLife = halfLife;
	}
	
	/**
	 * Gets the exponential decay rate.
	 * 
	 * @return the decay rate (1 / day)
	 */
	public double getDecayRate() {
		return Math.log(2)/halfLife;
	}
	
	/**
	 * Gets the exponential rate constant.
	 * 
	 * @return the rate constant (days)
	 */
	public double getRateConstant() {
		return halfLife/Math.log(2);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#getDemandModelType()
	 */
	public DemandModelType getDemandModelType() {
		return DemandModelType.EXPONENTIAL_DECAY;
	}
}