/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.domain.model;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.domain.resource.DemandSet;
import edu.mit.spacenet.domain.resource.GenericResource;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.simulator.I_Simulator;

/**
 * ISRU Demand model based on a power law equation:
 * y = a*x^k
 * where
 * y = plant mass (kg)
 * x = O2 production (kg/year).
 * 
 * @author Paul Grogan, Ariane Chepko
 */
public class IsruProductionModel extends AbstractDemandModel {
	private I_Resource resource;
	private I_Element element;
	private static final double a = 0.4656;
	private static final double k = 0.8373;
	
	/**
	 * Instantiates a new isru production model.
	 * 
	 * @param element the element
	 */
	public IsruProductionModel(I_Element element) {
		this.element = element;
		setResource(new GenericResource(ClassOfSupply.COS203));
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#generateDemands(double, edu.mit.spacenet.simulator.I_Simulator)
	 */
	public DemandSet generateDemands(double duration, I_Simulator simulator) {
		DemandSet demands = new DemandSet();
		Demand demand = new Demand();
		demand.setResource(resource);
		demand.setAmount(-getAnnualProduction()/365);
		demands.add(demand);
		return demands;
	}
	
	/**
	 * Gets the annual production.
	 * 
	 * @return the annual production
	 */
	public double getAnnualProduction() {
		return Math.pow((element.getMass()/a),(1/k));
	}
	
	/**
	 * Gets the element.
	 * 
	 * @return the element
	 */
	public I_Element getElement() {
		return element;
	}
	
	/**
	 * Sets the element.
	 * 
	 * @param element the new element
	 */
	public void setElement(I_Element element) {
		this.element = element;
	}
	
	/**
	 * Gets the resource.
	 * 
	 * @return the resource
	 */
	public I_Resource getResource() {
		return resource;
	}
	
	/**
	 * Sets the resource.
	 * 
	 * @param resource the new resource
	 */
	public void setResource(I_Resource resource) {
		this.resource = resource;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#getDemandModelType()
	 */
	public DemandModelType getDemandModelType() {
		return DemandModelType.ISRU;
	}
}
