/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.domain.model;

import edu.mit.spacenet.domain.element.ResourceTank;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.domain.resource.DemandSet;
import edu.mit.spacenet.simulator.I_Simulator;

/**
 * Demand model that demands all of the remaining resources in the container
 * after the specified shelf life (only occurs once).
 * 
 * @author Paul Grogan
 */
public class ShelfLifeDemandModel extends AbstractDemandModel {
	private ResourceTank container;
	private double shelfLife;
	private double timer;
	private boolean hasTriggered;
	
	/**
	 * The default constructor initializes the timer to 0.
	 */
	public ShelfLifeDemandModel() {
		timer = 0;
		hasTriggered = false;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#generateDemands(double, edu.mit.spacenet.simulator.I_Simulator)
	 */
	public DemandSet generateDemands(double duration, I_Simulator simulator) {
		timer += duration;
		if(timer > shelfLife && !hasTriggered) {
			DemandSet demands = new DemandSet();
			Demand demand = new Demand();
			demand.setResource(container.getResource());
			demand.setAmount(container.getAmount());
			demands.add(demand);
			return demands;
		}
		else return new DemandSet();
	}
	
	/**
	 * Gets the targeted resource container.
	 * 
	 * @return the container
	 */
	public ResourceTank getContainer() {
		return container;
	}
	
	/**
	 * Sets the targeted resource container.
	 * 
	 * @param container the container
	 */
	public void setContainer(ResourceTank container) {
		this.container = container;
	}
	
	/**
	 * Gets the shelf life.
	 * 
	 * @return the shelf life (days)
	 */
	public double getShelfLife() {
		return shelfLife;
	}
	
	/**
	 * Sets the shelf life.
	 * 
	 * @param shelfLife the shelf life (days)
	 */
	public void setShelfLife(double shelfLife) {
		this.shelfLife = shelfLife;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#getDemandModelType()
	 */
	public DemandModelType getDemandModelType() {
		return DemandModelType.SHELF_LIFE;
	}
}