/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.domain.model;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.Environment;
import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.element.PartApplication;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.domain.resource.DemandSet;
import edu.mit.spacenet.domain.resource.GenericResource;
import edu.mit.spacenet.simulator.I_Simulator;

/**
 * Crude sparing demand model that demands generic class of supply 4 resources
 * proportional to the element's mass, a annual sparing rate, and the duration.
 * 
 * @author Paul Grogan
 */
public class SparingByMassDemandModel extends AbstractDemandModel {
	private I_Element element;
	private double unpressurizedSparesRate;
	private double pressurizedSparesRate;
	
	/**
	 * The default constructor sets the default packaging factors.
	 * 
	 * @param element the element
	 */
	public SparingByMassDemandModel(I_Element element) {
		this.element = element;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#generateDemands(double, edu.mit.spacenet.simulator.I_Simulator)
	 */
	public DemandSet generateDemands(double duration, I_Simulator simulator) {
		DemandSet demands = new DemandSet();
		
		double pressPartMass = 0;
		double unpressPartMass = 0;
		for(PartApplication p : element.getParts()) {
			if(p.getQuantity()>0) {
				Demand demand = new Demand();
				demand.setResource(p.getPart());
				demand.setAmount(duration*p.getQuantity()*unpressurizedSparesRate/365);
				if(p.getPart().getEnvironment().equals(Environment.PRESSURIZED)) {
					pressPartMass += demand.getMass();
				} else if(p.getPart().getEnvironment().equals(Environment.UNPRESSURIZED)) {
					unpressPartMass += demand.getMass();
				}
				demands.add(demand);
			}
		}
		
		Demand unpressSpares = new Demand();
		unpressSpares.setResource(new GenericResource(ClassOfSupply.COS4));
		unpressSpares.setAmount(duration*unpressurizedSparesRate*element.getMass()/365 - unpressPartMass);
		demands.add(unpressSpares);
		
		Demand pressSpares = new Demand();
		pressSpares.setResource(new GenericResource(ClassOfSupply.COS4));
		pressSpares.getResource().setEnvironment(Environment.PRESSURIZED);
		pressSpares.setAmount(duration*pressurizedSparesRate*element.getMass()/365);
		demands.add(pressSpares);
		
		return demands;
	}
	
	/**
	 * Gets the element.
	 * 
	 * @return the element
	 */
	public I_Element getElement() {
		return element;
	}
	
	/**
	 * Sets the element.
	 * 
	 * @param element the element
	 */
	public void setElement(I_Element element) {
		this.element = element;
	}
	
	/**
	 * Gets the unpressurized sparing rate.
	 * 
	 * @return the sparing rate (percent element mass per year)
	 */
	public double getUnpressurizedSparesRate() {
		return unpressurizedSparesRate;
	}
	
	/**
	 * Sets the unpressurized sparing rate.
	 * 
	 * @param sparesRate the sparing rate (percent element mass per year)
	 */
	public void setUnpressurizedSparesRate(double sparesRate) {
		this.unpressurizedSparesRate = sparesRate;
	}
	
	/**
	 * Gets the pressurized sparing rate.
	 * 
	 * @return the sparing rate (percent element mass per year)
	 */
	public double getPressurizedSparesRate() {
		return pressurizedSparesRate;
	}
	
	/**
	 * Sets the pressurized sparing rate.
	 * 
	 * @param sparesRate the spares rate
	 */
	public void setPressurizedSparesRate(double sparesRate) {
		this.pressurizedSparesRate = sparesRate;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.domain.model.I_DemandModel#getDemandModelType()
	 */
	public DemandModelType getDemandModelType() {
		return DemandModelType.SPARING_BY_MASS;
	}
}