/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerNumberModel;

import edu.mit.spacenet.gui.component.UnitsWrapper;
import edu.mit.spacenet.scenario.ItemDiscretization;

/**
 * Dialog that displays options related to a specific scenario.
 * 
 * @author Paul Grogan
 */
public class OptionsDialog extends JDialog {
	private static final long serialVersionUID = -802497885876127342L;
	
	private ScenarioPanel scenarioPanel;
	
	private JPanel precisionPanel, constraintsPanel, demandsPanel, simulationPanel;
	private SpinnerNumberModel timeModel, demandModel, massModel, volumeModel;
	private JSpinner timeSpinner, demandSpinner, massSpinner, volumeSpinner;
	private JCheckBox volumeConstrained, environmentConstrained;
	
	private JComboBox discretizationCombo;
	private JSlider aggregationSlider;
	private JCheckBox scavengeSparesCheck;
	
	private JCheckBox explorationCheck, evaCheck;
	
	/**
	 * The constructor.
	 * 
	 * @param scenarioPanel the parent scenario panel component
	 */
	public OptionsDialog(ScenarioPanel scenarioPanel) {
		super(scenarioPanel.getSpaceNetFrame(), "Scenario Options");
		this.scenarioPanel = scenarioPanel;
		
		buildDialog();
	}
	private void buildDialog() {
		buildPrecisionPanel();
		buildConstraintsPanel();
		buildDemandsPanel();
		buildSimulationPanel();
		
		JPanel contentPanel = new JPanel();
		contentPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Precision", precisionPanel);
		tabbedPane.addTab("Constraints", constraintsPanel);
		tabbedPane.addTab("Demands", demandsPanel);
		tabbedPane.addTab("Simulation", simulationPanel);
		contentPanel.add(tabbedPane, c);
		c.gridy++;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LAST_LINE_END;
		c.fill = GridBagConstraints.NONE;
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 3, 0));
		JButton okButton = new JButton("OK");
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveOptions();
				scenarioPanel.updateView();
				dispose();
			}
		});
		buttonPanel.add(okButton, c);
		getRootPane().setDefaultButton(okButton);
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		buttonPanel.add(cancelButton, c);
		contentPanel.add(buttonPanel, c);
		
		setModal(true);
		setContentPane(contentPanel);
		setMinimumSize(new Dimension(300,250));
	}
	private void buildPrecisionPanel() {
		precisionPanel = new JPanel();
		precisionPanel.setBackground(Color.WHITE);
		precisionPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		precisionPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		precisionPanel.add(new JLabel("Time Precision: "), c);
		c.gridy++;
		precisionPanel.add(new JLabel("Demand Precision: "), c);
		c.gridy++;
		precisionPanel.add(new JLabel("Mass Precision: "), c);
		c.gridy++;
		precisionPanel.add(new JLabel("Volume Precision: "), c);
		c.gridy++;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.NONE;
		c.gridx = 1;
		c.gridy = 0;
		timeModel = new SpinnerNumberModel(scenarioPanel.getScenario().getTimePrecision(),0.001,1.000,0.001);
		timeSpinner = new JSpinner(timeModel);
		precisionPanel.add(new UnitsWrapper(timeSpinner, "days"), c);
		c.gridy++;
		demandModel = new SpinnerNumberModel(scenarioPanel.getScenario().getDemandPrecision(),0.001,1.000,0.001);
		demandSpinner = new JSpinner(demandModel);
		precisionPanel.add(new UnitsWrapper(demandSpinner, "units"), c);
		c.gridy++;
		massModel = new SpinnerNumberModel(scenarioPanel.getScenario().getMassPrecision(),0.001,1.000,0.001);
		massSpinner = new JSpinner(massModel);
		precisionPanel.add(new UnitsWrapper(massSpinner, "kg"), c);
		c.gridy++;
		volumeModel = new SpinnerNumberModel(scenarioPanel.getScenario().getVolumePrecision()*1000000,0.1,100.,0.1);
		volumeSpinner = new JSpinner(volumeModel);
		precisionPanel.add(new UnitsWrapper(volumeSpinner, "cm^3"), c);
		c.gridy++;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		precisionPanel.add(new JLabel(), c);
	}
	private void buildConstraintsPanel() {
		constraintsPanel = new JPanel();
		constraintsPanel.setBackground(Color.WHITE);
		constraintsPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		constraintsPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		volumeConstrained = new JCheckBox("Volume Constraints Enforced");
		volumeConstrained.setSelected(scenarioPanel.getScenario().isVolumeConstrained());
		volumeConstrained.setOpaque(false);
		constraintsPanel.add(volumeConstrained, c);
		c.gridy++;
		environmentConstrained = new JCheckBox("Environment Constraints Enforced");
		environmentConstrained.setSelected(scenarioPanel.getScenario().isEnvironmentConstrained());
		environmentConstrained.setOpaque(false);
		environmentConstrained.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				scenarioPanel.getScenario().setEnvironmentConstrained(e.getStateChange()==ItemEvent.SELECTED);
			}
		});
		constraintsPanel.add(environmentConstrained, c);
		c.gridy++;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		constraintsPanel.add(new JLabel(), c);
	}
	private void buildDemandsPanel() {
		demandsPanel = new JPanel();
		demandsPanel.setBackground(Color.WHITE);
		demandsPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		demandsPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_END;
		demandsPanel.add(new JLabel("Item Discretization: "), c);
		c.gridy++;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		demandsPanel.add(new JLabel("Item Aggregation: "), c);
		c.weightx = 1;
		c.gridx = 1;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		discretizationCombo = new JComboBox();
		discretizationCombo.setToolTipText("Discretize demands for items to integer values at the selected level");
		for(ItemDiscretization t : ItemDiscretization.values()) {
			discretizationCombo.addItem(t);
		}
		discretizationCombo.setSelectedItem(scenarioPanel.getScenario().getItemDiscretization());
		discretizationCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED) {
					aggregationSlider.setEnabled(discretizationCombo.getSelectedItem()!=ItemDiscretization.NONE);
				}
			}
		});
		demandsPanel.add(discretizationCombo, c);
		c.gridy++;
		aggregationSlider = new JSlider(JSlider.VERTICAL, 0,4,(int)(scenarioPanel.getScenario().getItemAggregation()*4));
		aggregationSlider.setOpaque(false);
		aggregationSlider.setEnabled(discretizationCombo.getSelectedItem()!=ItemDiscretization.NONE);
		aggregationSlider.setFocusable(false);
		aggregationSlider.setToolTipText("Aggregate discretized item demands ahead or behind demands.");
		Hashtable<Integer, JLabel> labelTable  = new Hashtable<Integer, JLabel>();
		labelTable.put(new Integer(0), new JLabel("First Demand", JLabel.LEFT));
		labelTable.put(new Integer(2), new JLabel("Half Demand", JLabel.LEFT));
		labelTable.put(new Integer(4), new JLabel("Unit Demand", JLabel.RIGHT));
		aggregationSlider.setLabelTable(labelTable);
		aggregationSlider.setPaintLabels(true);
		aggregationSlider.setMajorTickSpacing(1);
		aggregationSlider.setSnapToTicks(true);
		aggregationSlider.setPaintTicks(true);
		aggregationSlider.setPreferredSize(new Dimension(100,100));
		demandsPanel.add(aggregationSlider, c);
		c.gridy++;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 2;
		c.gridx = 0;
		scavengeSparesCheck = new JCheckBox("Scavenge Spares");
		scavengeSparesCheck.setOpaque(false);
		scavengeSparesCheck.setSelected(scenarioPanel.getScenario().isScavengeSpares());
		demandsPanel.add(scavengeSparesCheck, c);
		c.gridy++;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		demandsPanel.add(new JLabel(), c);
	}
	private void buildSimulationPanel() {
		simulationPanel = new JPanel();
		simulationPanel.setBackground(Color.WHITE);
		simulationPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		simulationPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		explorationCheck = new JCheckBox("Detailed Explorations");
		explorationCheck.setSelected(scenarioPanel.getScenario().isDetailedExploration());
		explorationCheck.setOpaque(false);
		explorationCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				scenarioPanel.getScenario().setDetailedExploration(e.getStateChange()==ItemEvent.SELECTED);
			}
		});
		simulationPanel.add(explorationCheck, c);
		c.gridy++;
		evaCheck = new JCheckBox("Detailed EVAs");
		evaCheck.setSelected(scenarioPanel.getScenario().isDetailedEva());
		evaCheck.setOpaque(false);
		evaCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				scenarioPanel.getScenario().setDetailedEva(e.getStateChange()==ItemEvent.SELECTED);
			}
		});
		simulationPanel.add(evaCheck, c);
		c.gridy++;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		simulationPanel.add(new JLabel(), c);
	}
	private void saveOptions() {
		scenarioPanel.updateView();
		scenarioPanel.getScenario().setTimePrecision(timeModel.getNumber().doubleValue());
		scenarioPanel.getScenario().setDemandPrecision(demandModel.getNumber().doubleValue());
		scenarioPanel.getScenario().setVolumePrecision(volumeModel.getNumber().doubleValue()/1000000D);
		scenarioPanel.getScenario().setVolumeConstrained(volumeConstrained.isSelected());
		scenarioPanel.getScenario().setEnvironmentConstrained(environmentConstrained.isSelected());
		scenarioPanel.getScenario().setItemDiscretization((ItemDiscretization)discretizationCombo.getSelectedItem());
		scenarioPanel.getScenario().setItemAggregation(aggregationSlider.getValue()/4D);
		scenarioPanel.getScenario().setScavengeSpares(scavengeSparesCheck.isSelected());
		scenarioPanel.getScenario().setDetailedEva(evaCheck.isSelected());
		scenarioPanel.getScenario().setDetailedExploration(explorationCheck.isSelected());
	}
	
	/**
	 * Creates and displays the options dialog.
	 * 
	 * @param scenarioPanel the parent scenario panel component
	 */
	public static void createAndShowGUI(ScenarioPanel scenarioPanel) {
		OptionsDialog dialog = new OptionsDialog(scenarioPanel);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setResizable(false);
		dialog.setVisible(true);
	}
}
