/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileNotFoundException;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import edu.mit.spacenet.Preferences;
import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.gui.command.ExitCommand;
import edu.mit.spacenet.io.XStreamEngine;
import edu.mit.spacenet.scenario.Scenario;
import edu.mit.spacenet.util.IdGenerator;

/**
 * The frame to contain the SpaceNet application.
 * 
 * @author Paul Grogan
 */
public class SpaceNetFrame extends JFrame {
	private static final long serialVersionUID = -6005760557525734285L;
	
	private static SpaceNetFrame instance;
	
	/**
	 * Gets the singleton reference to the SpaceNet frame.
	 * 
	 * @return the singleton frame
	 */
	public static SpaceNetFrame getSingleton() {
		if(instance==null) instance = new SpaceNetFrame();
		return instance;
	}
	
	private Preferences preferences;
	private SpaceNetMenuBar menuBar;
	private JPanel contentPanel;
	private ScenarioPanel scenarioPanel;
	private SpaceNetStatusBar statusBar;
	
	private SpaceNetFrame() {
		setPreferences(XStreamEngine.readPreferences());
		buildFrame();
		updateView();
	}
	private void buildFrame() {
		setIconImage(new ImageIcon(getClass().getClassLoader().getResource("icons/spacenet_icon.png")).getImage());
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				ExitCommand command = new ExitCommand(getThis());
				command.execute();
			}
		});

		menuBar = new SpaceNetMenuBar(this);
		setJMenuBar(menuBar);
		
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 1;
		c.anchor = GridBagConstraints.CENTER;
		c.fill = GridBagConstraints.BOTH;
		contentPanel = new JPanel();
		contentPanel.setLayout(new BorderLayout());
		contentPanel.setBackground(Color.LIGHT_GRAY);
		add(contentPanel, c);
		c.gridy++;
		c.weighty = 0;
		statusBar = new SpaceNetStatusBar();
		add(statusBar, c);
		
		setMinimumSize(new Dimension(800,600));
	}
	
	/**
	 * Requests the frame to update itself and also requests the menu bar update
	 * itself.
	 */
	public void updateView() {
		if(scenarioPanel==null) setTitle("SpaceNet 2.5");
		else setTitle("SpaceNet 2.5 | " + scenarioPanel.getScenario().getName());
		menuBar.updateView();
	}
	
	/**
	 * Gets the status bar.
	 * 
	 * @return the status bar
	 */
	public SpaceNetStatusBar getStatusBar() {
		return statusBar;
	}
	
	/**
	 * Creates and displays a new scenario.
	 */
	public void newScenario() {
		setScenario(new Scenario());
	}
	
	/**
	 * Opens a saved scenario from a passed file path.
	 * 
	 * @param filePath the file path of the saved scenario
	 */
	public void openScenario(String filePath) {
		try {
			Scenario scenario = XStreamEngine.openScenario(filePath);
			scenario.setFilePath(filePath);
			setScenario(scenario);
		} catch(FileNotFoundException e) {
			JOptionPane.showMessageDialog(this, 
					"The scenario file path (" + filePath + ") is invalid.", 
					"SpaceNet Error",
					JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * Closes the scenario.
	 */
	public void closeScenario() {
    	setScenario(null);
	}
	
	/**
	 * Saves the open scenario.
	 */
	public void saveScenario() {
		try {
			XStreamEngine.saveScenario(scenarioPanel.getScenario());
		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog(this, 
					"The scenario file path (" + scenarioPanel.getScenario().getFilePath() + ") is invalid.", 
					"SpaceNet Error",
					JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * Gets the SpaceNet preferences.
	 * 
	 * @return the preferences
	 */
	public Preferences getPreferences() {
		return preferences;
	}
	private void setPreferences(Preferences preferences) {
		this.preferences = preferences;
	}
	private void setScenario(Scenario scenario) {
		if(scenario==null) {
			scenarioPanel = null;
	    	contentPanel.removeAll();
		} else {
			int lastUid = 0;
			for(I_Element element : scenario.getElements()) {
				if(element.getUid()>lastUid) lastUid = element.getUid();
			}
			for(I_Element container : scenario.getManifest().getContainers()) {
				if(container.getUid()>lastUid) lastUid = container.getUid();
			}
			IdGenerator.setLastUid(lastUid);
			scenarioPanel = new ScenarioPanel(this, scenario);
			if(scenario.getMissionList().size()>0) {
				scenarioPanel.editMissions();
			}
	    	contentPanel.removeAll();
			contentPanel.add(scenarioPanel, BorderLayout.CENTER);
			scenarioPanel.updateView();
		}
		validate();
		repaint();
		updateView();
	}
	private SpaceNetFrame getThis() {
		return this;
	}
	
	/**
	 * Gets the scenario panel component.
	 * 
	 * @return the scenario panel.
	 */
	public ScenarioPanel getScenarioPanel() {
		return scenarioPanel;
	}
}
