/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;

import edu.mit.spacenet.gui.command.CloseScenarioCommand;
import edu.mit.spacenet.gui.command.EditDataSourceCommand;
import edu.mit.spacenet.gui.command.EditOptionsCommand;
import edu.mit.spacenet.gui.command.EditPreferencesCommand;
import edu.mit.spacenet.gui.command.ExitCommand;
import edu.mit.spacenet.gui.command.NewScenarioCommand;
import edu.mit.spacenet.gui.command.OpenAboutCommand;
import edu.mit.spacenet.gui.command.OpenHelpCommand;
import edu.mit.spacenet.gui.command.OpenScenarioCommand;
import edu.mit.spacenet.gui.command.SaveScenarioAsCommand;
import edu.mit.spacenet.gui.command.SaveScenarioCommand;
import edu.mit.spacenet.io.XStreamEngine;

/**
 * The menu bar that contains all of the items in the pull-down menus. Most menu
 * items use commands which abstract the actual triggering of the various
 * features so they can be used elsewhere (e.g. exit on window close OR on
 * selecting the exit command).
 * 
 * @author Paul Grogan
 */
public class SpaceNetMenuBar extends JMenuBar {
	private static final long serialVersionUID = -5251667632821734398L;
	
	private SpaceNetFrame spaceNet;
	private JMenu fileMenu, editMenu, toolsMenu, helpMenu;
	private JMenuItem newScenarioMenuItem, openScenarioMenuItem, 
		closeScenarioMenuItem, saveScenarioMenuItem, saveScenarioAsMenuItem, 
		exitMenuItem, optionsMenuItem, dataSourceMenuItem, preferencesMenuItem, 
		helpMenuItem, aboutMenuItem;
	private JRadioButtonMenuItem networkMenuItem, missionsMenuItem, 
		demandsMenuItem, manifestMenuItem, simulationMenuItem;
	private JCheckBoxMenuItem autoRefreshMenuItem;
	
	/**
	 * The constructor.
	 * 
	 * @param spaceNet the parent SpaceNet frame
	 */
	public SpaceNetMenuBar(SpaceNetFrame spaceNet) {
		this.spaceNet = spaceNet;
		
	    buildFileMenu();
	    add(fileMenu);
	    
	    buildEditMenu();
	    add(editMenu);
	    
	    buildToolsMenu();
	    add(toolsMenu);
	    
	    buildHelpMenu();
	    add(helpMenu);
	    
	    updateView();
	}
	
	/* (non-Javadoc)
	 * @see java.awt.Component#repaint()
	 */
	public void repaint() {
		
	}
	
	/**
	 * Updates the menu to enable/disable various items based on whether a
	 * scenario is open or not, also sets the proper radio button or check box
	 * for certain selections and configurations.
	 */
	public void updateView() {
		boolean isFileOpen = spaceNet.getScenarioPanel()!=null;
		newScenarioMenuItem.setEnabled(!isFileOpen);
		openScenarioMenuItem.setEnabled(!isFileOpen);
		closeScenarioMenuItem.setEnabled(isFileOpen);
		saveScenarioMenuItem.setEnabled(isFileOpen && spaceNet.getScenarioPanel().getScenario().getFilePath() != null);
		saveScenarioAsMenuItem.setEnabled(isFileOpen);
		editMenu.setEnabled(isFileOpen);
		if(isFileOpen) {
			networkMenuItem.setSelected(spaceNet.getScenarioPanel().isEditingNetwork());
			missionsMenuItem.setSelected(spaceNet.getScenarioPanel().isEditingMissions());
			demandsMenuItem.setSelected(spaceNet.getScenarioPanel().isEditingDemands());
			manifestMenuItem.setSelected(spaceNet.getScenarioPanel().isEditingManifest());
			simulationMenuItem.setSelected(spaceNet.getScenarioPanel().isEditingSimulation());
		}
	}
	private void buildFileMenu() {
		fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);
		
        newScenarioMenuItem = new JMenuItem("New Scenario", KeyEvent.VK_N);
        newScenarioMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        newScenarioMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		NewScenarioCommand command = new NewScenarioCommand(spaceNet);
        		command.execute();
        	}
        });
        fileMenu.add(newScenarioMenuItem);
        
        openScenarioMenuItem = new JMenuItem("Open Scenario", KeyEvent.VK_O);
        openScenarioMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        openScenarioMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		OpenScenarioCommand command = new OpenScenarioCommand(spaceNet);
        		command.execute();
        	}
        });
        fileMenu.add(openScenarioMenuItem);
        
        closeScenarioMenuItem = new JMenuItem("Close Scenario", KeyEvent.VK_C);
        closeScenarioMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_W, ActionEvent.CTRL_MASK));
        closeScenarioMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		CloseScenarioCommand command = new CloseScenarioCommand(spaceNet);
        		command.execute();
        	}
        });
        fileMenu.add(closeScenarioMenuItem);
        
        fileMenu.addSeparator();
        
        saveScenarioMenuItem = new JMenuItem("Save Scenario", KeyEvent.VK_S);
        saveScenarioMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        saveScenarioMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		SaveScenarioCommand command = new SaveScenarioCommand(spaceNet);
        		command.execute();
        	}
        });
        fileMenu.add(saveScenarioMenuItem);
        
        saveScenarioAsMenuItem = new JMenuItem("Save Scenario As...", KeyEvent.VK_A);
        saveScenarioAsMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_S, ActionEvent.CTRL_MASK + ActionEvent.SHIFT_MASK));
        saveScenarioAsMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		SaveScenarioAsCommand command = new SaveScenarioAsCommand(spaceNet);
        		command.execute();
        	}
        });
        fileMenu.add(saveScenarioAsMenuItem);
        
        fileMenu.addSeparator();
        
        exitMenuItem = new JMenuItem("Exit");
        exitMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
        exitMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		ExitCommand command = new ExitCommand(spaceNet);
        		command.execute();
        	}
        });
        fileMenu.add(exitMenuItem);
	}
	private void buildEditMenu() {
		editMenu = new JMenu("Edit");
		editMenu.setMnemonic(KeyEvent.VK_E);
		
		optionsMenuItem = new JMenuItem("Options");
		optionsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EditOptionsCommand command = new EditOptionsCommand(spaceNet.getScenarioPanel());
				command.execute();
			}
		});
		editMenu.add(optionsMenuItem);
		
		dataSourceMenuItem = new JMenuItem("Data Source");
		dataSourceMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EditDataSourceCommand command = new EditDataSourceCommand(spaceNet.getScenarioPanel());
				command.execute();
			}
		});
		editMenu.add(dataSourceMenuItem);
		
		editMenu.add(new JSeparator());
		
		networkMenuItem = new JRadioButtonMenuItem("Network");
		networkMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				spaceNet.getScenarioPanel().editNetwork();
			}
		});
		editMenu.add(networkMenuItem);
		
		missionsMenuItem = new JRadioButtonMenuItem("Missions");
		missionsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				spaceNet.getScenarioPanel().editMissions();
			}
		});
		editMenu.add(missionsMenuItem);
        
        demandsMenuItem = new JRadioButtonMenuItem("Demands");
        demandsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				spaceNet.getScenarioPanel().editDemands();
			}
		});
        editMenu.add(demandsMenuItem);
        
        manifestMenuItem = new JRadioButtonMenuItem("Manifest");
        manifestMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				spaceNet.getScenarioPanel().editManifest();
			}
		});
        editMenu.add(manifestMenuItem);
        
        simulationMenuItem = new JRadioButtonMenuItem("Simulation");
        simulationMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				spaceNet.getScenarioPanel().editSimulation();
			}
		});
        editMenu.add(simulationMenuItem);
	}
	private void buildToolsMenu() {
		toolsMenu = new JMenu("Tools");
		toolsMenu.setMnemonic(KeyEvent.VK_T);
		
		autoRefreshMenuItem = new JCheckBoxMenuItem("Auto-Refresh Charts");
		autoRefreshMenuItem.setSelected(spaceNet.getPreferences().isAutoRefresh());
		autoRefreshMenuItem.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				spaceNet.getPreferences().setAutoRefresh(e.getStateChange()==ItemEvent.SELECTED);
				XStreamEngine.writePreferences(spaceNet.getPreferences());
				spaceNet.updateView();
			}
        });
        toolsMenu.add(autoRefreshMenuItem);
        
    	toolsMenu.addSeparator();
        
        preferencesMenuItem = new JMenuItem("Preferences", KeyEvent.VK_P);
        preferencesMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		EditPreferencesCommand command = new EditPreferencesCommand(spaceNet);
        		command.execute();
        	}
        });
        toolsMenu.add(preferencesMenuItem);
	}
	private void buildHelpMenu() {
		helpMenu = new JMenu("Help");
	    helpMenu.setMnemonic(KeyEvent.VK_H);
	    
        helpMenuItem = new JMenuItem("Help", KeyEvent.VK_H);
        helpMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		OpenHelpCommand command = new OpenHelpCommand(spaceNet);
        		command.execute();
        	}
        });
        helpMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
        helpMenu.add(helpMenuItem);
        
        helpMenu.addSeparator();
        
        aboutMenuItem = new JMenuItem("About SpaceNet", KeyEvent.VK_A);
        aboutMenuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		OpenAboutCommand command = new OpenAboutCommand(spaceNet);
        		command.execute();
        	}
        });
        helpMenu.add(aboutMenuItem);
	}
}
