/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.demand;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.geom.Ellipse2D;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;

import org.jdesktop.swingworker.SwingWorker;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.time.Hour;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleInsets;

import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.resource.I_Item;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.CheckBoxTableModel;
import edu.mit.spacenet.gui.component.SNChartPanel;
import edu.mit.spacenet.scenario.Scenario;
import edu.mit.spacenet.simulator.SimScavenge;
import edu.mit.spacenet.util.DateFunctions;

/**
 * Component for commonality analysis.
 * 
 * @author Paul Grogan
 */
public class CommonalityTab extends JSplitPane {
	private static final long serialVersionUID = -6405124067606236705L;
	private static final String CUMULATIVE_MASS = "Total Cumulative Mass";
	private static final String CUMULATIVE_UNITS = "Cumulative Units";
	private static final String MASS_BY_SOURCE = "Mass by Source";
	private DemandsTab demandsTab;
	private ChartPanel scavengedChartPanel;

	private CommonalityTable commonalityTable;

	private JComboBox optionCombo;
	private CheckBoxTableModel<I_Item> partsModel;
	private CheckBoxTableModel<I_Element> elementsModel;
	private JButton refreshButton;
	
	/**
	 * The constructor.
	 * 
	 * @param tab the demands tab component
	 */
	public CommonalityTab(DemandsTab tab) {
		super(JSplitPane.VERTICAL_SPLIT);
		this.demandsTab = tab;
		
		commonalityTable = new CommonalityTable(getScenario());
		JScrollPane commonalityScroll = new JScrollPane(commonalityTable);
		commonalityScroll.setBackground(Color.WHITE);
		commonalityScroll.getViewport().setBackground(Color.WHITE);
		JPanel topPanel = new JPanel();
		topPanel.setBackground(Color.WHITE);
		topPanel.setPreferredSize(new Dimension(400,100));
		topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.LINE_AXIS));
		topPanel.setBorder(BorderFactory.createTitledBorder("Commonality Matrix (% of Mass Common)"));
		topPanel.add(commonalityScroll);
		setTopComponent(topPanel);
		
		JSplitPane bottomPanel = new JSplitPane();
		bottomPanel.setBorder(BorderFactory.createTitledBorder("Scavenging History"));
		scavengedChartPanel = new SNChartPanel();
		scavengedChartPanel.setPreferredSize(new Dimension(600,400));
		bottomPanel.setLeftComponent(scavengedChartPanel);
		
		JPanel controlPanel = new JPanel();
		controlPanel.setBackground(Color.WHITE);
		controlPanel.setLayout(new GridBagLayout());
		controlPanel.setBorder(BorderFactory.createTitledBorder("Chart Options"));
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_END;
		controlPanel.add(new JLabel("Display: "), c);
		c.gridx++;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		optionCombo = new JComboBox();
		optionCombo.addItem(CUMULATIVE_MASS);
		optionCombo.addItem(CUMULATIVE_UNITS);
		optionCombo.addItem(MASS_BY_SOURCE);
		optionCombo.setSelectedItem(CUMULATIVE_MASS);
		optionCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED && isAutoRefresh()) updateChart();
			}
		});
		controlPanel.add(optionCombo, c);
		c.gridy++;
		JPanel prop0 = new JPanel();
		prop0.setOpaque(false);
		prop0.setPreferredSize(new Dimension(1,15));
		controlPanel.add(prop0, c);
		c.gridx = 0;
		c.gridy++;
		c.gridwidth = 2;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		elementsModel = new CheckBoxTableModel<I_Element>();
		elementsModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				if(e.getType()==TableModelEvent.UPDATE && isAutoRefresh()) updateChart();
			}
		});
		JTable elementsTable = new JTable(elementsModel);
		elementsTable.getColumnModel().getColumn(0).setHeaderRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/eye.png")));
                setHorizontalAlignment(JLabel.CENTER);
                return this;
			}
		});
		elementsTable.setBackground(Color.WHITE);
		elementsTable.getTableHeader().setReorderingAllowed(false);
		elementsTable.getTableHeader().setBackground(Color.WHITE);
		elementsTable.getTableHeader().setFont(elementsTable.getTableHeader().getFont().deriveFont(Font.BOLD));
		elementsTable.getTableHeader().setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		elementsTable.getTableHeader().setOpaque(false);
		elementsTable.getColumnModel().getColumn(0).setMaxWidth(25);
		elementsTable.getColumnModel().getColumn(1).setHeaderValue("Filter Source Elements");
		elementsTable.setShowGrid(false);
		JScrollPane elementsScroll = new JScrollPane(elementsTable);
		elementsScroll.setBackground(Color.WHITE);
		elementsScroll.getViewport().setBackground(Color.WHITE);
		elementsScroll.setPreferredSize(new Dimension(150,150));
		controlPanel.add(elementsScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel elementsButtonPanel = new JPanel();
		elementsButtonPanel.setOpaque(false);
		elementsButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllElementsButton = new JButton("Select All");
		selectAllElementsButton.setToolTipText("Select All Elements");
		selectAllElementsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				elementsModel.selectAll();
			}
		});
		elementsButtonPanel.add(selectAllElementsButton);
		JButton deselectAllElementsButton = new JButton("Deselect All");
		deselectAllElementsButton.setToolTipText("Deselect All Elements");
		deselectAllElementsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				elementsModel.deselectAll();
			}
		});
		elementsButtonPanel.add(deselectAllElementsButton);
		controlPanel.add(elementsButtonPanel, c);
		c.weightx = 0;
		c.fill = GridBagConstraints.BOTH;
		c.gridy++;
		JPanel prop2 = new JPanel();
		prop2.setOpaque(false);
		prop2.setPreferredSize(new Dimension(1,15));
		controlPanel.add(prop2, c);
		c.gridy++;
		c.weighty = 1;
		partsModel = new CheckBoxTableModel<I_Item>();
		partsModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				if(e.getType()==TableModelEvent.UPDATE && isAutoRefresh()) updateChart();
			}
		});
		JTable partsTable = new JTable(partsModel);
		partsTable.getColumnModel().getColumn(0).setHeaderRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/eye.png")));
                setHorizontalAlignment(JLabel.CENTER);
                return this;
			}
		});
		partsTable.setBackground(Color.WHITE);
		partsTable.getTableHeader().setReorderingAllowed(false);
		partsTable.getTableHeader().setBackground(Color.WHITE);
		partsTable.getTableHeader().setFont(partsTable.getTableHeader().getFont().deriveFont(Font.BOLD));
		partsTable.getTableHeader().setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		partsTable.getTableHeader().setOpaque(false);
		partsTable.getColumnModel().getColumn(0).setMaxWidth(25);
		partsTable.getColumnModel().getColumn(1).setHeaderValue("Filter Parts");
		partsTable.getColumnModel().getColumn(1).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 3205543512740776042L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if(value instanceof I_Element) {
                	setIcon(((I_Element)value).getIcon());
                	setText(((I_Element)value).getName() + " [" + ((I_Element)value).getUid() + "]");
                }
                return this;
			}
		});
		partsTable.setShowGrid(false);
		JScrollPane partsScroll = new JScrollPane(partsTable);
		partsScroll.setBackground(Color.WHITE);
		partsScroll.getViewport().setBackground(Color.WHITE);
		partsScroll.setPreferredSize(new Dimension(150,200));
		controlPanel.add(partsScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel partsButtonPanel = new JPanel();
		partsButtonPanel.setOpaque(false);
		partsButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllPartsButton = new JButton("Select All");
		selectAllPartsButton.setToolTipText("Select All Parts");
		selectAllPartsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				partsModel.selectAll();
			}
		});
		partsButtonPanel.add(selectAllPartsButton);
		JButton deselectAllPartsButton = new JButton("Deselect All");
		deselectAllPartsButton.setToolTipText("Deselect All Parts");
		deselectAllPartsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				partsModel.deselectAll();
			}
		});
		partsButtonPanel.add(deselectAllPartsButton);
		controlPanel.add(partsButtonPanel, c);
		c.gridy++;
		JPanel prop3 = new JPanel();
		prop3.setOpaque(false);
		prop3.setPreferredSize(new Dimension(1,15));
		controlPanel.add(prop3, c);
		c.gridy++;
		c.anchor = GridBagConstraints.LINE_END;
		refreshButton = new JButton("Refresh", new ImageIcon(getClass().getClassLoader().getResource("icons/arrow_refresh.png")));
		refreshButton.setVisible(false);
		refreshButton.setMargin(new Insets(3,3,3,3));
		refreshButton.setToolTipText("Refresh Chart");
		refreshButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				refreshChart();
			}
		});
		controlPanel.add(refreshButton, c);
		bottomPanel.setRightComponent(controlPanel);
		
		bottomPanel.setOneTouchExpandable(true);
		bottomPanel.setBackground(Color.WHITE);
		bottomPanel.setDividerSize(10);
		bottomPanel.setResizeWeight(1);
		bottomPanel.setDividerLocation(490);
		setBottomComponent(bottomPanel);
		
		setName("Commonality Analysis");
		setDividerSize(0);
		setBorder(BorderFactory.createEmptyBorder());
		setResizeWeight(1);
		setDividerLocation(100);
		refreshChart();
	}
	
	/**
	 * Gets the scenario.
	 * 
	 * @return the sceanrio
	 */
	public Scenario getScenario() {
		return demandsTab.getScenario();
	}
	private boolean isAutoRefresh() {
		return SpaceNetFrame.getSingleton().getPreferences().isAutoRefresh();
	}
	
	/**
	 * Requests that the component and associated charts update based on the
	 * underlying model.
	 */
	public void updateView() {
		commonalityTable.updateView();
		if(isAutoRefresh()) {
			refreshButton.setVisible(false);
			refreshChart();
		} else {
			refreshButton.setVisible(true);
		}
	}
	private boolean isUpdating = false;
	private void refreshChart() {
		if(isUpdating) return;
		isUpdating = true;
		SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
			public Void doInBackground() throws Exception {
				SpaceNetFrame.getSingleton().getStatusBar().setStatusMessage("Refreshing Chart...");
				demandsTab.getScenarioPanel().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				SortedSet<I_Element> elements = new TreeSet<I_Element>();
				Set<I_Item> items = new HashSet<I_Item>();
				for(SimScavenge scavenge : demandsTab.getSimulator().getScavengedParts()) {
					elements.add(scavenge.getElement());
					items.add(scavenge.getItem());
				}
				List<I_Element> deselectedElements = elementsModel.getDeselectedObjects();
				elementsModel.clear();
				for(I_Element element : elements) {
					elementsModel.addObject(element, !deselectedElements.contains(element));
				}
				List<I_Item> deselectedItems = partsModel.getDeselectedObjects();
				partsModel.clear();
				for(I_Item item : items) {
					partsModel.addObject(item, !deselectedItems.contains(item));
				}
				updateChart();
				return null;
			}
			public void done() {
				SpaceNetFrame.getSingleton().getStatusBar().clearStatusMessage();
				demandsTab.getScenarioPanel().setCursor(Cursor.getDefaultCursor());
				isUpdating = false;
			}
		};
		worker.execute();
	}
	private void updateChart() {
		JFreeChart chart;
		
		TimeSeriesCollection dataset = new TimeSeriesCollection();
		
		chart = ChartFactory.createTimeSeriesChart(
				null, 
				null, 
				optionCombo.getSelectedItem().equals(CUMULATIVE_UNITS)?"Units":"Mass (kg)", 
				dataset, 
				!optionCombo.getSelectedItem().equals(CUMULATIVE_MASS), // legend
				true, 
				false);
		chart.setBackgroundPaint(Color.WHITE);
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.WHITE);
		plot.setDomainGridlinePaint(Color.GRAY);
		plot.setRangeGridlinePaint(Color.GRAY);
		plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
		plot.setRangeGridlinesVisible(true);
		plot.setDomainGridlinesVisible(true);
		
		XYItemRenderer r = chart.getXYPlot().getRenderer();
		if(optionCombo.getSelectedItem().equals(CUMULATIVE_UNITS)) {
			for(I_Item item : partsModel.getSelectedObjects()) {
				TimeSeries s = new TimeSeries(item.getName() + " [" + item.getTid() + "]", Hour.class);
				double amount = 0;
				for(SimScavenge scavenge : demandsTab.getSimulator().getScavengedParts()) {
					if(elementsModel.getSelectedObjects().contains(scavenge.getElement())
							&& scavenge.getItem().equals(item)) {
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), scavenge.getTime())).previous(), amount);
						amount += scavenge.getAmount();
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), scavenge.getTime())), amount);
					}
				}
				dataset.addSeries(s);
				r.setSeriesShape(dataset.getSeriesCount() - 1, new Ellipse2D.Double(0,0,0,0));
			}
		} else if(optionCombo.getSelectedItem().equals(CUMULATIVE_MASS)) {
			TimeSeries s = new TimeSeries("Total", Hour.class);
			double amount = 0;
			for(SimScavenge scavenge : demandsTab.getSimulator().getScavengedParts()) {
				if(elementsModel.getSelectedObjects().contains(scavenge.getElement())
						&& partsModel.getSelectedObjects().contains(scavenge.getItem())) {
					s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), scavenge.getTime())).previous(), amount);
					amount += scavenge.getItem().getUnitMass()*scavenge.getAmount();
					s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), scavenge.getTime())), amount);
				}
			}
			dataset.addSeries(s);
			r.setSeriesShape(dataset.getSeriesCount() - 1, new Ellipse2D.Double(0,0,0,0));
		
		} else {
			for(I_Element element : elementsModel.getSelectedObjects()) {
				TimeSeries s = new TimeSeries(element.getName() + " [" + element.getUid() + "]", Hour.class);
				double amount = 0;
				for(SimScavenge scavenge : demandsTab.getSimulator().getScavengedParts()) {
					if(scavenge.getElement().equals(element)
							&& partsModel.getSelectedObjects().contains(scavenge.getItem())) {
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), scavenge.getTime())).previous(), amount);
						amount += scavenge.getItem().getUnitMass()*scavenge.getAmount();
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), scavenge.getTime())), amount);
					}
				}
				dataset.addSeries(s);
				r.setSeriesShape(dataset.getSeriesCount() - 1, new Ellipse2D.Double(0,0,0,0));
			}
		}
		scavengedChartPanel.setChart(chart);
	}
}
