/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.demand;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jdesktop.swingworker.SwingWorker;

import edu.mit.spacenet.gui.ScenarioPanel;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.scenario.ItemDiscretization;
import edu.mit.spacenet.scenario.Scenario;
import edu.mit.spacenet.simulator.DemandSimulator;

/**
 * Component for viewing and editing the demands associated with a sceanrio.
 * 
 * @author Paul Grogan
 */
public class DemandsTab extends JSplitPane {
	private static final long serialVersionUID = 1065899429343403823L;

	private JPanel controlPanel;
	private JComboBox discretizationCombo;
	private JSlider aggregationSlider;
	
	private JCheckBox scavengeSparesCheck;
	
	private JTabbedPane tabs;

	private ScenarioFeasibilityChart feasibilityChart;
	private SupplyNetworkChart networkChart;
	private ElementDemandsChart elementDemandChart;
	private MissionDemandsChart missionDemandChart;
	private LocationDemandsChart locationDemandChart;
	private CommonalityTab commonalityTab;
	private RepairabilityTab repairabilityTab;
	
	private ScenarioPanel scenarioPanel;
	private DemandSimulator simulator;
	
	/**
	 * The constructor.
	 * 
	 * @param scenarioPanel the scenario panel
	 */
	public DemandsTab(ScenarioPanel scenarioPanel) {
		this.scenarioPanel = scenarioPanel;
		simulator = new DemandSimulator(getScenario());
		
		controlPanel = new JPanel();
		controlPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		controlPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.CENTER;
		c.weightx = 1;
		controlPanel.add(getOptionsPanel(), c);
		c.gridy++;
		controlPanel.add(getDisplayPanel(), c);
		c.gridy++;
		c.weighty = 1;
		controlPanel.add(new JPanel(), c);
		
		setLeftComponent(new JScrollPane(controlPanel));
		
		tabs = new JTabbedPane();
		feasibilityChart = new ScenarioFeasibilityChart(this);
		tabs.add(feasibilityChart);
		networkChart = new SupplyNetworkChart(this);
		tabs.add(networkChart);
		elementDemandChart = new ElementDemandsChart(this);
		tabs.add(elementDemandChart);
		missionDemandChart = new MissionDemandsChart(this);
		tabs.add(missionDemandChart);
		locationDemandChart = new LocationDemandsChart(this);
		tabs.add(locationDemandChart);
		commonalityTab = new CommonalityTab(this);
		tabs.add(commonalityTab);
		repairabilityTab = new RepairabilityTab(this);
		tabs.add(repairabilityTab);
		tabs.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				updateChart();
			}
		});
		
		setRightComponent(tabs);
		
		setResizeWeight(.1);
		setDividerLocation(250);
		setBorder(BorderFactory.createEmptyBorder());
	}
	private JPanel getOptionsPanel() {
		JPanel optionsPanel = new JPanel();
		optionsPanel.setBorder(BorderFactory.createTitledBorder("Demand Options"));
		optionsPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		c.gridx = 0;
		c.gridy = 0;
		c.weighty = 0;
		c.weightx = 0;
		optionsPanel.add(new JLabel("Item Discretization by: "), c);
		c.gridy++;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		optionsPanel.add(new JLabel("Item Aggregation at: "), c);
		c.gridy = 0;
		c.gridx++;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		discretizationCombo = new JComboBox();
		discretizationCombo.setToolTipText("Discretize demands for items to integer values at the selected level");
		for(ItemDiscretization t : ItemDiscretization.values()) {
			discretizationCombo.addItem(t);
		}
		discretizationCombo.setSelectedItem(getScenario().getItemDiscretization());
		discretizationCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED) {
					getScenario().setItemDiscretization((ItemDiscretization)discretizationCombo.getSelectedItem());
					aggregationSlider.setEnabled(getScenario().getItemDiscretization()!=ItemDiscretization.NONE);
					updateView();
				}
			}
		});
		optionsPanel.add(discretizationCombo, c);
		c.gridy++;
		aggregationSlider = new JSlider(JSlider.VERTICAL, 0,4,1);
		aggregationSlider.setToolTipText("Aggregate discretized item demands ahead or behind demands.");
		Hashtable<Integer, JLabel> labelTable  = new Hashtable<Integer, JLabel>();
		labelTable.put(new Integer(0), new JLabel("First Demand", JLabel.LEFT));
		labelTable.put(new Integer(2), new JLabel("Half Demand", JLabel.LEFT));
		labelTable.put(new Integer(4), new JLabel("Unit Demand", JLabel.RIGHT));
		aggregationSlider.setLabelTable(labelTable);
		aggregationSlider.setPaintLabels(true);
		aggregationSlider.setMajorTickSpacing(1);
		aggregationSlider.setSnapToTicks(true);
		aggregationSlider.setPaintTicks(true);
		aggregationSlider.setValue((int)(getScenario().getItemAggregation()*4));
		aggregationSlider.setEnabled(getScenario().getItemDiscretization()!=ItemDiscretization.NONE);
		aggregationSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if(!aggregationSlider.getValueIsAdjusting()) {
					getScenario().setItemAggregation(aggregationSlider.getValue()/4d);
					updateView();
				}
			}
		});
		aggregationSlider.setPreferredSize(new Dimension(50,100));
		optionsPanel.add(aggregationSlider, c);
		c.gridy++;
		c.gridx = 0;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		scavengeSparesCheck = new JCheckBox("Scavenge Spares");
		scavengeSparesCheck.setToolTipText("Scavenge spare parts from co-located decommissioned elements");
		scavengeSparesCheck.setSelected(getScenario().isScavengeSpares());
		scavengeSparesCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				getScenario().setScavengeSpares(scavengeSparesCheck.isSelected());
				updateView();
			}
		});
		optionsPanel.add(scavengeSparesCheck, c);
		return optionsPanel;
	}
	private JPanel getDisplayPanel() {
		JPanel displayPanel = new JPanel();
		displayPanel.setBorder(BorderFactory.createTitledBorder("Display Options"));
		JCheckBox packingDemandsCheck = new JCheckBox("Estimate Logistics Container Masses");
		packingDemandsCheck.setToolTipText("Estimate mass (Generic COS 5) for containers using resource packing factors.");
		packingDemandsCheck.setSelected(simulator.isPackingDemandsAdded());
		packingDemandsCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				simulator.setPackingDemandsAdded(e.getStateChange()==ItemEvent.SELECTED);
				updateView();
			}
		});
		displayPanel.add(packingDemandsCheck);
		return displayPanel;
	}
	
	/**
	 * Gets the scenario panel component.
	 * 
	 * @return the scenario panel
	 */
	public ScenarioPanel getScenarioPanel() {
		return scenarioPanel;
	}
	
	/**
	 * Gets the scenario.
	 * 
	 * @return the scenario
	 */
	public Scenario getScenario() {
		return scenarioPanel.getScenario();
	}
	
	/**
	 * Gets the demand simulator.
	 * 
	 * @return the demand simulator
	 */
	public DemandSimulator getSimulator() {
		return simulator;
	}
	private boolean isUpdating = false;
	
	/**
	 * Requests that the component and any nested components update themselves.
	 */
	public void updateView() {
		discretizationCombo.setSelectedItem(getScenario().getItemDiscretization());
		aggregationSlider.setValue((int)(getScenario().getItemAggregation()*4));
		scavengeSparesCheck.setSelected(getScenario().isScavengeSpares());
		
		if(isUpdating) return;
		isUpdating = true;
		SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
			public Void doInBackground() throws Exception {
				SpaceNetFrame.getSingleton().getStatusBar().setStatusMessage("Simulating...");
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				simulate();
				return null;
			}
			public void done() {
				updateChart();
				SpaceNetFrame.getSingleton().getStatusBar().clearStatusMessage();
				setCursor(Cursor.getDefaultCursor());
				isUpdating = false;
			}
		};
		worker.execute();
	}
	private void updateChart() {
		if(tabs.getSelectedComponent().equals(feasibilityChart)) {
			feasibilityChart.updateView();
		} else if(tabs.getSelectedComponent().equals(networkChart)) {
			networkChart.updateView();
		} else if(tabs.getSelectedComponent().equals(elementDemandChart)) { 
			elementDemandChart.updateView();
		} else if(tabs.getSelectedComponent().equals(missionDemandChart)) { 
			missionDemandChart.updateView();
		} else if(tabs.getSelectedComponent().equals(locationDemandChart)) { 
			locationDemandChart.updateView();
		} else if(tabs.getSelectedComponent().equals(repairabilityTab)) {
			repairabilityTab.updateView();
		} else if(tabs.getSelectedComponent().equals(commonalityTab)) {
			commonalityTab.updateView();
		}
	}
	private void simulate() {
		if(tabs.getSelectedComponent().equals(repairabilityTab)) {
			simulator.setItemsRepaired(false);
		} else {
			simulator.setItemsRepaired(true);
		}
		simulator.simulate();
	}
}
