/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.demand;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;
import java.util.Date;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;

import org.jdesktop.swingworker.SwingWorker;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.time.Hour;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleInsets;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.CheckBoxTableModel;
import edu.mit.spacenet.gui.component.SNChartPanel;
import edu.mit.spacenet.simulator.SimDemand;
import edu.mit.spacenet.util.DateFunctions;

/**
 * Component for displaying the element-level demands.
 * 
 * @author Paul Grogan
 */
public class ElementDemandsChart extends JSplitPane {
	private static final long serialVersionUID = -6405124067606236705L;
	private DemandsTab demandsTab;
	private ChartPanel demandChartPanel;
	private CheckBoxTableModel<I_Element> elementsModel;
	private CheckBoxTableModel<ClassOfSupply> cosModel;
	private JButton refreshButton;
	
	/**
	 * Instantiates a new element demands chart.
	 * 
	 * @param tab the demands tab
	 */
	public ElementDemandsChart(DemandsTab tab) {
		this.demandsTab = tab;
		
		demandChartPanel = new SNChartPanel();
		demandChartPanel.setPreferredSize(new Dimension(600,400));
		setLeftComponent(demandChartPanel);
		
		JPanel controlPanel = new JPanel();
		controlPanel.setBackground(Color.WHITE);
		controlPanel.setLayout(new GridBagLayout());
		controlPanel.setBorder(BorderFactory.createTitledBorder("Chart Options"));
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		elementsModel = new CheckBoxTableModel<I_Element>();
		elementsModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				if(e.getType()==TableModelEvent.UPDATE && isAutoRefresh()) updateChart();
			}
		});
		JTable elementsTable = new JTable(elementsModel);
		elementsTable.getColumnModel().getColumn(0).setHeaderRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/eye.png")));
                setHorizontalAlignment(JLabel.CENTER);
                return this;
			}
		});
		elementsTable.setBackground(Color.WHITE);
		elementsTable.getTableHeader().setReorderingAllowed(false);
		elementsTable.getTableHeader().setBackground(Color.WHITE);
		elementsTable.getTableHeader().setFont(elementsTable.getTableHeader().getFont().deriveFont(Font.BOLD));
		elementsTable.getTableHeader().setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		elementsTable.getTableHeader().setOpaque(false);
		elementsTable.getColumnModel().getColumn(0).setMaxWidth(25);
		elementsTable.getColumnModel().getColumn(1).setHeaderValue("Filter Elements");
		elementsTable.getColumnModel().getColumn(1).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 3205543512740776042L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if(value instanceof I_Element) {
                	setIcon(((I_Element)value).getIcon());
                	setText(((I_Element)value).getName() + " [" + ((I_Element)value).getUid() + "]");
                }
                return this;
			}
		});
		elementsTable.setShowGrid(false);
		JScrollPane elementsScroll = new JScrollPane(elementsTable);
		elementsScroll.setBackground(Color.WHITE);
		elementsScroll.getViewport().setBackground(Color.WHITE);
		elementsScroll.setPreferredSize(new Dimension(150,200));
		controlPanel.add(elementsScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel elementsButtonPanel = new JPanel();
		elementsButtonPanel.setOpaque(false);
		elementsButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllElementsButton = new JButton("Select All");
		selectAllElementsButton.setToolTipText("Select All Elements");
		selectAllElementsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				elementsModel.selectAll();
			}
		});
		elementsButtonPanel.add(selectAllElementsButton);
		JButton deselectAllElementsButton = new JButton("Deselect All");
		deselectAllElementsButton.setToolTipText("Deselect All Elements");
		deselectAllElementsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				elementsModel.deselectAll();
			}
		});
		elementsButtonPanel.add(deselectAllElementsButton);
		controlPanel.add(elementsButtonPanel, c);
		c.weightx = 0;
		c.fill = GridBagConstraints.BOTH;
		c.gridy++;
		JPanel prop2 = new JPanel();
		prop2.setOpaque(false);
		prop2.setPreferredSize(new Dimension(1,15));
		controlPanel.add(prop2, c);
		c.gridy++;
		c.weighty = 1;
		cosModel = new CheckBoxTableModel<ClassOfSupply>();
		for(int i=1; i<=10; i++) {
			cosModel.addObject(ClassOfSupply.getInstance(i));
		}
		
		cosModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				if(e.getType()==TableModelEvent.UPDATE && isAutoRefresh()) updateChart();
			}
		});
		JTable cosTable = new JTable(cosModel);
		cosTable.getColumnModel().getColumn(0).setHeaderRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/eye.png")));
                setHorizontalAlignment(JLabel.CENTER);
                return this;
			}
		});
		cosTable.setBackground(Color.WHITE);
		cosTable.getTableHeader().setReorderingAllowed(false);
		cosTable.getTableHeader().setBackground(Color.WHITE);
		cosTable.getTableHeader().setFont(cosTable.getTableHeader().getFont().deriveFont(Font.BOLD));
		cosTable.getTableHeader().setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		cosTable.getTableHeader().setOpaque(false);
		cosTable.getColumnModel().getColumn(0).setMaxWidth(25);
		cosTable.getColumnModel().getColumn(1).setHeaderValue("Filter Classes of Supply");
		cosTable.setShowGrid(false);
		JScrollPane cosScroll = new JScrollPane(cosTable);
		cosScroll.setBackground(Color.WHITE);
		cosScroll.getViewport().setBackground(Color.WHITE);
		cosScroll.setPreferredSize(new Dimension(150,150));
		controlPanel.add(cosScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel dataButtonPanel = new JPanel();
		dataButtonPanel.setOpaque(false);
		dataButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllDataButton = new JButton("Select All");
		selectAllDataButton.setToolTipText("Select All Data");
		selectAllDataButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cosModel.selectAll();
			}
		});
		dataButtonPanel.add(selectAllDataButton);
		JButton deselectAllDataButton = new JButton("Deselect All");
		deselectAllDataButton.setToolTipText("Deselect All Data");
		deselectAllDataButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cosModel.deselectAll();
			}
		});
		dataButtonPanel.add(deselectAllDataButton);
		controlPanel.add(dataButtonPanel, c);
		c.gridy++;
		JPanel prop3 = new JPanel();
		prop3.setOpaque(false);
		prop3.setPreferredSize(new Dimension(1,15));
		controlPanel.add(prop3, c);
		c.gridy++;
		c.anchor = GridBagConstraints.LINE_END;
		refreshButton = new JButton("Refresh", new ImageIcon(getClass().getClassLoader().getResource("icons/arrow_refresh.png")));
		refreshButton.setVisible(false);
		refreshButton.setMargin(new Insets(3,3,3,3));
		refreshButton.setToolTipText("Refresh Chart");
		refreshButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				refreshChart();
			}
		});
		controlPanel.add(refreshButton, c);
		
		setRightComponent(controlPanel);
		
		setName("Demands by Element");
		setOneTouchExpandable(true);
		setDividerSize(10);
		setBorder(BorderFactory.createEmptyBorder());
		setResizeWeight(1);
		//setLastDividerLocation(490);
		setDividerLocation(490);
	}
	private boolean isAutoRefresh() {
		return SpaceNetFrame.getSingleton().getPreferences().isAutoRefresh();
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		if(isAutoRefresh()) {
			refreshButton.setVisible(false);
			refreshChart();
		} else {
			refreshButton.setVisible(true);
		}
	}
	private boolean isUpdating = false;
	private void refreshChart() {
		if(isUpdating) return;
		isUpdating = true;
		SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
			public Void doInBackground() throws Exception {
				SpaceNetFrame.getSingleton().getStatusBar().setStatusMessage("Refreshing Chart...");
				demandsTab.getScenarioPanel().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				List<I_Element> selectedElements = elementsModel.getSelectedObjects();
				elementsModel.clear();
				for(I_Element element : demandsTab.getScenario().getElements()) {
					elementsModel.addObject(element, selectedElements.contains(element));
				}
				updateChart();
				return null;
			}
			public void done() {
				SpaceNetFrame.getSingleton().getStatusBar().clearStatusMessage();
				demandsTab.getScenarioPanel().setCursor(Cursor.getDefaultCursor());
				isUpdating = false;
			}
		};
		worker.execute();
	}
	private void updateChart() {
		if(!getLeftComponent().equals(demandChartPanel)) {
			setLeftComponent(demandChartPanel);
		}
		demandsTab.getScenarioPanel().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		JFreeChart chart;
		
		if(elementsModel.getSelectedObjects().size()==1) {
			I_Element element = elementsModel.getSelectedObjects().get(0);
			
			TimeSeriesCollection dataset = new TimeSeriesCollection();
			
			TimeSeries[] s = new TimeSeries[11];
			s[0] = new TimeSeries("Total", Hour.class);
			for(int i = 1; i <= 10; i++) {
				s[i] = new TimeSeries("COS " + i, Hour.class);
			}
			
			double amount[] = new double[11];
			for(SimDemand demand : demandsTab.getSimulator().getDemandHistory(element)) {
				Date date = DateFunctions.getDate(demandsTab.getScenario().getStartDate(), demand.getTime());
				for(Demand d : demand.getDemands()) {
					amount[0] += d.getMass();
					int cos = d.getResource().getClassOfSupply().getBaseClass().getId();
					amount[cos] += d.getMass();
				}
				for(int i = 0; i <=10; i++) {
					s[i].addOrUpdate(new Hour(date), amount[i]);
				}
			}
			
			chart = ChartFactory.createTimeSeriesChart(
					null, 
					null, 
					"Mass (kg)", 
					dataset, 
					true, // legend
					true, 
					false);
			chart.setBackgroundPaint(Color.WHITE);
			XYPlot plot = (XYPlot) chart.getPlot();
			plot.setBackgroundPaint(Color.WHITE);
			plot.setDomainGridlinePaint(Color.GRAY);
			plot.setRangeGridlinePaint(Color.GRAY);
			plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
			plot.setRangeGridlinesVisible(true);
			plot.setDomainGridlinesVisible(true);
			
			XYItemRenderer r = chart.getXYPlot().getRenderer();
			for(int i = 0; i <=10; i++) {
				if(i==0) {
					dataset.addSeries(s[i]);
					r.setSeriesShape(dataset.getSeriesCount() - 1, new Ellipse2D.Double(0, 0, 0, 0));
					r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.BLACK);
				} else if(cosModel.getSelectedObjects().contains(ClassOfSupply.getInstance(i))) {
					dataset.addSeries(s[i]);
					r.setSeriesShape(dataset.getSeriesCount() - 1, new Ellipse2D.Double(0, 0, 0, 0));
					r.setSeriesPaint(dataset.getSeriesCount() - 1, ClassOfSupply.getInstance(i).getColor());
				}
			}
			
			/* old code for pie chart
			DefaultPieDataset dataset = new DefaultPieDataset();
			for(int i=1; i<=10; i++) {
				if(dataModel.getSelectedObjects().contains("COS " + i + ": " + ClassOfSupply.getInstance(i).getName())
						&& tab.getMiniSim().getDemands(element, ClassOfSupply.getInstance(i)).getTotalMass()>0) {
					dataset.setValue("COS " + i, tab.getMiniSim().getDemands(element, ClassOfSupply.getInstance(i)).getTotalMass());
				}
			}
			chart = ChartFactory.createPieChart(
		            null,
		            dataset,
		            true,
		            true,
		            false
		        );
			chart.setBackgroundPaint(Color.WHITE);
			PiePlot plot = (PiePlot) chart.getPlot();
			plot.setBackgroundPaint(Color.WHITE);
			plot.setLabelGenerator(null);
			for(int i=1; i<=10; i++) {
				//plot.setSectionPaint("COS " + i, ClassOfSupply.getInstance(i).getColor());
			}
			*/
			
		} else {
			DefaultCategoryDataset dataset = new DefaultCategoryDataset();
			
			chart = ChartFactory.createStackedBarChart(
					null, 
					"", 
					"Mass (kg)", 
					dataset, 
					PlotOrientation.VERTICAL, // the plot orientation
					true, // legend
					true, 
					false);

			chart.setBackgroundPaint(Color.WHITE);
			CategoryPlot plot = (CategoryPlot) chart.getPlot();
			plot.getDomainAxis().setCategoryLabelPositions(CategoryLabelPositions.UP_90);
			plot.setBackgroundPaint(Color.WHITE);
			plot.setDomainGridlinePaint(Color.GRAY);
			plot.setRangeGridlinePaint(Color.GRAY);
			plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
			plot.setRangeGridlinesVisible(true);
			plot.setDomainGridlinesVisible(true);
			
			CategoryItemRenderer r = chart.getCategoryPlot().getRenderer();
			
			for(I_Element element : elementsModel.getSelectedObjects()) {
				for(int i=1; i<=10; i++) {
					if(cosModel.getSelectedObjects().contains(ClassOfSupply.getInstance(i))) {
						dataset.addValue(demandsTab.getSimulator().getDemands(element, ClassOfSupply.getInstance(i)).getTotalMass(),
								"COS " + i, element.getName() + " [" + element.getUid() + "]");
						r.setSeriesPaint(dataset.getRowCount() - 1, ClassOfSupply.getInstance(i).getColor());
					}
				}
			}
		}
		
		demandChartPanel.setChart(chart);
		demandsTab.getScenarioPanel().setCursor(Cursor.getDefaultCursor());
	}
}
