/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.demand;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashSet;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.RectangleInsets;

import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.gui.component.SNChartPanel;
import edu.mit.spacenet.gui.component.UnitsWrapper;
import edu.mit.spacenet.scenario.ItemDiscretization;
import edu.mit.spacenet.scenario.Mission;
import edu.mit.spacenet.scenario.RepairItem;
import edu.mit.spacenet.scenario.Scenario;

/**
 * Component for performing repairability analysis.
 * 
 * @author Paul Grogan
 */
public class RepairabilityTab extends JSplitPane {
	private static final long serialVersionUID = -6405124067606236705L;
	private DemandsTab tab;
	
	private ListSelectionListener summaryListener;
	private RepairSummaryTable summaryTable;
	private SpinnerNumberModel autoRepairModel;
	private JSpinner autoRepairSpinner;
	private RepairableDemandsTable repairTable;
	
	private ChartPanel repairabilityChartPanel;
	
	/**
	 * Instantiates a new repairability tab.
	 * 
	 * @param tab the demands tab
	 */
	public RepairabilityTab(DemandsTab tab) {
		super(JSplitPane.VERTICAL_SPLIT);
		this.tab = tab;
		
		JSplitPane topPanel = new JSplitPane();
		summaryTable = new RepairSummaryTable(getScenario());
		summaryListener = new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				ChangeListener autoListener = autoRepairSpinner.getChangeListeners()[0];
				autoRepairSpinner.removeChangeListener(autoListener);
				autoRepairModel.setValue(summaryTable.getValueAt(summaryTable.getSelectedRow(), 2));
				autoRepairSpinner.addChangeListener(autoListener);
				
				updateView();
			}
		};
		summaryTable.getSelectionModel().addListSelectionListener(summaryListener);
		JScrollPane summaryScroll = new JScrollPane(summaryTable);
		summaryScroll.setBackground(Color.WHITE);
		summaryScroll.getViewport().setBackground(Color.WHITE);
		topPanel.setLeftComponent(summaryScroll);
		
		repairabilityChartPanel = new SNChartPanel();
		repairabilityChartPanel.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY));
		repairabilityChartPanel.setPreferredSize(new Dimension(600,400));
		topPanel.setRightComponent(repairabilityChartPanel);
		
		topPanel.setBackground(Color.WHITE);
		topPanel.setBorder(BorderFactory.createEmptyBorder());
		topPanel.setDividerLocation(300);
		topPanel.setResizeWeight(0.5);
		setTopComponent(topPanel);
		
		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new GridBagLayout());
		bottomPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx=0;
		c.gridy=0;
		c.weightx = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		bottomPanel.add(new JLabel("Auto-Repair: "), c);
		c.gridx++;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		autoRepairModel = new SpinnerNumberModel(0,0,10000,0.25);
		autoRepairSpinner = new JSpinner(autoRepairModel);
		autoRepairSpinner.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				autoRepair();
			}
		});
		bottomPanel.add(new UnitsWrapper(autoRepairSpinner, "hours"), c);
		c.gridx--;
		c.gridy++;
		c.gridwidth = 2;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.LINE_START;
		repairTable = new RepairableDemandsTable();
		repairTable.getModel().addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				summaryTable.getSelectionModel().removeListSelectionListener(summaryListener);
				summaryTable.updateView();
				summaryTable.getSelectionModel().addListSelectionListener(summaryListener);
				
				updateChart();
			}
		});
		JScrollPane repairScroll = new JScrollPane(repairTable);
		repairScroll.setBackground(Color.WHITE);
		repairScroll.getViewport().setBackground(Color.WHITE);
		bottomPanel.add(repairScroll, c);
		setBottomComponent(bottomPanel);
		
		setBackground(Color.WHITE);
		setBorder(BorderFactory.createEmptyBorder());
		setDividerLocation(300);
		setResizeWeight(.25);
		setName("Repairability Analysis");
		updateView();
	}
	
	/**
	 * Gets the scenario.
	 * 
	 * @return the scenario
	 */
	public Scenario getScenario() {
		return tab.getScenario();
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		Mission mission = summaryTable.getSelectedMission();
		if(mission !=null) {
			summaryTable.getSelectionModel().removeListSelectionListener(summaryListener);
			summaryTable.updateView();
			summaryTable.getSelectionModel().addListSelectionListener(summaryListener);
			
			repairTable.forceUpdate(getScenario().getMissionList().indexOf(mission));
		}
	}
	private void autoRepair() {
		Mission mission = summaryTable.getSelectedMission();
		int index = getScenario().getMissionList().indexOf(mission);
		if(getScenario().getRepairedItems().get(mission)==null) {
			getScenario().getRepairedItems().put(mission, new HashSet<RepairItem>());
		} else {
			getScenario().getRepairedItems().get(mission).clear();
		}
		
		double repairTime = autoRepairModel.getNumber().doubleValue();
		ArrayList<RepairItem> repairItems = new ArrayList<RepairItem>();
		if(tab.getSimulator().getSortedRepairItems().get(index)!=null) {
			for(RepairItem i : tab.getSimulator().getSortedRepairItems().get(index)) {
				repairItems.add(i);
			}
		}
		int i = 0;
		while(repairTime > 0
				&& repairItems.size() > 0
				&& i < repairItems.size()) {
			// while there is still time left to repair and there are items that could
			// be repaired
			if(repairTime >= repairItems.get(i).getMeanRepairTime()) {
				// enough time to repair entire demand
				repairTime -= repairItems.get(i).getMeanRepairTime();
				boolean isFound = false;
				for(RepairItem item : getScenario().getRepairedItems().get(getScenario().getMissionList().get(index))) {
					if(item.getDemand().getResource().equals(repairItems.get(i).getDemand().getResource())
							&& item.getElement().equals(repairItems.get(i).getElement())) {
						isFound = true;
						item.getDemand().setAmount(item.getDemand().getAmount() + repairItems.get(i).getDemand().getAmount());
					}
				}
				if(!isFound) {
					getScenario().getRepairedItems().get(mission).add(new RepairItem(
							new Demand(repairItems.get(i).getDemand().getResource(), 
									repairItems.get(i).getDemand().getAmount()), 
									repairItems.get(i).getElement()));
				}
				repairItems.remove(i);
			} else if(getScenario().getItemDiscretization().equals(ItemDiscretization.NONE)) {
				// can only repair part of demand
				double repairFraction = repairTime/repairItems.get(i).getMeanRepairTime();
				double amountRepaired = repairItems.get(i).getDemand().getAmount()*repairFraction;
				repairTime = 0;
				boolean isFound = false;
				for(RepairItem item : getScenario().getRepairedItems().get(getScenario().getMissionList().get(index))) {
					if(item.getDemand().getResource().equals(repairItems.get(i).getDemand().getResource())
							&& item.getElement().equals(repairItems.get(i).getElement())) {
						isFound = true;
						item.getDemand().setAmount(repairItems.get(i).getDemand().getAmount()-amountRepaired);
					}
				}
				if(!isFound) {
					getScenario().getRepairedItems().get(mission).add(new RepairItem(
							new Demand(repairItems.get(i).getDemand().getResource(), 
									amountRepaired), 
									repairItems.get(i).getElement()));
				}
				break;
			} else {
				// else look at next item
				i++;
			}
		}
		updateView();
	}
	private void updateChart() {
		Mission mission = summaryTable.getSelectedMission();
		int index = getScenario().getMissionList().indexOf(mission);

		XYSeriesCollection dataset = new XYSeriesCollection();
		JFreeChart chart = ChartFactory.createXYLineChart(
				null, 
				"Repair Time (hr)", 
				"Spares Mass Savings (kg)", 
				dataset, 
				PlotOrientation.VERTICAL,
				true, 
				true, 
				false);
		
		chart.setBackgroundPaint(Color.WHITE);
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.WHITE);
		plot.setDomainGridlinePaint(Color.GRAY);
		plot.setRangeGridlinePaint(Color.GRAY);
		plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
		plot.setRangeGridlinesVisible(true);
		plot.setDomainGridlinesVisible(true);

		XYItemRenderer r = chart.getXYPlot().getRenderer();

		if (r instanceof XYLineAndShapeRenderer) {
			XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
			renderer.setBaseShapesVisible(true);
			renderer.setBaseShapesFilled(true);
		}

		XYSeries s = new XYSeries("Unsorted");
		double mass = 0;
		double time = 0;
		s.addOrUpdate(mass, time);
		if(tab.getSimulator().getUnsortedRepairItems().get(index)!=null) {
			for(RepairItem i : tab.getSimulator().getUnsortedRepairItems().get(index)) {
				mass += i.getDemand().getMass()-i.getMassToRepair();
				time += i.getMeanRepairTime();
				s.addOrUpdate(time, mass);
			}
		}
		dataset.addSeries(s);
		r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.RED);
		r.setSeriesShape(dataset.getSeriesCount() - 1, 
				new Ellipse2D.Double(0, 0, 0, 0));

		mass = 0;
		time = 0;
		XYSeries s2 = new XYSeries("Optimal Order");
		s2.addOrUpdate(0, 0);
		if(tab.getSimulator().getSortedRepairItems().get(index)!=null) {
			for(RepairItem i :tab.getSimulator().getSortedRepairItems().get(index)) {
				mass += i.getDemand().getMass()-i.getMassToRepair();
				time += i.getMeanRepairTime();
				s2.addOrUpdate(time, mass);
			}
		}
		dataset.addSeries(s2);
		r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.BLUE);
		r.setSeriesShape(dataset.getSeriesCount() - 1, 
				new Ellipse2D.Double(0, 0, 0, 0));
		
		XYSeries s3 = new XYSeries("Repair Selection");
		mass = 0;
		time = 0;
		if(getScenario().getRepairedItems().get(mission) != null) {
			for(RepairItem i : getScenario().getRepairedItems().get(mission)) {
				mass += (i.getDemand().getMass()-i.getMassToRepair());
				time += i.getMeanRepairTime();
			}
		}
		s3.addOrUpdate(time, mass);
		dataset.addSeries(s3);
		r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.BLACK);
		r.setSeriesShape(dataset.getSeriesCount() - 1, 
				new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
		
		repairabilityChartPanel.setChart(chart);
		invalidate();
	}
	private class RepairSummaryTable extends JTable {
		private static final long serialVersionUID = -5981730193560436124L;
		public RepairSummaryTable(Scenario scenario) {
			setModel(new RepairSummaryModel(scenario));
			getColumnModel().getColumn(0).setHeaderValue("");
			getColumnModel().getColumn(0).setMaxWidth(20);
			getColumnModel().getColumn(1).setHeaderValue("Mission");
			getColumnModel().getColumn(2).setHeaderValue("Repair Time");
			getColumnModel().getColumn(2).setMaxWidth(75);
			getColumnModel().getColumn(2).setCellRenderer(new DefaultTableCellRenderer() {
				private static final long serialVersionUID = 1L;
				private DecimalFormat format = new DecimalFormat("0.00");
				@Override
				public Component getTableCellRendererComponent(JTable table, Object value,
						boolean isSelected, boolean hasFocus, int row, int column) {
					super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					setHorizontalAlignment(JLabel.RIGHT);
					setText(format.format(value) + " hr");
					return this;
				}
			});
			getColumnModel().getColumn(3).setHeaderValue("Mass");
			getColumnModel().getColumn(3).setMaxWidth(75);
			getColumnModel().getColumn(3).setCellRenderer(new DefaultTableCellRenderer() {
				private static final long serialVersionUID = 1L;
				private DecimalFormat format = new DecimalFormat("0.00");
				@Override
				public Component getTableCellRendererComponent(JTable table, Object value,
						boolean isSelected, boolean hasFocus, int row, int column) {
					super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					setHorizontalAlignment(JLabel.RIGHT);
					setText(format.format(value) + " kg");
					return this;
				}
			});
			getTableHeader().setBackground(Color.WHITE);
			getTableHeader().setFont(getTableHeader().getFont().deriveFont(Font.BOLD));
			getTableHeader().setBorder(BorderFactory.createMatteBorder(0,0,1,0,Color.BLACK));
			getTableHeader().setOpaque(false);
		}
		public Mission getSelectedMission() {
			if(getSelectedRow()>=0)
				return (Mission)getValueAt(getSelectedRow(), 1);
			else return null;
		}
		public void updateView() {
			int row = getSelectedRow();
			((RepairSummaryModel)getModel()).fireTableDataChanged();
			getSelectionModel().setSelectionInterval(row, row);
		}
		private class RepairSummaryModel extends AbstractTableModel {
			private static final long serialVersionUID = -1788294077902504936L;
			private Scenario scenario;
			
			public RepairSummaryModel(Scenario scenario) {
				this.scenario = scenario;
			}
			public int getColumnCount() {
				return 4;
			}
			public int getRowCount() {
				return scenario.getMissionList().size();
			}
			public Object getValueAt(int row, int col) {
				if(col==0) return row;
				if(col==1) return scenario.getMissionList().get(row);
				if(col==2) return getDeltaTime(row);
				else return getDeltaMass(row);
			}
			private double getDeltaTime(int row) {
				double amount = 0;
				if(scenario.getRepairedItems().get(scenario.getMissionList().get(row))!=null) {
					for(RepairItem item : scenario.getRepairedItems().get(scenario.getMissionList().get(row))) {
						amount += item.getMeanRepairTime();
					}
				}
				return amount;
			}
			private double getDeltaMass(int row) {
				double amount = 0;
				if(scenario.getRepairedItems().get(scenario.getMissionList().get(row))!=null) {
					for(RepairItem item : scenario.getRepairedItems().get(scenario.getMissionList().get(row))) {
						amount += item.getDemand().getMass() - item.getMassToRepair();
					}
				}
				return amount;
			}
		}
	}
	private class RepairableDemandsTable extends JTable {
		private static final long serialVersionUID = -8427772886041511648L;
		public RepairableDemandsTable() {
			setModel(new RepairableDemandsModel());
			getColumnModel().getColumn(0).setHeaderValue("Element");
			getColumnModel().getColumn(1).setHeaderValue("Item");
			getColumnModel().getColumn(2).setHeaderValue("Amount");
			getColumnModel().getColumn(2).setCellRenderer(new DefaultTableCellRenderer() {
				private static final long serialVersionUID = 1L;
				private DecimalFormat format = new DecimalFormat("0.00");
				@Override
				public Component getTableCellRendererComponent(JTable table, Object value,
						boolean isSelected, boolean hasFocus, int row, int column) {
					super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					setHorizontalAlignment(JLabel.RIGHT);
					setText(format.format(value) + " " + ((RepairableDemandsModel)getModel()).getDemand(row).getResource().getUnits());
					return this;
				}
			});
			getColumnModel().getColumn(3).setHeaderValue("Mass");
			getColumnModel().getColumn(3).setCellRenderer(new DefaultTableCellRenderer() {
				private static final long serialVersionUID = 1L;
				private DecimalFormat format = new DecimalFormat("0.00");
				@Override
				public Component getTableCellRendererComponent(JTable table, Object value,
						boolean isSelected, boolean hasFocus, int row, int column) {
					super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					setHorizontalAlignment(JLabel.RIGHT);
					setText(format.format(value) + " kg");
					return this;
				}
			});
			getColumnModel().getColumn(4).setHeaderValue("Mass to Repair");
			getColumnModel().getColumn(4).setCellRenderer(new DefaultTableCellRenderer() {
				private static final long serialVersionUID = 1L;
				private DecimalFormat format = new DecimalFormat("0.00");
				@Override
				public Component getTableCellRendererComponent(JTable table, Object value,
						boolean isSelected, boolean hasFocus, int row, int column) {
					super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					setHorizontalAlignment(JLabel.RIGHT);
					setText(format.format(value) + " kg");
					return this;
				}
			});
			getColumnModel().getColumn(5).setHeaderValue("Time to Repair");
			getColumnModel().getColumn(5).setCellRenderer(new DefaultTableCellRenderer() {
				private static final long serialVersionUID = 1L;
				private DecimalFormat format = new DecimalFormat("0.00");
				@Override
				public Component getTableCellRendererComponent(JTable table, Object value,
						boolean isSelected, boolean hasFocus, int row, int column) {
					super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					setHorizontalAlignment(JLabel.RIGHT);
					setText(format.format(value) + " hr");
					return this;
				}
			});
			getColumnModel().getColumn(6).setHeaderValue("Amount Repaired");
			getColumnModel().getColumn(6).setCellRenderer(new DefaultTableCellRenderer() {
				private static final long serialVersionUID = 1L;
				private DecimalFormat format = new DecimalFormat("0.00");
				@Override
				public Component getTableCellRendererComponent(JTable table, Object value,
						boolean isSelected, boolean hasFocus, int row, int column) {
					super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					setHorizontalAlignment(JLabel.RIGHT);
					setText(format.format(value) + " " + ((RepairableDemandsModel)getModel()).getDemand(row).getResource().getUnits());
					return this;
				}
			});
			setBackground(Color.WHITE);
			getTableHeader().setReorderingAllowed(false);
			getTableHeader().setBackground(Color.WHITE);
			getTableHeader().setFont(getTableHeader().getFont().deriveFont(Font.BOLD));
			getTableHeader().setBorder(BorderFactory.createMatteBorder(0,0,1,0,Color.BLACK));
			getTableHeader().setOpaque(false);
		}
		public void forceUpdate(int index) {
			((RepairableDemandsModel)getModel()).setIndex(index);
			((RepairableDemandsModel)getModel()).fireTableDataChanged();
		}
		private class RepairableDemandsModel extends AbstractTableModel {
			private static final long serialVersionUID = -2630916184656214544L;
			private int index;
			public int getColumnCount() {
				return 7;
			}
			public Class<?> getColumnClass(int column) {
				if(column!=0 && column!=1) return Double.class;
				else return Object.class;
			}
			public int getRowCount() {
				if(tab.getSimulator().getUnsortedRepairItems().get(index)==null) return 0;
				return tab.getSimulator().getUnsortedRepairItems().get(index).size();
			}
			public Object getValueAt(int row, int column) {
				if(column==0) return getElement(row);
				else if(column==1) return getDemand(row).getResource();
				else if(column==2) return getDemand(row).getAmount();
				else if(column==3) return getDemand(row).getMass();
				else if(column==4) return getRepairItem(row).getMassToRepair();
				else if(column==5) return getRepairItem(row).getMeanRepairTime();
				else if(column==6) {
					HashSet<RepairItem> repairedItems = new HashSet<RepairItem>();
					if(getScenario().getRepairedItems().get(getScenario().getMissionList().get(index))!=null) {
						for(RepairItem item : getScenario().getRepairedItems().get(getScenario().getMissionList().get(index))) {
							repairedItems.add(new RepairItem(new Demand(item.getDemand().getResource(), item.getDemand().getAmount()), item.getElement()));
						}
					}
					for(RepairItem simItem : tab.getSimulator().getUnsortedRepairItems().get(index)) {
						double amountRepaired = 0;
						for(RepairItem item : repairedItems) {
							if(simItem.getDemand().getResource().equals(item.getDemand().getResource())
									&& simItem.getElement().equals(item.getElement())) {
								double amountNewlyRepaired = Math.min(simItem.getDemand().getAmount() - amountRepaired, item.getDemand().getAmount());
								item.getDemand().setAmount(item.getDemand().getAmount()-amountNewlyRepaired);
								amountRepaired = amountRepaired + amountNewlyRepaired;
							}
						}
						if(getRepairItem(row).equals(simItem)) return amountRepaired;
					}
					return 0;
				}
				else return null;
			}
			public void setValueAt(Object value, int row, int column) {
				if(column==6) {
					double oldValue = (Double)getValueAt(row, column);
					double newValue = Math.min(getDemand(row).getAmount(), (Double)value);
					
					if(newValue > oldValue) {
						if(getScenario().getRepairedItems().get(getScenario().getMissionList().get(index))==null) {
							getScenario().getRepairedItems().put(getScenario().getMissionList().get(index), new HashSet<RepairItem>());
						}
						boolean isFound = false;
						for(RepairItem item : getScenario().getRepairedItems().get(getScenario().getMissionList().get(index))) {
							if(item.getDemand().getResource().equals(getDemand(row).getResource())
									&& item.getElement().equals(getElement(row))) {
								isFound = true;
								item.getDemand().setAmount(item.getDemand().getAmount() + (newValue - oldValue));
							}
						}
						if(!isFound) {
							getScenario().getRepairedItems().get(getScenario().getMissionList().get(index)).add(
									new RepairItem(new Demand(getDemand(row).getResource(), newValue - oldValue), 
											getElement(row)));
						}
					} else if(newValue < oldValue) {
						for(RepairItem item : getScenario().getRepairedItems().get(getScenario().getMissionList().get(index))) {
							if(item.getDemand().getResource().equals(getDemand(row).getResource())
									&& item.getElement().equals(getElement(row))) {
								item.getDemand().setAmount(item.getDemand().getAmount() + (newValue - oldValue));
							}
						}
					}
					fireTableRowsUpdated(row, row);
				}
			}
			public boolean isCellEditable(int row, int column) {
				if(column==6) return true;
				else return false;
			}
			private RepairItem getRepairItem(int row) {
				return tab.getSimulator().getUnsortedRepairItems().get(index).get(row);
			}
			private I_Element getElement(int row) {
				return getRepairItem(row).getElement();
			}
			private Demand getDemand(int row) {
				return getRepairItem(row).getDemand();
			}
			public void setIndex(int index) {
				this.index = index;
			}
		}
	}
}
