/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.demand;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;

import org.jdesktop.swingworker.SwingWorker;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.time.Hour;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleInsets;

import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.CheckBoxTableModel;
import edu.mit.spacenet.gui.component.SNChartPanel;
import edu.mit.spacenet.scenario.SupplyEdge;
import edu.mit.spacenet.util.DateFunctions;

/**
 * Component for visualizing the feasibility of a scenario based on the used and
 * remaining delivery capacity.
 * 
 * @author Paul Grogan
 */
public class ScenarioFeasibilityChart extends JSplitPane {
	private static final long serialVersionUID = -6405124067606236705L;
	private static final String RAW_CAPACITY = "Raw Delivery Capacity";
	private static final String REMAINING_CAPACITY = "Remaining Delivery Capacity";
	private static final String ESTIMATED_DEMANDS = "Estimated Demands";
	private DemandsTab demandsTab;
	private ChartPanel demandChartPanel;
	private JComboBox nodeCombo;
	private CheckBoxTableModel<SupplyEdge> supplyEdgesModel;
	private CheckBoxTableModel<String> dataModel;
	private JCheckBox cumulativeCheck, dateCheck;
	private JButton refreshButton;
	
	/**
	 * Instantiates a new scenario feasibility chart.
	 * 
	 * @param tab the demands tab
	 */
	public ScenarioFeasibilityChart(DemandsTab tab) {
		this.demandsTab = tab;
		
		demandChartPanel = new SNChartPanel();
		demandChartPanel.setPreferredSize(new Dimension(600,400));
		setLeftComponent(demandChartPanel);
		
		JPanel controlPanel = new JPanel();
		controlPanel.setBackground(Color.WHITE);
		controlPanel.setLayout(new GridBagLayout());
		controlPanel.setBorder(BorderFactory.createTitledBorder("Chart Options"));
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_START;
		nodeCombo = new JComboBox();
		for(Node node : tab.getScenario().getNetwork().getNodes()) {
			nodeCombo.addItem(node);
		}
		if(tab.getScenario().getMissionList().size()>0
				&& tab.getScenario().getMissionList().get(0).getDestination()!=null) {
			nodeCombo.setSelectedItem(tab.getScenario().getMissionList().get(0).getDestination());
		}
		nodeCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED && isAutoRefresh()) refreshChart();
			}
		});
		JPanel nodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 3, 0));
		nodePanel.setOpaque(false);
		nodePanel.add(new JLabel("Feasibility at Node: "));
		nodePanel.add(nodeCombo);
		controlPanel.add(nodePanel, c);
		c.gridy++;
		cumulativeCheck = new JCheckBox("Plot Cumulative Demands");
		cumulativeCheck.setOpaque(false);
		cumulativeCheck.setSelected(true);
		cumulativeCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(isAutoRefresh()) refreshChart();
				dateCheck.setEnabled(cumulativeCheck.isSelected());
				if(!cumulativeCheck.isSelected()) dateCheck.setSelected(false);
			}
		});
		controlPanel.add(cumulativeCheck, c);
		c.gridy++;
		dateCheck = new JCheckBox("Plot over Time");
		dateCheck.setOpaque(false);
		dateCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(isAutoRefresh()) refreshChart();
			}
		});
		controlPanel.add(dateCheck, c);
		c.gridy++;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		supplyEdgesModel = new CheckBoxTableModel<SupplyEdge>();
		supplyEdgesModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				if(e.getType()==TableModelEvent.UPDATE && isAutoRefresh()) updateChart();
			}
		});
		JTable supplyEdgesTable = new JTable(supplyEdgesModel);
		supplyEdgesTable.getColumnModel().getColumn(0).setHeaderRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/eye.png")));
                setHorizontalAlignment(JLabel.CENTER);
                return this;
			}
		});
		supplyEdgesTable.getTableHeader().setReorderingAllowed(false);
		supplyEdgesTable.getTableHeader().setBackground(Color.WHITE);
		supplyEdgesTable.getTableHeader().setFont(supplyEdgesTable.getTableHeader().getFont().deriveFont(Font.BOLD));
		supplyEdgesTable.getTableHeader().setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		supplyEdgesTable.getTableHeader().setOpaque(false);
		supplyEdgesTable.getColumnModel().getColumn(0).setMaxWidth(25);
		supplyEdgesTable.getColumnModel().getColumn(1).setHeaderValue("Filter Supply Edges");
		supplyEdgesTable.setShowGrid(false);
		JScrollPane supplyEdgesScroll = new JScrollPane(supplyEdgesTable);
		supplyEdgesScroll.setPreferredSize(new Dimension(150,150));
		supplyEdgesScroll.setBackground(Color.WHITE);
		supplyEdgesScroll.getViewport().setBackground(Color.WHITE);
		controlPanel.add(supplyEdgesScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel supplyEdgesButtonPanel = new JPanel();
		supplyEdgesButtonPanel.setOpaque(false);
		supplyEdgesButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllSupplyEdgesButton = new JButton("Select All");
		selectAllSupplyEdgesButton.setToolTipText("Select All Supply Edges");
		selectAllSupplyEdgesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				supplyEdgesModel.selectAll();
			}
		});
		supplyEdgesButtonPanel.add(selectAllSupplyEdgesButton);
		JButton deselectAllSupplyEdgesButton = new JButton("Deselect All");
		deselectAllSupplyEdgesButton.setToolTipText("Deselect All Supply Edges");
		deselectAllSupplyEdgesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				supplyEdgesModel.deselectAll();
			}
		});
		supplyEdgesButtonPanel.add(deselectAllSupplyEdgesButton);
		controlPanel.add(supplyEdgesButtonPanel, c);
		c.weightx = 0;
		c.fill = GridBagConstraints.BOTH;
		c.gridy++;
		c.weighty = 1;
		dataModel = new CheckBoxTableModel<String>();
		dataModel.addObject(RAW_CAPACITY);
		dataModel.addObject(REMAINING_CAPACITY);
		dataModel.addObject(ESTIMATED_DEMANDS);
		dataModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				if(e.getType()==TableModelEvent.UPDATE && isAutoRefresh()) updateChart();
			}
		});
		JTable dataTable = new JTable(dataModel);
		dataTable.getColumnModel().getColumn(0).setHeaderRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/eye.png")));
                setHorizontalAlignment(JLabel.CENTER);
                return this;
			}
		});
		dataTable.setBackground(Color.WHITE);
		dataTable.getTableHeader().setReorderingAllowed(false);
		dataTable.getTableHeader().setBackground(Color.WHITE);
		dataTable.getTableHeader().setFont(dataTable.getTableHeader().getFont().deriveFont(Font.BOLD));
		dataTable.getTableHeader().setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		dataTable.getTableHeader().setOpaque(false);
		dataTable.getColumnModel().getColumn(0).setMaxWidth(25);
		dataTable.getColumnModel().getColumn(1).setHeaderValue("Filter Data");
		dataTable.setShowGrid(false);
		JScrollPane dataScroll = new JScrollPane(dataTable);
		dataScroll.setBackground(Color.WHITE);
		dataScroll.getViewport().setBackground(Color.WHITE);
		dataScroll.setPreferredSize(new Dimension(150,150));
		controlPanel.add(dataScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel dataButtonPanel = new JPanel();
		dataButtonPanel.setOpaque(false);
		dataButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllDataButton = new JButton("Select All");
		selectAllDataButton.setToolTipText("Select All Data");
		selectAllDataButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dataModel.selectAll();
			}
		});
		dataButtonPanel.add(selectAllDataButton);
		JButton deselectAllDataButton = new JButton("Deselect All");
		deselectAllDataButton.setToolTipText("Deselect All Data");
		deselectAllDataButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dataModel.deselectAll();
			}
		});
		dataButtonPanel.add(deselectAllDataButton);
		controlPanel.add(dataButtonPanel, c);
		c.gridy++;
		c.anchor = GridBagConstraints.LINE_END;
		refreshButton = new JButton("Refresh", new ImageIcon(getClass().getClassLoader().getResource("icons/arrow_refresh.png")));
		refreshButton.setVisible(false);
		refreshButton.setMargin(new Insets(3,3,3,3));
		refreshButton.setToolTipText("Refresh Chart");
		refreshButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				refreshChart();
			}
		});
		controlPanel.add(refreshButton, c);
		
		setRightComponent(controlPanel);
		
		setName("Scenario Feasibility");
		setOneTouchExpandable(true);
		setDividerSize(10);
		setBorder(BorderFactory.createEmptyBorder());
		setResizeWeight(1);
		//setLastDividerLocation(490);
		setDividerLocation(490);
	}
	private boolean isAutoRefresh() {
		return SpaceNetFrame.getSingleton().getPreferences().isAutoRefresh();
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		if(isAutoRefresh()) {
			refreshButton.setVisible(false);
			refreshChart();
		} else {
			refreshButton.setVisible(true);
		}
	}
	private boolean isUpdating = false;
	private void refreshChart() {
		if(isUpdating) return;
		isUpdating = true;
		SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
			public Void doInBackground() throws Exception {
				SpaceNetFrame.getSingleton().getStatusBar().setStatusMessage("Refreshing Chart...");
				demandsTab.getScenarioPanel().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				Node selectedNode = (Node)nodeCombo.getSelectedItem();
				ItemListener nodeListener = nodeCombo.getItemListeners()[0];
				nodeCombo.removeItemListener(nodeListener);
				nodeCombo.removeAllItems();
				for(Node node : demandsTab.getScenario().getNetwork().getNodes()) {
					nodeCombo.addItem(node);
				}
				nodeCombo.setSelectedItem(selectedNode);
				nodeCombo.addItemListener(nodeListener);
				
				List<SupplyEdge> deselectedSupplyEdges = supplyEdgesModel.getDeselectedObjects();
				supplyEdgesModel.clear();
				ArrayList<SupplyEdge> edges = new ArrayList<SupplyEdge>();
				for(SupplyEdge edge : demandsTab.getSimulator().getSupplyEdges()) {
					edges.add(edge);
				}
				Collections.reverse(edges);
				for(SupplyEdge edge : edges) {
					if(edge.getDestination().equals(nodeCombo.getSelectedItem())) {
						supplyEdgesModel.addObject(edge, !deselectedSupplyEdges.contains(edge));
					}
				}
				updateChart();
				return null;
			}
			public void done() {
				SpaceNetFrame.getSingleton().getStatusBar().clearStatusMessage();
				demandsTab.getScenarioPanel().setCursor(Cursor.getDefaultCursor());
				isUpdating = false;
			}
		};
		worker.execute();
	}
	private void updateChart() {
		if(!getLeftComponent().equals(demandChartPanel)) {
			setLeftComponent(demandChartPanel);
		}
		demandsTab.getScenarioPanel().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		Node node = (Node)nodeCombo.getSelectedItem();
		JFreeChart chart;
		
		if(cumulativeCheck.isSelected() && supplyEdgesModel.getSelectedObjects().size()>1) {
			if(dateCheck.isSelected()) {
				TimeSeriesCollection dataset = new TimeSeriesCollection();
				chart = ChartFactory.createTimeSeriesChart(
						null, 
						null, 
						"Mass (kg)", 
						dataset, 
						true, // legend
						true, 
						false);
				chart.setBackgroundPaint(Color.WHITE);
				XYPlot plot = (XYPlot) chart.getPlot();
				plot.setBackgroundPaint(Color.WHITE);
				plot.setDomainGridlinePaint(Color.GRAY);
				plot.setRangeGridlinePaint(Color.GRAY);
				plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
				plot.setRangeGridlinesVisible(true);
				plot.setDomainGridlinesVisible(true);
				XYItemRenderer r = chart.getXYPlot().getRenderer();
				
				TimeSeries rawCapacitySeries = new TimeSeries("Raw Capacity", Hour.class);
				dataset.addSeries(rawCapacitySeries);
				r.setSeriesPaint(dataset.getSeriesCount()-1, Color.BLUE);
				TimeSeries remainingCapacitySeries = new TimeSeries("Remaining Capacity", Hour.class);
				dataset.addSeries(remainingCapacitySeries);
				r.setSeriesPaint(dataset.getSeriesCount()-1, Color.GREEN);
				TimeSeries estimatedDemandsSeries = new TimeSeries("Estimated Demands", Hour.class);
				dataset.addSeries(estimatedDemandsSeries);
				r.setSeriesPaint(dataset.getSeriesCount()-1, Color.RED);
				
				double rawCapacity = 0;
				double remainingCapacity = 0;
				double estimatedDemands = 0;
				SupplyEdge prevEdge = null;
				
				ArrayList<SupplyEdge> edges = new ArrayList<SupplyEdge>();
				for(SupplyEdge edge : demandsTab.getSimulator().getSupplyEdges()) {
					edges.add(edge);
				}
				Collections.reverse(edges);
				
				for(SupplyEdge edge : edges) {
					Hour hour = new Hour(DateFunctions.getDate(demandsTab.getScenario().getStartDate(), edge.getStartTime()));
					if(edge.getDestination().equals(node)) {
						if(prevEdge != null) {
							if(dataModel.getSelectedObjects().contains(ESTIMATED_DEMANDS)
									&& supplyEdgesModel.getSelectedObjects().contains(edge)) 
								estimatedDemandsSeries.addOrUpdate(hour, estimatedDemands);
							if(dataModel.getSelectedObjects().contains(RAW_CAPACITY)
									&& supplyEdgesModel.getSelectedObjects().contains(edge)) 
								rawCapacitySeries.addOrUpdate(hour, rawCapacity);
							if(dataModel.getSelectedObjects().contains(REMAINING_CAPACITY)
									&& supplyEdgesModel.getSelectedObjects().contains(edge)) 
								remainingCapacitySeries.addOrUpdate(hour, remainingCapacity);
						}
						
						rawCapacity += edge.getRawCapacity();
						remainingCapacity += edge.getCapacity();
						estimatedDemands += demandsTab.getSimulator().getAggregatedEdgeDemands().get(edge).getTotalMass();
						estimatedDemands += demandsTab.getSimulator().getAggregatedNodeDemands().get(edge.getPoint()).getTotalMass();
						
						if(dataModel.getSelectedObjects().contains(ESTIMATED_DEMANDS)
								&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
							estimatedDemandsSeries.addOrUpdate(hour, estimatedDemands);
						}
						if(dataModel.getSelectedObjects().contains(RAW_CAPACITY)
								&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
							rawCapacitySeries.addOrUpdate(hour, rawCapacity);
						}
						if(dataModel.getSelectedObjects().contains(REMAINING_CAPACITY)
								&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
							remainingCapacitySeries.addOrUpdate(hour, remainingCapacity);
						}
						
						prevEdge = edge;
					}
				}
				
			} else {
				DefaultCategoryDataset dataset = new DefaultCategoryDataset();
				
				chart = ChartFactory.createLineChart(
						null, 
						"", 
						"Mass (kg)", 
						dataset, 
						PlotOrientation.VERTICAL, // the plot orientation
						true, // legend
						true, 
						false);
				
				CategoryItemRenderer r = chart.getCategoryPlot().getRenderer();
				
				double rawCapacity = 0;
				double remainingCapacity = 0;
				double estimatedDemands = 0;
				SupplyEdge prevEdge = null;
				
				ArrayList<SupplyEdge> edges = new ArrayList<SupplyEdge>();
				for(SupplyEdge edge : demandsTab.getSimulator().getSupplyEdges()) {
					edges.add(edge);
				}
				Collections.reverse(edges);
				
				for(SupplyEdge edge : edges) {
					if(edge.getDestination().equals(node)) {
						if(prevEdge != null) {
							if(dataModel.getSelectedObjects().contains(ESTIMATED_DEMANDS)
									&& supplyEdgesModel.getSelectedObjects().contains(edge)) 
								dataset.addValue(estimatedDemands, ESTIMATED_DEMANDS, edge);
							if(dataModel.getSelectedObjects().contains(RAW_CAPACITY)
									&& supplyEdgesModel.getSelectedObjects().contains(edge)) 
								dataset.addValue(rawCapacity, RAW_CAPACITY, edge);
							if(dataModel.getSelectedObjects().contains(REMAINING_CAPACITY)
									&& supplyEdgesModel.getSelectedObjects().contains(edge)) 
								dataset.addValue(remainingCapacity, REMAINING_CAPACITY, edge);
						}
						
						rawCapacity += edge.getRawCapacity();
						remainingCapacity += edge.getCapacity();
						estimatedDemands += demandsTab.getSimulator().getAggregatedEdgeDemands().get(edge).getTotalMass();
						estimatedDemands += demandsTab.getSimulator().getAggregatedNodeDemands().get(edge.getPoint()).getTotalMass();
						
						if(dataModel.getSelectedObjects().contains(ESTIMATED_DEMANDS)
								&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
							dataset.addValue(estimatedDemands, ESTIMATED_DEMANDS, edge);
							r.setSeriesPaint(dataset.getRowCount() - 1, Color.RED);
						}
						if(dataModel.getSelectedObjects().contains(RAW_CAPACITY)
								&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
							dataset.addValue(rawCapacity, RAW_CAPACITY, edge);
							r.setSeriesPaint(dataset.getRowCount() - 1, Color.BLUE);
						}
						if(dataModel.getSelectedObjects().contains(REMAINING_CAPACITY)
								&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
							dataset.addValue(remainingCapacity, REMAINING_CAPACITY, edge);
							r.setSeriesPaint(dataset.getRowCount() - 1, Color.GREEN);
						}
						
						prevEdge = edge;
					}
				}

				CategoryPlot plot = (CategoryPlot) chart.getPlot();
				plot.getDomainAxis().setCategoryLabelPositions(CategoryLabelPositions.UP_90);
				plot.setBackgroundPaint(Color.WHITE);
				plot.setDomainGridlinePaint(Color.GRAY);
				plot.setRangeGridlinePaint(Color.GRAY);
				plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
				plot.setRangeGridlinesVisible(true);
				plot.setDomainGridlinesVisible(true);
			}
		} else {
			DefaultCategoryDataset dataset = new DefaultCategoryDataset();
			
			chart = ChartFactory.createBarChart(
					null, 
					"", 
					"Mass (kg)", 
					dataset, 
					PlotOrientation.VERTICAL, // the plot orientation
					true, // legend
					true, 
					false);
			
			BarRenderer r = (BarRenderer) chart.getCategoryPlot().getRenderer();
			r.setItemMargin(0);
			
			ArrayList<SupplyEdge> edges = new ArrayList<SupplyEdge>();
			for(SupplyEdge edge : demandsTab.getSimulator().getSupplyEdges()) {
				edges.add(edge);
			}
			Collections.reverse(edges);
			for(SupplyEdge edge : edges) {
				if(edge.getDestination().equals(node)) {
					if(dataModel.getSelectedObjects().contains(ESTIMATED_DEMANDS)
							&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
						dataset.addValue(demandsTab.getSimulator().getAggregatedEdgeDemands().get(edge).getTotalMass() 
								+ demandsTab.getSimulator().getAggregatedNodeDemands().get(edge.getPoint()).getTotalMass(),
								ESTIMATED_DEMANDS, edge);
						r.setSeriesPaint(dataset.getRowCount() - 1, Color.RED);
					}
					if(dataModel.getSelectedObjects().contains(RAW_CAPACITY)
							&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
						dataset.addValue(edge.getRawCapacity(), RAW_CAPACITY, edge);
						r.setSeriesPaint(dataset.getRowCount() - 1, Color.BLUE);
					}
					if(dataModel.getSelectedObjects().contains(REMAINING_CAPACITY)
							&& supplyEdgesModel.getSelectedObjects().contains(edge)) {
						dataset.addValue(edge.getCapacity(), REMAINING_CAPACITY, edge);
						r.setSeriesPaint(dataset.getRowCount() - 1, Color.GREEN);
					}
				}
			}

			CategoryPlot plot = (CategoryPlot) chart.getPlot();
			plot.getDomainAxis().setCategoryLabelPositions(CategoryLabelPositions.UP_90);
			plot.setBackgroundPaint(Color.WHITE);
			plot.setDomainGridlinePaint(Color.GRAY);
			plot.setRangeGridlinePaint(Color.GRAY);
			plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
			plot.setRangeGridlinesVisible(true);
			plot.setDomainGridlinesVisible(true);
		}
		
		chart.setBackgroundPaint(Color.WHITE);
		
		demandChartPanel.setChart(chart);
		demandsTab.getScenarioPanel().setCursor(Cursor.getDefaultCursor());
	}
}
