/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.demand;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.geom.Ellipse2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.axis.SymbolAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.time.Hour;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleInsets;

import com.toedter.calendar.JDateChooser;

import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.SNChartPanel;
import edu.mit.spacenet.scenario.Scenario;
import edu.mit.spacenet.scenario.SupplyEdge;
import edu.mit.spacenet.scenario.SupplyEdge.SupplyPoint;
import edu.mit.spacenet.util.DateFunctions;

/**
 * Component for visualizing the supply network of the scenario.
 * 
 * @author Paul Grogan
 */
public class SupplyNetworkChart extends JSplitPane {
	private static final long serialVersionUID = -6405124067606236705L;
	private DemandsTab tab;

	private ChartPanel networkChartPanel;
	
	private JDateChooser startDate, endDate;
	private JCheckBox demandsCheck, capacitiesCheck;
	private JButton refreshButton;
	
	/**
	 * Instantiates a new supply network chart.
	 * 
	 * @param tab the demands tab
	 */
	public SupplyNetworkChart(DemandsTab tab) {
		this.tab = tab;
		
		networkChartPanel = new SNChartPanel();
		networkChartPanel.setPreferredSize(new Dimension(500,300));
		setLeftComponent(networkChartPanel);
		
		JPanel controlPanel = new JPanel();
		controlPanel.setBackground(Color.WHITE);
		controlPanel.setLayout(new GridBagLayout());
		controlPanel.setBorder(BorderFactory.createTitledBorder("Chart Options"));
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_END;
		controlPanel.add(new JLabel("Start Date: "), c);
		c.gridy++;
		controlPanel.add(new JLabel("End Date: "), c);
		c.gridy--;
		c.gridx++;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		startDate = new JDateChooser();
		startDate.setSelectableDateRange(getScenario().getStartDate(), null);
		startDate.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if(e.getPropertyName().equals("date") && isAutoRefresh()) updateChart();
			}
		});
		controlPanel.add(startDate, c);
		c.gridy++;
		endDate = new JDateChooser();
		endDate.setSelectableDateRange(getScenario().getStartDate(), null);
		endDate.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if(e.getPropertyName().equals("date") && isAutoRefresh()) updateChart();
			}
		});
		controlPanel.add(endDate, c);
		c.gridx--;
		c.gridy++;
		c.gridwidth = 2;
		c.weightx = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		demandsCheck = new JCheckBox("Label Aggregated Demands");
		demandsCheck.setOpaque(false);
		demandsCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(isAutoRefresh()) updateChart();
			}
		});
		controlPanel.add(demandsCheck, c);
		c.gridy++;
		capacitiesCheck = new JCheckBox("Label Remaining Capacities");
		capacitiesCheck.setOpaque(false);
		capacitiesCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(isAutoRefresh()) updateChart();
			}
		});
		controlPanel.add(capacitiesCheck, c);
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1;
		JPanel prop3 = new JPanel();
		prop3.setOpaque(false);
		controlPanel.add(prop3, c);
		c.gridy++;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		refreshButton = new JButton("Refresh", new ImageIcon(getClass().getClassLoader().getResource("icons/arrow_refresh.png")));
		refreshButton.setVisible(false);
		refreshButton.setMargin(new Insets(3,3,3,3));
		refreshButton.setToolTipText("Refresh Chart");
		refreshButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateChart();
			}
		});
		controlPanel.add(refreshButton, c);
		
		setRightComponent(controlPanel);
		
		setName("Supply Network");
		setOneTouchExpandable(true);
		setDividerSize(10);
		setBorder(BorderFactory.createEmptyBorder());
		setResizeWeight(1);
		setDividerLocation(525);
	}
	
	/**
	 * Gets the scenario.
	 * 
	 * @return the scenario
	 */
	public Scenario getScenario() {
		return tab.getScenario();
	}
	private boolean isAutoRefresh() {
		return SpaceNetFrame.getSingleton().getPreferences().isAutoRefresh();
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		if(isAutoRefresh()) {
			refreshButton.setVisible(false);
			updateChart();
		} else {
			refreshButton.setVisible(true);
		}
	}
	private void updateChart() {
		double minTime =  startDate.getDate()==null?0:Math.max(0, DateFunctions.getDaysBetween(tab.getScenario().getStartDate(), startDate.getDate()));
		double maxTime = endDate.getDate()==null?Double.MAX_VALUE:Math.max(minTime, DateFunctions.getDaysBetween(tab.getScenario().getStartDate(), endDate.getDate()));
		TimeSeriesCollection dataset = new TimeSeriesCollection();
		
		JFreeChart chart = ChartFactory.createTimeSeriesChart(
				null, 
				"Date", 
				"Node", 
				dataset, 
				false, 
				true, 
				false);
		
		XYPlot plot = (XYPlot) chart.getPlot();
		SymbolAxis rangeAxis = createRangeAxis();
		plot.setRangeAxis(rangeAxis);
		plot.setBackgroundPaint(Color.WHITE);
		plot.setDomainGridlinePaint(Color.GRAY);
		plot.setRangeGridlinePaint(Color.GRAY);
		plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));

		XYItemRenderer r = chart.getXYPlot().getRenderer();
		chart.setBackgroundPaint(Color.WHITE);

		if (r instanceof XYLineAndShapeRenderer) {
			XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
			renderer.setBaseShapesVisible(true);
			renderer.setBaseShapesFilled(true);
		}
		double maxCapacity = 0;
		double minCapacity = Double.MAX_VALUE;
		for(SupplyEdge e : tab.getSimulator().getSupplyEdges()) {
			double capacity = e.getCapacity() - tab.getSimulator().getAggregatedEdgeDemands().get(e).getTotalMass();
			if(Math.abs(capacity)<minCapacity) minCapacity = capacity;
			if(Math.abs(capacity)>maxCapacity) maxCapacity = capacity;
		}
		for(SupplyEdge e : tab.getSimulator().getSupplyEdges()) {
			if(e.getStartTime() >= minTime && e.getEndTime() <= maxTime) {
				double capacity = e.getCapacity() - tab.getSimulator().getAggregatedEdgeDemands().get(e).getTotalMass();
				TimeSeries s = new TimeSeries(e.toString(), Hour.class);
				s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), e.getStartTime())), 
						node2Number(e.getOrigin(), rangeAxis));
				s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), e.getEndTime())), 
						node2Number(e.getDestination(), rangeAxis));
				((TimeSeriesCollection)dataset).addSeries(s);
				r.setSeriesShape(dataset.getSeriesCount() - 1, 
						new Ellipse2D.Double(0,0,0,0));
				int thickness = (int)(8-7*(maxCapacity-Math.abs(capacity))/(maxCapacity-minCapacity));
				r.setSeriesStroke(dataset.getSeriesCount() - 1, new BasicStroke(thickness));
				if(capacity >= 0) r.setSeriesPaint(dataset.getSeriesCount() - 1, new Color(0x66,0xff,0x66));
				else r.setSeriesPaint(dataset.getSeriesCount() - 1, new Color(0xff,0,0));
			}
		}
		double maxDemands = 0;
		double minDemands = Double.MAX_VALUE;
		for(SupplyPoint p : tab.getSimulator().getSupplyPoints()) {
			if(tab.getSimulator().getAggregatedNodeDemands().get(p).getTotalMass()<minDemands) minDemands = tab.getSimulator().getAggregatedNodeDemands().get(p).getTotalMass();
			if(tab.getSimulator().getAggregatedNodeDemands().get(p).getTotalMass()>maxDemands) maxDemands = tab.getSimulator().getAggregatedNodeDemands().get(p).getTotalMass();
		}
		for(SupplyPoint p : tab.getSimulator().getSupplyPoints()) {
			if(p.getTime() >= minTime && p.getTime() <= maxTime) {
				TimeSeries s = new TimeSeries(p.toString(), Hour.class);
				s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), p.getTime())), 
						node2Number(p.getNode(), rangeAxis));
				((TimeSeriesCollection)dataset).addSeries(s);
				r.setSeriesPaint(dataset.getSeriesCount() - 1, new Color(0x00,0x99,0xff));
				double minSize = minDemands==0?0:4;
				double size = minSize+16*(tab.getSimulator().getAggregatedNodeDemands().get(p).getTotalMass()-minDemands)/(maxDemands-minDemands);
				r.setSeriesShape(dataset.getSeriesCount() - 1, 
						new Ellipse2D.Double(-size/2D, -size/2D, size, size));
			}
			if(demandsCheck.isSelected()&&tab.getSimulator().getAggregatedNodeDemands().get(p).getTotalMass()>0) {
				double x = new Hour(DateFunctions.getDate(getScenario().getStartDate(), p.getTime())).getFirstMillisecond();
				//x -= (p.getEdge().getEndTime()-p.getEdge().getStartTime())*24*3600*1000*2;
				double y = node2Number(p.getNode(), rangeAxis);// + 0.25;
				DecimalFormat format = new DecimalFormat("0");
				XYTextAnnotation annotation = new XYTextAnnotation(format.format(tab.getSimulator().getAggregatedNodeDemands().get(p).getTotalMass()) + " kg", x, y);
				annotation.setPaint(Color.BLACK);
				r.addAnnotation(annotation);
			}
		}
		if(capacitiesCheck.isSelected()) {
			for(SupplyEdge e : tab.getSimulator().getSupplyEdges()) {
				double x1 = new Hour(DateFunctions.getDate(getScenario().getStartDate(), (e.getStartTime()+e.getEndTime())/2D)).getFirstMillisecond();
				//x -= (e.getEndTime()-e.getStartTime())*24*3600*1000/2D;
				double y1 = (node2Number(e.getDestination(), rangeAxis) + node2Number(e.getOrigin(), rangeAxis))/2D;
				DecimalFormat format = new DecimalFormat("0");
				double capacity = e.getCapacity() - tab.getSimulator().getAggregatedEdgeDemands().get(e).getTotalMass();
				XYTextAnnotation capacityAnnotation = new XYTextAnnotation(format.format(capacity) + " kg", x1, y1);
				double angle1 = Math.atan(-(
						(node2Number(e.getDestination(), rangeAxis)-node2Number(e.getOrigin(), rangeAxis))
						/(plot.getRangeAxis().getUpperBound()-plot.getRangeAxis().getLowerBound()))
						/((new Hour(DateFunctions.getDate(getScenario().getStartDate(), e.getEndTime())).getFirstMillisecond()-new Hour(DateFunctions.getDate(getScenario().getStartDate(), e.getStartTime())).getFirstMillisecond())
							/(plot.getDomainAxis().getUpperBound()-plot.getDomainAxis().getLowerBound())));
				capacityAnnotation.setRotationAngle(angle1);
				capacityAnnotation.setPaint(Color.BLACK);
				r.addAnnotation(capacityAnnotation);
			}
		}
			
		networkChartPanel.setChart(chart);
	}
	private SymbolAxis createRangeAxis() {
		Vector<String> nodeLabels = new Vector<String>();
		for (Node n : tab.getScenarioPanel().getScenario().getNetwork().getNodes()) {
			nodeLabels.add(n.getName());
		}
		String[] array = {};
		return new SymbolAxis(null, nodeLabels.toArray(array));
	}
	private int node2Number(Node node, SymbolAxis axis) {
		for (int i = 0; i < axis.getSymbols().length; i++) {
			if (axis.getSymbols()[i].equals(node.getName())) {
				return i;
			}
		}
		return -1;
	}
}
