/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.element;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.element.PropulsiveVehicle;
import edu.mit.spacenet.domain.element.ResourceTank;
import edu.mit.spacenet.domain.resource.GenericResource;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.UnitsWrapper;

/**
 * The element panel for viewing and editing propulsive vehicle-specific inputs.
 * 
 * @author Paul Grogan
 */
public class PropulsiveVehiclePanel extends AbstractElementPanel {
	private static final long serialVersionUID = 6335483106289763369L;
	
	private PropulsiveVehicle vehicle;
	
	private JCheckBox omsCheck, rcsCheck, sharedCheck;
	private SpinnerNumberModel omsIspModel, maxOmsFuelModel, omsFuelModel, 
		rcsIspModel, maxRcsFuelModel, rcsFuelModel;
	private JSpinner omsIspSpinner, maxOmsFuelSpinner, omsFuelSpinner,
		rcsIspSpinner, maxRcsFuelSpinner, rcsFuelSpinner;
	private UnitsWrapper maxOmsFuelWrapper, omsFuelWrapper, maxRcsFuelWrapper, rcsFuelWrapper;
	private JComboBox omsResourceCombo, rcsResourceCombo;
	
	/**
	 * Instantiates a new propulsive vehicle panel.
	 * 
	 * @param elementDialog the element dialog
	 * @param vehicle the vehicle
	 */
	public PropulsiveVehiclePanel(ElementDialog elementDialog, PropulsiveVehicle vehicle) {
		super(elementDialog, vehicle);
		this.vehicle = vehicle;
		buildPanel();
		initialize();
	}
	private void buildPanel() {
		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		
		JPanel omsPanel = new JPanel();
		omsPanel.setLayout(new GridBagLayout());
		omsPanel.setBorder(BorderFactory.createTitledBorder("OMS Engine"));
		add(omsPanel);
		
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		c.gridx = 0;
		c.gridy = 1;
		c.weightx = 0;
		omsPanel.add(new JLabel("Isp: "), c);
		c.gridy++;
		omsPanel.add(new JLabel("Fuel Type: "), c);
		c.gridy++;
		omsPanel.add(new JLabel("Max Fuel: "), c);
		c.gridy++;
		omsPanel.add(new JLabel("Initial Fuel: "), c);
		
		c.gridx = 1;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		omsCheck = new JCheckBox("OMS Capabilities");
		omsPanel.add(omsCheck, c);
		c.gridy++;
		omsIspModel = new SpinnerNumberModel(0,0,500,5.);
		omsIspSpinner = new JSpinner(omsIspModel);
		omsIspSpinner.setPreferredSize(new Dimension(50,20));
		omsPanel.add(new UnitsWrapper(omsIspSpinner, "s"), c);
		c.gridy++;
		omsResourceCombo = new JComboBox();
		omsResourceCombo.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				if(value!=null)setIcon(((I_Resource)value).getResourceType().getIcon());
				return this;
			}
		});
		omsResourceCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED
						&& omsResourceCombo.getSelectedItem() != null
						&& omsResourceCombo.getSelectedItem() instanceof I_Resource) {
					String fuelUnits = ((I_Resource)omsResourceCombo.getSelectedItem()).getUnits();
					maxOmsFuelWrapper.setUnits(fuelUnits);
					omsFuelWrapper.setUnits(fuelUnits);
				}
			}
		});
		omsPanel.add(omsResourceCombo, c);
		c.gridy++;
		maxOmsFuelModel = new SpinnerNumberModel(0,0,10000,100.);
		maxOmsFuelSpinner = new JSpinner(maxOmsFuelModel);
		maxOmsFuelWrapper = new UnitsWrapper(maxOmsFuelSpinner, "");
		omsPanel.add(maxOmsFuelWrapper, c);
		c.gridy++;
		omsFuelModel = new SpinnerNumberModel(0,0,10000,100.);
		omsFuelSpinner = new JSpinner(omsFuelModel);
		omsFuelWrapper = new UnitsWrapper(omsFuelSpinner, "");
		omsPanel.add(omsFuelWrapper, c);
		
		JPanel rcsPanel = new JPanel();
		rcsPanel.setLayout(new GridBagLayout());
		rcsPanel.setBorder(BorderFactory.createTitledBorder("RCS Engine"));
		add(rcsPanel);
		
		c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		c.gridx = 0;
		c.gridy = 1;
		c.weightx = 0;
		rcsPanel.add(new JLabel("Isp: "), c);
		c.gridy+=2;
		rcsPanel.add(new JLabel("Fuel Type: "), c);
		c.gridy++;
		rcsPanel.add(new JLabel("Max Fuel: "), c);
		c.gridy++;
		rcsPanel.add(new JLabel("Initial Fuel: "), c);
		
		c.gridx = 1;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		rcsCheck = new JCheckBox("RCS Capabilities");
		rcsPanel.add(rcsCheck, c);
		c.gridy++;
		rcsIspModel = new SpinnerNumberModel(0,0,500,5.);
		rcsIspSpinner = new JSpinner(rcsIspModel);
		rcsIspSpinner.setPreferredSize(new Dimension(50,20));
		rcsPanel.add(new UnitsWrapper(rcsIspSpinner, "s"), c);
		c.gridy++;
		sharedCheck = new JCheckBox("Shared OMS Fuel Tank");
		rcsPanel.add(sharedCheck, c);
		c.gridy++;
		rcsResourceCombo = new JComboBox();
		rcsResourceCombo.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				if(value!=null)setIcon(((I_Resource)value).getResourceType().getIcon());
				return this;
			}
		});
		rcsResourceCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED
						&& rcsResourceCombo.getSelectedItem() != null
						&& rcsResourceCombo.getSelectedItem() instanceof I_Resource) {
					String fuelUnits = ((I_Resource)rcsResourceCombo.getSelectedItem()).getUnits();
					maxRcsFuelWrapper.setUnits(fuelUnits);
					rcsFuelWrapper.setUnits(fuelUnits);
				}
			}
		});
		rcsPanel.add(rcsResourceCombo, c);
		c.gridy++;
		maxRcsFuelModel = new SpinnerNumberModel(0,0,10000,100.);
		maxRcsFuelSpinner = new JSpinner(maxRcsFuelModel);
		maxRcsFuelWrapper = new UnitsWrapper(maxRcsFuelSpinner, "");
		rcsPanel.add(maxRcsFuelWrapper, c);
		c.gridy++;
		rcsFuelModel = new SpinnerNumberModel(0,0,10000,100.);
		rcsFuelSpinner = new JSpinner(rcsFuelModel);
		rcsFuelWrapper = new UnitsWrapper(rcsFuelSpinner, "");
		rcsPanel.add(rcsFuelWrapper, c);
	}
	private void initialize() {
		omsResourceCombo.addItem(null);
		rcsResourceCombo.addItem(null);
		for(I_Resource r : SpaceNetFrame.getSingleton().getScenarioPanel().getScenario().getDataSource().getResourceLibrary()) {
			if(r.getClassOfSupply().equals(ClassOfSupply.COS1)
					|| r.getClassOfSupply().isSubclassOf(ClassOfSupply.COS1)) {
				omsResourceCombo.addItem(r);
				rcsResourceCombo.addItem(r);
			}
		}
		for(ClassOfSupply cos : ClassOfSupply.values()) {
			if(cos.equals(ClassOfSupply.COS1)
					|| cos.isSubclassOf(ClassOfSupply.COS1)) {
				omsResourceCombo.addItem(new GenericResource(cos));
				rcsResourceCombo.addItem(new GenericResource(cos));
			}
		}
		if(vehicle.getOmsIsp() > 0) omsCheck.setSelected(true);
		else omsCheck.setSelected(false);
		
		if(vehicle.getRcsIsp() > 0) rcsCheck.setSelected(true);
		else rcsCheck.setSelected(false);
		
		if(vehicle.getOmsFuelTank() == vehicle.getRcsFuelTank())
			sharedCheck.setSelected(true);
		else sharedCheck.setSelected(false);
		
		omsCheck.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				update();
			}
		});
		rcsCheck.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				update();
			}
		});
		sharedCheck.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				update();
			}
		});
		
		if(omsCheck.isSelected()) {
			omsIspModel.setValue(vehicle.getOmsIsp());
			if(vehicle.getOmsFuelTank() != null) {
				maxOmsFuelModel.setValue(vehicle.getOmsFuelTank().getMaxAmount());
				omsFuelModel.setValue(vehicle.getOmsFuelTank().getAmount());
				omsResourceCombo.setSelectedItem(vehicle.getOmsFuelTank().getResource());
			}
		}
		if(rcsCheck.isSelected() && sharedCheck.isSelected()) {
			rcsIspModel.setValue(vehicle.getRcsIsp());
		} else if(rcsCheck.isSelected()) {
			rcsIspModel.setValue(vehicle.getRcsIsp());
			if(vehicle.getRcsFuelTank() != null) {
				maxRcsFuelModel.setValue(vehicle.getRcsFuelTank().getMaxAmount());
				rcsFuelModel.setValue(vehicle.getRcsFuelTank().getAmount());
				rcsResourceCombo.setSelectedItem(vehicle.getRcsFuelTank().getResource());
			}
		}
		update();
	}
	private void update() {
		if(omsCheck.isSelected()) {
			omsIspSpinner.setEnabled(true);
			maxOmsFuelSpinner.setEnabled(true);
			omsFuelSpinner.setEnabled(true);
			omsResourceCombo.setEnabled(true);
		} else {
			omsIspModel.setValue(0);
			omsIspSpinner.setEnabled(false);
			maxOmsFuelModel.setValue(0);
			maxOmsFuelSpinner.setEnabled(false);
			omsFuelModel.setValue(0);
			omsFuelSpinner.setEnabled(false);
			omsResourceCombo.setSelectedItem(null);
			omsResourceCombo.setEnabled(false);
		}
		if(rcsCheck.isSelected() && sharedCheck.isSelected()) {
			rcsIspSpinner.setEnabled(true);
			sharedCheck.setEnabled(true);
			maxRcsFuelModel.setValue(0);
			maxRcsFuelSpinner.setEnabled(false);
			rcsFuelModel.setValue(0);
			rcsFuelSpinner.setEnabled(false);
			rcsResourceCombo.setSelectedItem(null);
			rcsResourceCombo.setEnabled(false);
		} else if(rcsCheck.isSelected()) {
			rcsIspSpinner.setEnabled(true);
			sharedCheck.setEnabled(true);
			maxRcsFuelSpinner.setEnabled(true);
			rcsFuelSpinner.setEnabled(true);
			rcsResourceCombo.setEnabled(true);
		} else {
			sharedCheck.setSelected(false);
			sharedCheck.setEnabled(false);
			rcsIspModel.setValue(0);
			rcsIspSpinner.setEnabled(false);
			maxRcsFuelModel.setValue(0);
			maxRcsFuelSpinner.setEnabled(false);
			rcsFuelModel.setValue(0);
			rcsFuelSpinner.setEnabled(false);
			rcsResourceCombo.setSelectedItem(null);
			rcsResourceCombo.setEnabled(false);
		}
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.element.AbstractElementPanel#getElement()
	 */
	@Override
	public PropulsiveVehicle getElement() {
		return vehicle;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.element.AbstractElementPanel#saveElement()
	 */
	@Override
	public void saveElement() {
		if(omsCheck.isSelected()) {
			vehicle.setOmsIsp(omsIspModel.getNumber().doubleValue());
			ResourceTank omsFuelTank = new ResourceTank();
			omsFuelTank.setMaxAmount(maxOmsFuelModel.getNumber().doubleValue());
			omsFuelTank.setAmount(omsFuelModel.getNumber().doubleValue());
			omsFuelTank.setResource((I_Resource)omsResourceCombo.getSelectedItem());
			vehicle.setOmsFuelTank(omsFuelTank);
		} else {
			vehicle.setOmsIsp(0);
			vehicle.setOmsFuelTank(null);
		}
		if(rcsCheck.isSelected() && !sharedCheck.isSelected()) {
			vehicle.setRcsIsp(rcsIspModel.getNumber().doubleValue());
			ResourceTank rcsFuelTank = new ResourceTank();
			rcsFuelTank.setMaxAmount(maxRcsFuelModel.getNumber().doubleValue());
			rcsFuelTank.setAmount(rcsFuelModel.getNumber().doubleValue());
			rcsFuelTank.setResource((I_Resource)rcsResourceCombo.getSelectedItem());
			vehicle.setRcsFuelTank(rcsFuelTank);
		} else if(rcsCheck.isSelected() && sharedCheck.isSelected()) {
			vehicle.setRcsIsp(rcsIspModel.getNumber().doubleValue());
			vehicle.setRcsFuelTank(vehicle.getOmsFuelTank());
		} else {
			vehicle.setRcsIsp(0);
			vehicle.setRcsFuelTank(null);
		}
	}

}
