/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.element;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.element.SurfaceVehicle;
import edu.mit.spacenet.domain.resource.GenericResource;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.UnitsWrapper;

/**
 * An element panel for viewing and editing surface vehicle-specific inputs.
 * 
 * @author Paul Grogan
 */
public class SurfaceVehiclePanel extends AbstractElementPanel {
	private static final long serialVersionUID = 6335483106289763369L;
	
	private SurfaceVehicle vehicle;
	
	private JSpinner maxSpeedSpinner, maxFuelSpinner, fuelSpinner;
	private SpinnerNumberModel maxSpeedModel, maxFuelModel, fuelModel;
	private UnitsWrapper maxFuelWrapper, fuelWrapper;
	private JComboBox resourceCombo;
	
	/**
	 * Instantiates a new surface vehicle panel.
	 * 
	 * @param elementDialog the element dialog
	 * @param vehicle the vehicle
	 */
	public SurfaceVehiclePanel(ElementDialog elementDialog, SurfaceVehicle vehicle) {
		super(elementDialog, vehicle);
		this.vehicle = vehicle;
		buildPanel();
		initialize();
	}
	private void buildPanel() {
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridy = 0;
		c.gridx = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		add(new JLabel("Maximum Speed: "), c);
		c.gridy++;
		add(new JLabel("Fuel Type: "), c);
		c.gridy++;
		add(new JLabel("Max Fuel: "), c);
		c.gridx+=2;
		c.weightx = 1;
		add(new JLabel("Initial Fuel: "), c);
		
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 1;
		maxSpeedModel = new SpinnerNumberModel(0,0,100,5.);
		maxSpeedSpinner = new JSpinner(maxSpeedModel);
		add(new UnitsWrapper(maxSpeedSpinner, "km/hr"), c);
		c.gridy++;
		resourceCombo = new JComboBox();
		resourceCombo.addItem(null);
		for(I_Resource r : SpaceNetFrame.getSingleton().getScenarioPanel().getScenario().getDataSource().getResourceLibrary()) {					
			if(r.getClassOfSupply().equals(ClassOfSupply.COS1)
						|| r.getClassOfSupply().isSubclassOf(ClassOfSupply.COS1)) {
				resourceCombo.addItem(r);
			}
		}
		for(ClassOfSupply cos : ClassOfSupply.values()) {
			if(cos.equals(ClassOfSupply.COS1)
					|| cos.isSubclassOf(ClassOfSupply.COS1)) {
				resourceCombo.addItem(new GenericResource(cos));
			}
		}
		resourceCombo.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				if(value!=null)setIcon(((I_Resource)value).getResourceType().getIcon());
				return this;
			}
		});
		resourceCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED
						&& resourceCombo.getSelectedItem() != null) {
					String fuelUnits = ((I_Resource)resourceCombo.getSelectedItem()).getUnits();
					maxFuelWrapper.setUnits(fuelUnits);
					fuelWrapper.setUnits(fuelUnits);
				}
			}
		});
		add(resourceCombo, c);
		c.gridy++;
		
		maxFuelModel = new SpinnerNumberModel(0,0,100,5.);
		maxFuelSpinner = new JSpinner(maxFuelModel);
		maxFuelWrapper = new UnitsWrapper(maxFuelSpinner, "");
		add(maxFuelWrapper, c);
		c.gridx+=2;
		fuelModel = new SpinnerNumberModel(0,0,100,5.);
		fuelSpinner = new JSpinner(fuelModel);
		fuelWrapper = new UnitsWrapper(fuelSpinner, "");
		add(fuelWrapper, c);
	}
	private void initialize() {
		maxSpeedModel.setValue(vehicle.getMaxSpeed());
		maxFuelModel.setValue(vehicle.getFuelTank().getMaxAmount());
		fuelModel.setValue(vehicle.getFuelTank().getAmount());
		resourceCombo.setSelectedItem(vehicle.getFuelTank().getResource());
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.element.AbstractElementPanel#getElement()
	 */
	@Override
	public SurfaceVehicle getElement() {
		return vehicle;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.element.AbstractElementPanel#saveElement()
	 */
	@Override
	public void saveElement() {
		vehicle.setMaxSpeed(maxSpeedModel.getNumber().doubleValue());
		vehicle.getFuelTank().setMaxAmount(maxFuelModel.getNumber().doubleValue());
		vehicle.getFuelTank().setAmount(fuelModel.getNumber().doubleValue());
		vehicle.getFuelTank().setResource((I_Resource)resourceCombo.getSelectedItem());
	}
}
