/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.manifest;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import edu.mit.spacenet.domain.element.I_Carrier;
import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.element.I_ResourceContainer;
import edu.mit.spacenet.scenario.Manifest;
import edu.mit.spacenet.scenario.SupplyEdge;
import edu.mit.spacenet.scenario.SupplyEdge.SupplyPoint;

/**
 * The table used to view the available resource containers for demand packing.
 * 
 * @author Paul Grogan
 */
public class PackedDemandsTable extends JTable {
	private static final long serialVersionUID = -5960985748957372743L;
	private Manifest manifest;
	
	/**
	 * Instantiates a new packed demands table.
	 * 
	 * @param m the m
	 */
	public PackedDemandsTable(Manifest m) {
		this.manifest = m;
		setModel(new PackedDemandsTableModel(manifest));
		getColumnModel().getColumn(0).setHeaderValue("Supply Point");
		getColumnModel().getColumn(0).setMaxWidth(100);
		getColumnModel().getColumn(0).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 6799027210269525751L;
			
			public Component getTableCellRendererComponent(JTable table, Object value, 
					boolean isSelected, boolean hasFocus, int row, int column)  {
				super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if(manifest.isEdgeDemand(getContainer(row))) setText(getText() + "*");
				return this;
			}
		});		
		getColumnModel().getColumn(1).setHeaderValue("Container");
		getColumnModel().getColumn(1).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 6799027210269525751L;
			
			public Component getTableCellRendererComponent(JTable table, Object value, 
					boolean isSelected, boolean hasFocus, int row, int column)  {
				super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				setIcon(((I_Element)value).getIcon());
				return this;
			}
		});
		getColumnModel().getColumn(2).setHeaderValue("Supply Edge");
		getColumnModel().getColumn(2).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 6799027210269525751L;
			
			public Component getTableCellRendererComponent(JTable table, Object value, 
					boolean isSelected, boolean hasFocus, int row, int column)  {
				super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if(value!=null&&!isSelected) {
					setBackground(new Color(0xcc,0xff,0xcc));
				} else if(!isSelected) {
					setBackground(new Color(0xff,0xcc,0xcc));
				}
				return this;
			}
		});
		setBackground(Color.WHITE);
		getTableHeader().setReorderingAllowed(false);
		getTableHeader().setBackground(Color.WHITE);
		getTableHeader().setFont(getTableHeader().getFont().deriveFont(Font.BOLD));
		getTableHeader().setBorder(BorderFactory.createMatteBorder(0,0,1,0,Color.BLACK));
		getTableHeader().setOpaque(false);
	}
	
	/**
	 * Gets the container.
	 * 
	 * @param row the row
	 * 
	 * @return the container
	 */
	public I_ResourceContainer getContainer(int row) {
		return ((PackedDemandsTableModel)getModel()).getContainer(row);
	}
	
	/**
	 * Gets the selected container.
	 * 
	 * @return the selected container
	 */
	public I_ResourceContainer getSelectedContainer() {
		int row = getSelectedRow();
		if(row<0) return null;
		else return getContainer(row);
	}
	
	/**
	 * Gets the supply point.
	 * 
	 * @param row the row
	 * 
	 * @return the supply point
	 */
	public SupplyPoint getSupplyPoint(int row) {
		return ((PackedDemandsTableModel)getModel()).getSupplyPoint(row);
	}
	
	/**
	 * Gets the selected supply point.
	 * 
	 * @return the selected supply point
	 */
	public SupplyPoint getSelectedSupplyPoint() {
		int row = getSelectedRow();
		if(row<0) return null;
		else return getSupplyPoint(row);
	}
	
	/**
	 * Gets the supply edge.
	 * 
	 * @param row the row
	 * 
	 * @return the supply edge
	 */
	public SupplyEdge getSupplyEdge(int row) {
		return ((PackedDemandsTableModel)getModel()).getSupplyEdge(row);
	}
	
	/**
	 * Gets the selected supply edge.
	 * 
	 * @return the selected supply edge
	 */
	public SupplyEdge getSelectedSupplyEdge() {
		int row = getSelectedRow();
		if(row<0) return null;
		else return getSupplyEdge(row);
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		((PackedDemandsTableModel)getModel()).fireTableDataChanged();
	}
	
	/**
	 * The Class PackedDemandsTableModel.
	 */
	class PackedDemandsTableModel extends AbstractTableModel {
		private static final long serialVersionUID = -678300273079091701L;
		
		private Manifest manifest;
		
		/**
		 * Instantiates a new packed demands table model.
		 * 
		 * @param manifest the manifest
		 */
		public PackedDemandsTableModel(Manifest manifest) {
			super();
			this.manifest = manifest;
		}
		
		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		public int getColumnCount() {
			return 3;
		}
		
		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		public int getRowCount() {
			int rows = manifest.getEmptyContainers().size();
			for(SupplyPoint point : manifest.getSupplyPoints()) {
				rows+=manifest.getCachedContainerDemands().get(point).size();
			}
			return rows;
		}
		
		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		public Object getValueAt(int row, int col) {
			if(col==0) return getSupplyPoint(row);
			else if(col==1) return getContainer(row);
			else return getSupplyEdge(row);
		}
		private I_ResourceContainer getContainer(int row) {
			int lastRowSeen = -1;
			for(I_ResourceContainer container : manifest.getEmptyContainers()) {
				lastRowSeen++;
				if(lastRowSeen==row) return container;
			}
			for(SupplyPoint point : manifest.getSupplyPoints()) {
				for(I_ResourceContainer container : manifest.getCachedContainerDemands().get(point)) {
					lastRowSeen++;
					if(lastRowSeen==row) return container;
				}
			}
			return null;
		}
		private SupplyPoint getSupplyPoint(int row) {
			int lastRowSeen = manifest.getEmptyContainers().size()-1;
			if(row<=lastRowSeen) return null;
			for(SupplyPoint point : manifest.getSupplyPoints()) {
				lastRowSeen+=manifest.getCachedContainerDemands().get(point).size();
				if(row<=lastRowSeen) return point;
			}
			return null;
		}
		private SupplyEdge getSupplyEdge(int row) {
			SupplyPoint point = getSupplyPoint(row);
			I_ResourceContainer container = getContainer(row);
			
			if(point==null) return null;
			for(SupplyEdge edge : manifest.getSupplyEdges()) {
				if(edge.getDestination().equals(point.getNode())
						&& edge.getEndTime()<=point.getTime()) {
					for(I_Carrier carrier : edge.getCarriers()) {
						if(manifest.getManifestedContainers().get(edge).get(carrier).contains(container))
							return edge;
					}
				}
			}
			return null;
		}
	}
}
