/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.mission;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.mit.spacenet.gui.command.AddMissionCommand;
import edu.mit.spacenet.gui.command.CopyMissionCommand;
import edu.mit.spacenet.gui.command.EditMissionCommand;
import edu.mit.spacenet.gui.command.RemoveMissionCommand;
import edu.mit.spacenet.scenario.Mission;

/**
 * A component for displaying the list of missions in a scenario. 
 * 
 * @author Paul Grogan
 */
public class MissionListPanel extends JPanel {
	private static final long serialVersionUID = 7721194192994518653L;
	
	private MissionsTab missionsTab;
	
	private MissionTable missionTable;
	private JButton editMissionButton, addMissionButton, copyMissionButton, removeMissionButton;
	
	/**
	 * Instantiates a new mission list panel.
	 * 
	 * @param missionsTab the missions tab
	 */
	public MissionListPanel(MissionsTab missionsTab) {
		this.missionsTab = missionsTab;
		buildPanel();
	}
	private void buildPanel() {
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 0;
		add(new JLabel("Missions List: "), c);
		c.gridy++;
		c.weighty = 1;	
		missionTable = new MissionTable(missionsTab);
		missionTable.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount() == 2 && missionTable.getSelectedRow() >=0) {
					missionsTab.editMission(missionTable.getMission(missionTable.getSelectedRow()));
				}
			}
		});
		missionTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				updateButtons();
			}
		});
		JScrollPane missionScroll = new JScrollPane(missionTable);
		missionScroll.setBackground(Color.WHITE);
		missionScroll.getViewport().setBackground(Color.WHITE);
		missionScroll.setPreferredSize(new Dimension(150,250));
		add(missionScroll, c);
		c.gridy++;
		c.weighty = .1;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 0));
		addMissionButton = new JButton("Add", new ImageIcon(getClass().getClassLoader().getResource("icons/application_add.png")));
		addMissionButton.setMargin(new Insets(3,3,3,3));
		addMissionButton.setToolTipText("Add New Mission");
		addMissionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				AddMissionCommand command = new AddMissionCommand(missionsTab, new Mission(missionsTab.getScenarioPanel().getScenario()));
				command.execute();
			}
		});
		buttonPanel.add(addMissionButton);
		editMissionButton = new JButton("Edit", new ImageIcon(getClass().getClassLoader().getResource("icons/application_edit.png")));
		editMissionButton.setMargin(new Insets(3,3,3,3));
		editMissionButton.setToolTipText("Edit Mission");
		editMissionButton.setEnabled(false);
		editMissionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EditMissionCommand command = new EditMissionCommand(missionsTab, missionTable.getMission(missionTable.getSelectedRow()));
				command.execute();
			}
		});
		buttonPanel.add(editMissionButton);
		copyMissionButton = new JButton("Copy", new ImageIcon(getClass().getClassLoader().getResource("icons/application_double.png")));
		copyMissionButton.setMargin(new Insets(3,3,3,3));
		copyMissionButton.setToolTipText("Copy Mission");
		copyMissionButton.setEnabled(false);
		copyMissionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CopyMissionCommand command = new CopyMissionCommand(missionsTab, missionTable.getMission(missionTable.getSelectedRow()));
				command.execute();
			}
		});
		buttonPanel.add(copyMissionButton);
		removeMissionButton = new JButton("Remove", new ImageIcon(getClass().getClassLoader().getResource("icons/application_delete.png")));
		removeMissionButton.setMargin(new Insets(3,3,3,3));
		removeMissionButton.setToolTipText("Remove Mission");
		removeMissionButton.setEnabled(false);
		removeMissionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ArrayList<Mission> missions = new ArrayList<Mission>();
				for(int row : missionTable.getSelectedRows()) {
					missions.add(missionTable.getMission(row));
				}
				RemoveMissionCommand command = new RemoveMissionCommand(missionsTab, missions);
				command.execute();
				missionsTab.updateView();
			}
		});
		buttonPanel.add(removeMissionButton);
		add(buttonPanel, c);
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		missionTable.updateView();
		updateButtons();
	}
	private void updateButtons() {
		if(missionTable.getSelectedRowCount() == 1) {
			copyMissionButton.setEnabled(true);
			editMissionButton.setEnabled(true);
			removeMissionButton.setEnabled(true);
		} else if(missionTable.getSelectedRowCount() > 1) {
			copyMissionButton.setEnabled(false);
			editMissionButton.setEnabled(false);
			removeMissionButton.setEnabled(true);
		} else {
			copyMissionButton.setEnabled(false);
			editMissionButton.setEnabled(false);
			removeMissionButton.setEnabled(false);
		}
	}
}