/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.mission;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;

import edu.mit.spacenet.domain.event.I_Event;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.scenario.Mission;
import edu.mit.spacenet.simulator.SimSpatialError;

/**
 * A table to display and edit missions.
 * 
 * @author Paul Grogan
 */
public class MissionTable extends JTable {
	private static final long serialVersionUID = -5960985748957372743L;
	
	private CustomTableModel model;
	private MissionsTab tab;
	
	/**
	 * Instantiates a new mission table.
	 * 
	 * @param tab the tab
	 */
	public MissionTable(MissionsTab tab) {
		super();
		this.tab = tab;
		
		model = new CustomTableModel(tab.getScenarioPanel().getScenario().getMissionList());
		this.setModel(model);
		DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
		renderer.setHorizontalAlignment(DefaultTableCellRenderer.CENTER);
		renderer.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		getColumnModel().getColumn(0).setHeaderValue("#");
		getColumnModel().getColumn(0).setCellRenderer(renderer);
		getColumnModel().getColumn(0).setMaxWidth(25);
		getColumnModel().getColumn(1).setHeaderValue("Start Date");
		getColumnModel().getColumn(1).setMaxWidth(100);
		getColumnModel().getColumn(2).setHeaderValue("Mission Name");
		getColumnModel().getColumn(2).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 7793722593255866883L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
				super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				setIcon(null);
				setIconTextGap(0);
				setToolTipText(null);
				for(SimSpatialError error : getTab().getSimulator().getSpatialErrors()) {
					for(I_Event event : getMission(row).getEventList()) {
						if(error.getEvent().getRoot().equals(event)) {
							setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/bullet_error.png")));
							setToolTipText("This mission has errors.");
							return this;
						}
					}
				}
				return this;
			}
		});
		//this.getColumnModel().getColumn(3).setHeaderValue("Dest.");
		//this.getColumnModel().getColumn(3).setCellRenderer(renderer);
		//this.getColumnModel().getColumn(3).setMaxWidth(50);
		this.setShowGrid(false);
		
		setBackground(Color.WHITE);
		getTableHeader().setReorderingAllowed(false);
		getTableHeader().setBackground(Color.WHITE);
		getTableHeader().setFont(getTableHeader().getFont().deriveFont(Font.BOLD));
		getTableHeader().setBorder(BorderFactory.createMatteBorder(0,0,1,0,Color.BLACK));
		getTableHeader().setOpaque(false);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JTable#getCellEditor(int, int)
	 */
	public TableCellEditor getCellEditor(int row, int col) {
    	if(col==3) {
    		JComboBox comboBox = new JComboBox();
    		for(Node node : tab.getScenarioPanel().getScenario().getNetwork().getNodes()) {
    			comboBox.addItem(node);
    		}
    		return new DefaultCellEditor(comboBox);
    	} else  {
    		return super.getCellEditor(row, col);
    	}
    }
	
	/**
	 * The Class CustomTableModel.
	 */
	class CustomTableModel extends AbstractTableModel {
		private static final long serialVersionUID = -678300273079091701L;
		private List<Mission> data;
		
		/**
		 * Instantiates a new custom table model.
		 * 
		 * @param data the data
		 */
		public CustomTableModel(List<Mission> data) {
			super();
			this.data = data;
		}
		
		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		public int getColumnCount() {
			return 3;
		}
		
		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		public int getRowCount() {
			return data.size();
		}
		
		/* (non-Javadoc)
		 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
		 */
		public boolean isCellEditable(int row, int col) {
	    	if(col==0) return false;
	    	else return true;
	    }
		
		/* (non-Javadoc)
		 * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
		 */
		public Class<?> getColumnClass(int col) {
			if(col==0) {
				return Integer.class;
			} else if(col==1) {
				return Date.class;
			} else {
				return String.class;
			}
	    }
		
		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		public Object getValueAt(int row, int col) {
			if(col==0) {
				return new Integer(row+1);
			} else if(col==1) {
				return data.get(row).getStartDate();
			} else if(col==3) {
				return data.get(row).getDestination();
			} else {
				return data.get(row).getName();
			}
		}
	    
    	/* (non-Javadoc)
    	 * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object, int, int)
    	 */
    	public void setValueAt(Object value, int row, int col) {
	    	if(col==1) {
	    		data.get(row).setStartDate((Date)value);
	    	} else if(col==3) {
	    		data.get(row).setDestination((Node)value);
	    	} else if(col==2) {
	    		data.get(row).setName((String)value);
	    	}
	    	Collections.sort(data);
	    	fireTableDataChanged();
	    	tab.updateView();
	    }
	    
    	/**
    	 * Gets the data.
    	 * 
    	 * @return the data
    	 */
    	public List<Mission> getData() {
	    	return data;
	    }
	    
    	/**
    	 * Removes the.
    	 * 
    	 * @param mission the mission
    	 */
    	public void remove(Mission mission) {
			data.remove(mission);
			fireTableDataChanged();
		}
	    
    	/**
    	 * Adds the.
    	 * 
    	 * @param mission the mission
    	 */
    	public void add(Mission mission) {
	    	data.add(mission);
	    	Collections.sort(data);
	    	fireTableDataChanged();
	    }
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		int selectedRow = getSelectedRow();
		model.fireTableDataChanged();
		if(getSelectedRow()<getRowCount());
			getSelectionModel().setSelectionInterval(selectedRow, selectedRow);
	}
	
	/**
	 * Gets the tab.
	 * 
	 * @return the tab
	 */
	public MissionsTab getTab() {
		return tab;
	}
	
	/**
	 * Gets the mission.
	 * 
	 * @param row the row
	 * 
	 * @return the mission
	 */
	public Mission getMission(int row) {
		return model.getData().get(row);
	}
	
	/**
	 * Gets the selected mission.
	 * 
	 * @return the selected mission
	 */
	public Mission getSelectedMission() {
		if(getSelectedRow()<0) return null;
		else return getMission(getSelectedRow());
	}
}
