/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.mission;

import java.awt.CardLayout;
import java.awt.Cursor;
import java.util.Collections;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jdesktop.swingworker.SwingWorker;

import edu.mit.spacenet.domain.event.I_Event;
import edu.mit.spacenet.gui.ScenarioPanel;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.scenario.Mission;
import edu.mit.spacenet.simulator.MiniSimulator;

/**
 * Tab component for editing missions in a scenario.
 * 
 * @author Paul Grogan
 */
public class MissionsTab extends JSplitPane {
	private static final long serialVersionUID = 4813576345903475004L;
	private static final String LIST_PANEL = "LIST_PANEL";
	private static final String MISSION_PANEL = "MISSION_PANEL";
	
	private JPanel controlPanel;
	
	private MissionListPanel listPanel;
	private MissionPanel missionPanel;
	
	private JTabbedPane chartTabs;
	private ProcessBatChart processBatChart, missionBatChart;
	private ElementBatChart elementBatChart;
	
	private ScenarioPanel scenarioPanel;
	private MiniSimulator simulator;
	
	/**
	 * Instantiates a new missions tab.
	 * 
	 * @param scenarioPanel the scenario panel
	 */
	public MissionsTab(ScenarioPanel scenarioPanel) {
		super();
		this.scenarioPanel = scenarioPanel;
		this.simulator = new MiniSimulator(getScenarioPanel().getScenario());
		
		buildPanel();
		updateView();
	}
	private void buildPanel() {
		controlPanel = new JPanel();
		controlPanel.setLayout(new CardLayout());
		listPanel = new MissionListPanel(this);
		controlPanel.add(listPanel, LIST_PANEL);
		missionPanel = new MissionPanel(this);
		controlPanel.add(missionPanel, MISSION_PANEL);
		setLeftComponent(new JScrollPane(controlPanel));
		chartTabs = new JTabbedPane();
		chartTabs.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if(chartTabs.getSelectedComponent().equals(processBatChart)) {
					processBatChart.updateView();
				} else if(chartTabs.getSelectedComponent().equals(elementBatChart)) {
					elementBatChart.updateView();
				}
			}
		});
		processBatChart = new ProcessBatChart(this);
		missionBatChart = new ProcessBatChart(this);
		chartTabs.add(processBatChart);
		elementBatChart = new ElementBatChart(this);
		chartTabs.add(elementBatChart);
		setRightComponent(chartTabs);
		setResizeWeight(.1);
		setDividerLocation(275);
		setBorder(BorderFactory.createEmptyBorder());
	}
	private boolean isUpdating = false;
	
	/**
	 * Update view.
	 */
	public void updateView() {
		if(isUpdating) return;
		isUpdating = true;
		SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
			public Void doInBackground() throws Exception {
				SpaceNetFrame.getSingleton().getStatusBar().setStatusMessage("Simulating...");
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				simulator.simulate();
				return null;
			}
			public void done() {
				if(missionPanel.getMission() == null) {
					Collections.sort(getScenarioPanel().getScenario().getMissionList());
					listPanel.updateView();
					if(chartTabs.getSelectedComponent().equals(processBatChart)) {
						processBatChart.updateView();
					} else if(chartTabs.getSelectedComponent().equals(elementBatChart)) {
						elementBatChart.updateView();
					}
				} else {
					missionPanel.updateView();
					missionBatChart.updateView();
				}
				SpaceNetFrame.getSingleton().getStatusBar().clearStatusMessage();
				setCursor(Cursor.getDefaultCursor());
				isUpdating = false;
			}
		};
		worker.execute();
	}
	
	/**
	 * Gets the mission.
	 * 
	 * @return the mission
	 */
	public Mission getMission() {
		return missionPanel.getMission();
	}
	
	/**
	 * Gets the scenario panel.
	 * 
	 * @return the scenario panel
	 */
	public ScenarioPanel getScenarioPanel() {
		return scenarioPanel;
	}
	
	/**
	 * Edits the mission.
	 * 
	 * @param mission the mission
	 */
	public void editMission(Mission mission) {
		if(!getScenarioPanel().getScenario().getMissionList().contains(mission)) {
			getScenarioPanel().getScenario().getMissionList().add(mission);
		}
		
		missionPanel.setMission(mission);
		
		((CardLayout)controlPanel.getLayout()).show(controlPanel, MISSION_PANEL);
		setRightComponent(missionBatChart);
		missionBatChart.toggleEditingMission();
		setDividerLocation(275);
		updateView();
	}
	
	/**
	 * Save mission.
	 * 
	 * @param mission the mission
	 */
	public void saveMission(Mission mission) {
		missionBatChart.toggleEditingMission();
		setRightComponent(chartTabs);
		
		missionPanel.setMission(null);
		((CardLayout)controlPanel.getLayout()).show(controlPanel, LIST_PANEL);
		setDividerLocation(275);
		updateView();
	}
	
	/**
	 * Contains.
	 * 
	 * @param event the event
	 * 
	 * @return true, if successful
	 */
	public boolean contains(I_Event event) {
		return getMission().getEventList().contains(event);
	}
	
	/**
	 * Adds the event.
	 * 
	 * @param event the event
	 */
	public void addEvent(I_Event event) {
		getMission().getEventList().add(event);
	}
	
	/**
	 * Gets the simulator.
	 * 
	 * @return the simulator
	 */
	public MiniSimulator getSimulator() {
		return simulator;
	}
}
