/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.mission;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;

import org.jdesktop.swingworker.SwingWorker;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.SymbolAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.time.Hour;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleInsets;

import com.toedter.calendar.JDateChooser;

import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.event.BurnEvent;
import edu.mit.spacenet.domain.event.CreateEvent;
import edu.mit.spacenet.domain.event.DemandEvent;
import edu.mit.spacenet.domain.event.EvaEvent;
import edu.mit.spacenet.domain.event.EventType;
import edu.mit.spacenet.domain.event.ExplorationProcess;
import edu.mit.spacenet.domain.event.FlightTransport;
import edu.mit.spacenet.domain.event.I_Event;
import edu.mit.spacenet.domain.event.MoveEvent;
import edu.mit.spacenet.domain.event.ReconfigureEvent;
import edu.mit.spacenet.domain.event.ReconfigureGroupEvent;
import edu.mit.spacenet.domain.event.RemoveEvent;
import edu.mit.spacenet.domain.event.SpaceTransport;
import edu.mit.spacenet.domain.event.SurfaceTransport;
import edu.mit.spacenet.domain.event.TransferEvent;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.CheckBoxTableModel;
import edu.mit.spacenet.gui.component.SNChartPanel;
import edu.mit.spacenet.scenario.Mission;
import edu.mit.spacenet.scenario.Scenario;
import edu.mit.spacenet.simulator.SimState;
import edu.mit.spacenet.util.DateFunctions;
import edu.mit.spacenet.util.SerializeUtil;

/**
 * A bat-style chart that displays scenario events.
 * 
 * @author Paul Grogan
 */
public class ProcessBatChart extends JSplitPane {
	private static final long serialVersionUID = -6405124067606236705L;
	private MissionsTab missionsTab;
	private JDateChooser startDate, endDate;
	private CheckBoxTableModel<Mission> missionsModel;
	private CheckBoxTableModel<EventType> eventsModel;
	private ChartPanel batChartPanel;
	private JCheckBox waitCheck;
	private JButton refreshButton;
	
	/**
	 * Instantiates a new process bat chart.
	 * 
	 * @param tab the missions tab
	 */
	public ProcessBatChart(MissionsTab tab) {
		this.missionsTab = tab;
		
		batChartPanel = new SNChartPanel();
		batChartPanel.setPreferredSize(new Dimension(500,300));
		setLeftComponent(batChartPanel);
		
		JPanel controlPanel = new JPanel();
		controlPanel.setBackground(Color.WHITE);
		controlPanel.setLayout(new GridBagLayout());
		controlPanel.setBorder(BorderFactory.createTitledBorder("Chart Options"));
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.BOTH;
		controlPanel.add(new JLabel("Start Date: "), c);
		c.gridy++;
		controlPanel.add(new JLabel("End Date: "), c);
		c.gridy--;
		c.gridx++;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		startDate = new JDateChooser();
		startDate.setSelectableDateRange(getScenario().getStartDate(), null);
		startDate.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if(e.getPropertyName().equals("date") && isAutoRefresh()) updateChart();
			}
		});
		controlPanel.add(startDate, c);
		c.gridy++;
		endDate = new JDateChooser();
		endDate.setSelectableDateRange(getScenario().getStartDate(), null);
		endDate.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if(e.getPropertyName().equals("date") && isAutoRefresh()) updateChart();
			}
		});
		controlPanel.add(endDate, c);
		c.gridx--;
		c.gridwidth = 2;
		c.gridy++;
		c.weighty = 1;
		missionsModel = new CheckBoxTableModel<Mission>();
		missionsModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				if(e.getType()==TableModelEvent.UPDATE && isAutoRefresh()) updateChart();
			}
		});
		JTable missionsTable = new JTable(missionsModel);
		missionsTable.getColumnModel().getColumn(0).setHeaderRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/eye.png")));
                setHorizontalAlignment(JLabel.CENTER);
                return this;
			}
		});
		missionsTable.getTableHeader().setReorderingAllowed(false);
		missionsTable.getTableHeader().setBackground(Color.WHITE);
		missionsTable.getTableHeader().setFont(missionsTable.getTableHeader().getFont().deriveFont(Font.BOLD));
		missionsTable.getTableHeader().setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		missionsTable.getTableHeader().setOpaque(false);
		missionsTable.getColumnModel().getColumn(0).setMaxWidth(25);
		missionsTable.getColumnModel().getColumn(1).setHeaderValue("Filter Missions");
		missionsTable.setShowGrid(false);
		JScrollPane missionsScroll = new JScrollPane(missionsTable);
		missionsScroll.setPreferredSize(new Dimension(150,100));
		missionsScroll.setBackground(Color.WHITE);
		missionsScroll.getViewport().setBackground(Color.WHITE);
		controlPanel.add(missionsScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel missionsButtonPanel = new JPanel();
		missionsButtonPanel.setOpaque(false);
		missionsButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllMissionsButton = new JButton("Select All");
		selectAllMissionsButton.setToolTipText("Select All Missions");
		selectAllMissionsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				missionsModel.selectAll();
			}
		});
		missionsButtonPanel.add(selectAllMissionsButton);
		JButton deselectAllMissionsButton = new JButton("Deselect All");
		deselectAllMissionsButton.setToolTipText("Deselect All Missions");
		deselectAllMissionsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				missionsModel.deselectAll();
			}
		});
		missionsButtonPanel.add(deselectAllMissionsButton);
		controlPanel.add(missionsButtonPanel, c);
		c.weightx = 0;
		c.fill = GridBagConstraints.BOTH;
		c.gridy++;
		JPanel prop1 = new JPanel();
		prop1.setOpaque(false);
		prop1.setPreferredSize(new Dimension(1,15));
		controlPanel.add(prop1, c);
		c.gridy++;
		c.weighty = 1;
		eventsModel = new CheckBoxTableModel<EventType>();
		for(EventType t : EventType.values()) {
			eventsModel.addObject(t);
		}
		eventsModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				if(e.getType()==TableModelEvent.UPDATE && isAutoRefresh()) updateChart();
			}
		});
		JTable eventsTable = new JTable(eventsModel);
		eventsTable.getColumnModel().getColumn(0).setHeaderRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                setIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/eye.png")));
                setHorizontalAlignment(JLabel.CENTER);
                return this;
			}
		});
		eventsTable.getColumnModel().getColumn(1).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if(value instanceof EventType)
                	setIcon(((EventType)value).getIcon());
                return this;
			}
		});
		eventsTable.setBackground(Color.WHITE);
		eventsTable.getTableHeader().setReorderingAllowed(false);
		eventsTable.getTableHeader().setBackground(Color.WHITE);
		eventsTable.getTableHeader().setFont(missionsTable.getTableHeader().getFont().deriveFont(Font.BOLD));
		eventsTable.getTableHeader().setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
		eventsTable.getTableHeader().setOpaque(false);
		eventsTable.getColumnModel().getColumn(0).setMaxWidth(25);
		eventsTable.getColumnModel().getColumn(1).setHeaderValue("Filter Event Types");
		eventsTable.setShowGrid(false);
		JScrollPane eventsScroll = new JScrollPane(eventsTable);
		eventsScroll.setBackground(Color.WHITE);
		eventsScroll.getViewport().setBackground(Color.WHITE);
		eventsScroll.setPreferredSize(new Dimension(150,100));
		controlPanel.add(eventsScroll, c);
		c.gridy++;
		c.weighty = 0;
		waitCheck = new JCheckBox("Draw Elements Waiting (Null Process)");
		waitCheck.setOpaque(false);
		waitCheck.setSelected(true);
		waitCheck.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(isAutoRefresh()) updateChart();
			}
		});
		controlPanel.add(waitCheck, c);
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		JPanel eventsButtonPanel = new JPanel();
		eventsButtonPanel.setOpaque(false);
		eventsButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllEventsButton = new JButton("Select All");
		selectAllEventsButton.setToolTipText("Select All Events");
		selectAllEventsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				eventsModel.selectAll();
				waitCheck.setSelected(true);
			}
		});
		eventsButtonPanel.add(selectAllEventsButton);
		JButton deselectAllEventsButton = new JButton("Deselect All");
		deselectAllEventsButton.setToolTipText("Deselect All Events");
		deselectAllEventsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				eventsModel.deselectAll();
				waitCheck.setSelected(false);
			}
		});
		eventsButtonPanel.add(deselectAllEventsButton);
		controlPanel.add(eventsButtonPanel, c);
		c.gridy++;
		JPanel prop2 = new JPanel();
		prop2.setOpaque(false);
		prop2.setPreferredSize(new Dimension(1,15));
		controlPanel.add(prop2, c);
		c.gridy++;
		c.anchor = GridBagConstraints.LINE_END;
		refreshButton = new JButton("Refresh", new ImageIcon(getClass().getClassLoader().getResource("icons/arrow_refresh.png")));
		refreshButton.setVisible(false);
		refreshButton.setMargin(new Insets(3,3,3,3));
		refreshButton.setToolTipText("Refresh Chart");
		refreshButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				refreshChart();
			}
		});
		controlPanel.add(refreshButton, c);
		
		setRightComponent(controlPanel);
		
		setName("Process Bat Chart");
		setOneTouchExpandable(true);
		setDividerSize(10);
		setBorder(BorderFactory.createEmptyBorder());
		setResizeWeight(1);
		setDividerLocation(490);
	}
	
	/**
	 * Gets the scenario.
	 * 
	 * @return the scenario
	 */
	public Scenario getScenario() {
		return missionsTab.getScenarioPanel().getScenario();
	}
	private boolean isEditingMission = false;
	
	/**
	 * Toggle editing mission.
	 */
	public void toggleEditingMission() {
		isEditingMission = !isEditingMission;
		if(isEditingMission) {
			setDividerSize(0);
			setDividerLocation(5000);
		} else {
			setDividerLocation(getLastDividerLocation());
			setDividerSize(10);
		}
	}
	private boolean isAutoRefresh() {
		return SpaceNetFrame.getSingleton().getPreferences().isAutoRefresh();
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		if(isAutoRefresh()) {
			refreshButton.setVisible(false);
			refreshChart();
		} else {
			refreshButton.setVisible(true);
		}
	}
	private boolean isUpdating = false;
	private void refreshChart() {
		if(isUpdating) return;
		isUpdating = true;
		SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
			public Void doInBackground() throws Exception {
				SpaceNetFrame.getSingleton().getStatusBar().setStatusMessage("Refreshing Chart...");
				missionsTab.getScenarioPanel().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				List<Mission> deselectedMissions = missionsModel.getDeselectedObjects();
				missionsModel.clear();
				for(Mission mission : missionsTab.getScenarioPanel().getScenario().getMissionList()) {
					missionsModel.addObject(mission, !deselectedMissions.contains(mission));
				}
				updateChart();
				return null;
			}
			public void done() {
				SpaceNetFrame.getSingleton().getStatusBar().clearStatusMessage();
				missionsTab.getScenarioPanel().setCursor(Cursor.getDefaultCursor());
				isUpdating = false;
			}
		};
		worker.execute();
	}
	private void updateChart() {
		TimeSeriesCollection dataset = new TimeSeriesCollection();
		
		JFreeChart chart = ChartFactory.createTimeSeriesChart(
				null, 
				"Date", 
				"Node", 
				dataset, 
				false, 
				true, 
				false);
		
		XYPlot plot = (XYPlot) chart.getPlot();
		SymbolAxis rangeAxis = createRangeAxis();
		plot.setRangeAxis(rangeAxis);
		plot.setBackgroundPaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.white);
		plot.setRangeGridlinePaint(Color.white);
		plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));

		plot.setRangeGridlinesVisible(false);
		plot.setDomainGridlinesVisible(false);

		XYItemRenderer r = chart.getXYPlot().getRenderer();
		chart.setBackgroundPaint(Color.white);

		if (r instanceof XYLineAndShapeRenderer) {
			XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
			renderer.setBaseShapesVisible(true);
			renderer.setBaseShapesFilled(true);
		}
		
		List<Mission> missions;
		if(isEditingMission) {
			missions = new ArrayList<Mission>();
			if(missionsTab.getMission() != null) {
				Mission mission = (Mission)SerializeUtil.deepClone(missionsTab.getMission());
				mission.getEventList().clear();
				for(I_Event e : missionsTab.getMission().getEventList()) {
					mission.getEventList().add(e);
				}
				missions.add(mission);
			}
		} else {
			missions = missionsModel.getSelectedObjects();
		}
		
		for(Mission mission : missions) {
			for(I_Event event : mission.getEventList()) {
				if((startDate.getDate()==null||DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime()).compareTo(startDate.getDate())>=0) 
						&& (endDate.getDate()==null||DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime()).compareTo(endDate.getDate())<=0)
						&& event.getLocation() instanceof Node) {
					 if(event instanceof SpaceTransport 
							 && ((SpaceTransport)event).getEdge() != null
							 && eventsModel.getSelectedObjects().contains(EventType.SPACE_TRANSPORT)) {
						TimeSeries s = new TimeSeries(event.getName(), Hour.class);
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
								node2Number(((SpaceTransport) event).getEdge().getOrigin(), rangeAxis));
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime() + ((SpaceTransport)event).getEdge().getDuration())), 
								node2Number(((SpaceTransport) event).getEdge().getDestination(), rangeAxis));
						((TimeSeriesCollection)dataset).addSeries(s);
						r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.RED);
						r.setSeriesShape(dataset.getSeriesCount() - 1, 
								new Ellipse2D.Double(-2.0, -2.0, 4.0, 4.0));
					} else if(event instanceof SurfaceTransport 
							&& ((SurfaceTransport)event).getEdge() != null
							&& eventsModel.getSelectedObjects().contains(EventType.SURFACE_TRANSPORT)) {
						TimeSeries s = new TimeSeries(event.getName(), Hour.class);
						if(((SurfaceTransport)event).isReversed()) {
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number(((SurfaceTransport) event).getEdge().getDestination(), rangeAxis));
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime() + ((SurfaceTransport)event).getDuration())), 
									node2Number(((SurfaceTransport) event).getEdge().getOrigin(), rangeAxis));
						} else {
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number(((SurfaceTransport) event).getEdge().getOrigin(), rangeAxis));
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime() + ((SurfaceTransport)event).getDuration())), 
									node2Number(((SurfaceTransport) event).getEdge().getDestination(), rangeAxis));
						}
						((TimeSeriesCollection)dataset).addSeries(s);
						r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.GREEN);
						r.setSeriesShape(dataset.getSeriesCount() - 1, 
								new Ellipse2D.Double(-2.0, -2.0, 4.0, 4.0));
					} else if(event instanceof FlightTransport 
							&& ((FlightTransport)event).getEdge() != null
							&& eventsModel.getSelectedObjects().contains(EventType.FLIGHT_TRANSPORT)) {
						TimeSeries s = new TimeSeries(event.getName(), Hour.class);
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
								node2Number(((FlightTransport) event).getEdge().getOrigin(), rangeAxis));
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime() + ((FlightTransport)event).getEdge().getDuration())), 
								node2Number(((FlightTransport) event).getEdge().getDestination(), rangeAxis));
						((TimeSeriesCollection)dataset).addSeries(s);
						r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.YELLOW);
						r.setSeriesShape(dataset.getSeriesCount() - 1, 
								new Ellipse2D.Double(-2.0, -2.0, 4.0, 4.0));
					} else if(event instanceof ExplorationProcess 
							&& event.getLocation() != null
							&& eventsModel.getSelectedObjects().contains(EventType.EXPLORATION)) {
						TimeSeries s = new TimeSeries(event.getName(), Hour.class);
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
								node2Number((Node)event.getLocation(), rangeAxis));
						s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime() + ((ExplorationProcess)event).getDuration())), 
								node2Number((Node)event.getLocation(), rangeAxis));
						((TimeSeriesCollection)dataset).addSeries(s);
						r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.CYAN);
						r.setSeriesShape(dataset.getSeriesCount() - 1, 
								new Ellipse2D.Double(-2.0, -2.0, 4.0, 4.0));
					} else if(event.getLocation() != null){
						if(event instanceof CreateEvent
								&& eventsModel.getSelectedObjects().contains(EventType.CREATE)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.GREEN);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						} else if(event instanceof MoveEvent
								&& eventsModel.getSelectedObjects().contains(EventType.MOVE)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.ORANGE);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						} else if(event instanceof TransferEvent
								&& eventsModel.getSelectedObjects().contains(EventType.TRANSFER)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.ORANGE);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						} else if(event instanceof RemoveEvent
								&& eventsModel.getSelectedObjects().contains(EventType.REMOVE)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.BLACK);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						} else if(event instanceof DemandEvent
								&& eventsModel.getSelectedObjects().contains(EventType.DEMAND)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.BLUE);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						} else if(event instanceof ReconfigureEvent
								&& eventsModel.getSelectedObjects().contains(EventType.RECONFIGURE)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.PINK);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						} else if(event instanceof ReconfigureGroupEvent
								&& eventsModel.getSelectedObjects().contains(EventType.RECONFIGURE_GROUP)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.PINK);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						} else if(event instanceof BurnEvent
								&& eventsModel.getSelectedObjects().contains(EventType.BURN)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.RED);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						} else if(event instanceof EvaEvent
								&& eventsModel.getSelectedObjects().contains(EventType.EVA)) {
							TimeSeries s = new TimeSeries(event.getName(), Hour.class);
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime())), 
									node2Number((Node)event.getLocation(), rangeAxis));
							s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), DateFunctions.getDaysBetween(getScenario().getStartDate(), mission.getStartDate()) + event.getTime() + ((EvaEvent)event).getEvaDuration()/24)), 
									node2Number((Node)event.getLocation(), rangeAxis));
							((TimeSeriesCollection)dataset).addSeries(s);
							r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.CYAN);
							r.setSeriesShape(dataset.getSeriesCount() - 1, 
									new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0));
						}
					}
				}
			}
		}
		if(!isEditingMission&&waitCheck.isSelected()) {
			Mission firstMission = missionsModel.getSelectedObjects().size()>0?missionsModel.getSelectedObjects().get(0):null;
			double minTime1 = firstMission==null?Double.MAX_VALUE:DateFunctions.getDaysBetween(getScenario().getStartDate(), firstMission.getStartDate());
			Mission lastMission = missionsModel.getSelectedObjects().size()>0?missionsModel.getSelectedObjects().get(missionsModel.getSelectedObjects().size()-1):null;
			double maxTime1 = lastMission==null?0:DateFunctions.getDaysBetween(getScenario().getStartDate(), DateFunctions.getDate(lastMission.getStartDate(), lastMission.getDuration()));

			double minTime2 = startDate.getDate()==null?0:DateFunctions.getDaysBetween(missionsTab.getScenarioPanel().getScenario().getStartDate(), startDate.getDate());
			double maxTime2 = endDate.getDate()==null?Double.MAX_VALUE:DateFunctions.getDaysBetween(missionsTab.getScenarioPanel().getScenario().getStartDate(), endDate.getDate());
			
			double minTime = Math.max(minTime1, minTime2);
			double maxTime = Math.min(maxTime1, maxTime2);
			
			
			for(I_Element element : getScenario().getElements()) {
				Node prevNode = null;
				List<TimeSeries> series = new ArrayList<TimeSeries>();
				int i = -1;
				for(SimState state : missionsTab.getSimulator().getLocationHistory()) {
					if(prevNode!=null && prevNode.equals(state.getLocation(element))) {
						series.get(i).addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), Math.min(Math.max(minTime, state.getTime()), maxTime))), 
										node2Number((Node)state.getLocation(element), rangeAxis));
					} else if(state.getLocation(element) instanceof Node) {
						prevNode = (Node)state.getLocation(element);
						series.add(new TimeSeries(element + "waiting at " + prevNode + " (" + i + ")"));
						i++;
						series.get(i).addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), Math.min(Math.max(minTime, state.getTime()), maxTime))), 
								node2Number((Node)state.getLocation(element), rangeAxis));
					} else {
						prevNode = null;
					}
				}
				for(TimeSeries s : series) {
					((TimeSeriesCollection)dataset).addSeries(s);
					r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.GRAY);
					r.setSeriesShape(dataset.getSeriesCount() - 1, 
							new Ellipse2D.Double(0, 0, 0, 0));
				}
			}
			
			/*
			for(Node n : getScenario().getNetwork().getNodes()) {
				TimeSeries s = new TimeSeries("Wait at " + n, Hour.class);
				SortedSet<I_Element> prevElements = new TreeSet<I_Element>();
				double prevTime = 0;
				
				for(SimState state : tab.getSimulator().getLocationHistory()) {
					if(state.getTime() > prevTime) {
						for(I_Element e : state.getElements()) {
							if(state.getLocation(e).equals(n)
									&& prevElements.contains(e)) {
								s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), Math.min(Math.max(minTime, prevTime), maxTime))), 
										node2Number(n, rangeAxis));
								s.addOrUpdate(new Hour(DateFunctions.getDate(getScenario().getStartDate(), Math.min(Math.max(minTime, state.getTime()), maxTime))), 
										node2Number(n, rangeAxis));
								break;
							}
						}
					}
					prevElements.clear();
					for(I_Element e : state.getElements()) {
						if(state.getLocation(e).equals(n)) prevElements.add(e);
					}
					prevTime = state.getTime();
				}
				((TimeSeriesCollection)dataset).addSeries(s);
				r.setSeriesPaint(dataset.getSeriesCount() - 1, Color.GRAY);
				r.setSeriesShape(dataset.getSeriesCount() - 1, 
						new Ellipse2D.Double(0, 0, 0, 0));
			}*/
		}
		
		batChartPanel.setChart(chart);
	}
	private SymbolAxis createRangeAxis() {
		Vector<String> nodeLabels = new Vector<String>();
		for (Node n : missionsTab.getScenarioPanel().getScenario().getNetwork().getNodes()) {
			nodeLabels.add(n.getName());
		}
		String[] array = {};
		return new SymbolAxis(null, nodeLabels.toArray(array));
	}
	private int node2Number(Node node, SymbolAxis axis) {
		for (int i = 0; i < axis.getSymbols().length; i++) {
			if (axis.getSymbols()[i].equals(node.getName())) {
				return i;
			}
		}
		return -1;
	}
	
}
