/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.model;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import edu.mit.spacenet.domain.model.I_DemandModel;
import edu.mit.spacenet.gui.element.StateDialog;
import edu.mit.spacenet.gui.mission.MissionsTab;

/**
 * The dialog for viewing and editing demand models.
 * 
 * @author Paul Grogan
 */
public class DemandModelDialog extends JDialog {
	private static final long serialVersionUID = 5836025162293791120L;
	
	private StateDialog stateDialog;
	private MissionsTab missionsTab;
	private I_DemandModel demandModel;
	
	private JTextField nameText;
	private AbstractDemandModelPanel demandModelPanel;
	private JButton okButton;
	
	/**
	 * Instantiates a new demand model dialog.
	 * 
	 * @param stateDialog the state dialog
	 * @param demandModel the demand model
	 */
	public DemandModelDialog(StateDialog stateDialog, I_DemandModel demandModel) {
		super(stateDialog, demandModel.getDemandModelType().getName());
		this.stateDialog = stateDialog;
		this.demandModel = demandModel;
		buildDialog();
		initialize();
	}
	
	/**
	 * Instantiates a new demand model dialog.
	 * 
	 * @param missionsTab the missions tab
	 * @param demandModel the demand model
	 */
	public DemandModelDialog(MissionsTab missionsTab, I_DemandModel demandModel) {
		super(missionsTab.getScenarioPanel().getSpaceNetFrame(), demandModel.getDemandModelType().getName());
		this.missionsTab = missionsTab;
		this.demandModel = demandModel;
		buildDialog();
		initialize();
	}
	private void buildDialog() {
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		JPanel contentPanel = new JPanel();
		contentPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		contentPanel.setLayout(new GridBagLayout());
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		contentPanel.add(new JLabel("Name: "), c);
		
		c.gridx++;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		nameText = new JTextField(20);
		contentPanel.add(nameText, c);
		
		c.gridy++;
		c.gridx = 0;
		c.weighty = 1;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.CENTER;
		c.fill = GridBagConstraints.BOTH;
		demandModelPanel = DemandModelPanelFactory.createDemandModelPanel(this, demandModel);
		contentPanel.add(demandModelPanel, c);
		
		c.gridy++;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 0));
		contentPanel.add(buttonPanel, c);

		okButton = new JButton("OK", new ImageIcon(getClass().getClassLoader().getResource("icons/comment_go.png")));
		okButton.setMargin(new Insets(3,3,3,3));
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveDemandModel();
				if(missionsTab != null) {
					missionsTab.updateView();
				} else if(stateDialog != null) {
					stateDialog.updateView();
				}
				dispose();
			}
		});
		buttonPanel.add(okButton);
		JButton cancelButton = new JButton("Cancel", new ImageIcon(getClass().getClassLoader().getResource("icons/comment_delete.png")));
		cancelButton.setMargin(new Insets(3,3,3,3));
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		buttonPanel.add(cancelButton);

		getRootPane().setDefaultButton(okButton);
		setContentPane(contentPanel);
		setMinimumSize(new Dimension(300,150));
		setModal(true);
	}
	private void initialize() {
		nameText.setText(demandModel.getName());
	}
	private void saveDemandModel() {
		demandModel.setName(nameText.getText());
		demandModelPanel.saveDemandModel();
		if(missionsTab != null) {
			if(!missionsTab.getMission().getDemandModels().contains(demandModel)) {
				missionsTab.getMission().getDemandModels().add(demandModel);
			}
		} else if(stateDialog != null) {
			if(!stateDialog.containsDemandModel(demandModel)) {
				stateDialog.addDemandModel(demandModel);
			}
		}
	}
	
	/**
	 * Gets the missions tab.
	 * 
	 * @return the missions tab
	 */
	public MissionsTab getMissionsTab() {
		return missionsTab;
	}
	
	/**
	 * Gets the state dialog.
	 * 
	 * @return the state dialog
	 */
	public StateDialog getStateDialog() {
		return stateDialog;
	}
	
	/**
	 * Creates the and show gui.
	 * 
	 * @param missionsTab the missions tab
	 * @param demandModel the demand model
	 */
	public static void createAndShowGUI(MissionsTab missionsTab, I_DemandModel demandModel) {
		DemandModelDialog dialog = new DemandModelDialog(missionsTab, demandModel);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setVisible(true);
	}
	
	/**
	 * Creates the and show gui.
	 * 
	 * @param stateDialog the state dialog
	 * @param demandModel the demand model
	 */
	public static void createAndShowGUI(StateDialog stateDialog, I_DemandModel demandModel) {
		DemandModelDialog dialog = new DemandModelDialog(stateDialog, demandModel);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setVisible(true);
	}
}
