/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.network;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.mit.spacenet.domain.network.edge.Edge;
import edu.mit.spacenet.domain.network.edge.EdgeType;
import edu.mit.spacenet.domain.network.edge.FlightEdge;
import edu.mit.spacenet.domain.network.edge.SpaceEdge;
import edu.mit.spacenet.domain.network.edge.SurfaceEdge;
import edu.mit.spacenet.domain.network.node.LagrangeNode;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.domain.network.node.NodeType;
import edu.mit.spacenet.domain.network.node.OrbitalNode;
import edu.mit.spacenet.domain.network.node.SurfaceNode;
import edu.mit.spacenet.gui.ScenarioPanel;
import edu.mit.spacenet.gui.command.FilterNetworkCommand;
import edu.mit.spacenet.gui.component.SearchTextField;
import edu.mit.spacenet.gui.visualization.NetworkPanel;
import edu.mit.spacenet.scenario.Scenario;
import edu.mit.spacenet.scenario.ScenarioType;

/**
 * The tab component to view and edit the scenario network.
 * 
 * @author Paul Grogan
 */
public class NetworkTab extends JSplitPane {
	private static final long serialVersionUID = 5930409573819745343L;
	
	private JComboBox scenarioTypeCombo, nodeTypeCombo, edgeTypeCombo;
	private SearchTextField nodeSearchText, edgeSearchText;
	private NodeTable nodeTable;
	private EdgeTable edgeTable;
	private NetworkPanel networkPanel;
	
	private ScenarioPanel scenarioPanel;
	
	private JPanel infoPanel, nodePanel, edgePanel;
	private JLabel nodeTypeLabel, nodeDescriptionLabel, nodeBodyLabel;
	private JLabel nodeLatitudeLabel, nodeLongitudeLabel;
	private JLabel nodePeriapsisLabel, nodeApoapsisLabel, nodeInclinationLabel;
	private JLabel edgeTypeLabel, edgeDescriptionLabel, edgeOriginLabel, edgeDestinationLabel;
	private JLabel edgeDurationLabel;
	private JLabel edgeDistanceLabel;
	private JLabel edgeMaxCrewSizeLabel, edgeMaxCargoMassLabel;
	
	/**
	 * Instantiates a new network tab.
	 * 
	 * @param gui the gui
	 */
	public NetworkTab(ScenarioPanel gui) {
		this.scenarioPanel = gui;
		
		JPanel controlPanel = new JPanel();
		controlPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		controlPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.NONE;
		controlPanel.add(new JLabel("Scenario Type: "), c);
		c.gridx++;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		scenarioTypeCombo = new JComboBox();
		for(ScenarioType type : ScenarioType.values()) {
			scenarioTypeCombo.addItem(type);
		}
		scenarioTypeCombo.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				setIcon(((ScenarioType)value).getIcon());
				return this;
			}
		});
		scenarioTypeCombo.setSelectedItem(scenarioPanel.getScenario().getScenarioType());
		scenarioTypeCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				scenarioPanel.getScenario().setScenarioType((ScenarioType)scenarioTypeCombo.getSelectedItem());
				FilterNetworkCommand command = new FilterNetworkCommand(scenarioPanel);
				command.execute();
			}
		});
		controlPanel.add(scenarioTypeCombo, c);
		c.gridx = 0;
		c.gridy++;
		c.gridwidth = 2;
		controlPanel.add(new JLabel("Nodes: "), c);
		c.gridy++;
		JPanel nodeSearchPanel = new JPanel();
		nodeSearchPanel.setLayout(new BoxLayout(nodeSearchPanel, BoxLayout.LINE_AXIS));
		nodeSearchText = new SearchTextField("Enter Search Term");
		nodeSearchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent e) {
				nodeTable.updateView();
			}
		});
		nodeSearchPanel.add(nodeSearchText);
		nodeTypeCombo = new JComboBox();
		nodeTypeCombo.addItem("All");
		for(NodeType type : NodeType.values()) {
			nodeTypeCombo.addItem(type);
		}
		nodeTypeCombo.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				if(value instanceof NodeType) setIcon(((NodeType)value).getIcon());
				return this;
			}
		});
		nodeTypeCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				nodeTable.updateView();
			}
		});
		nodeSearchPanel.add(nodeTypeCombo);
		controlPanel.add(nodeSearchPanel, c);
		c.gridy++;
		c.weighty = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		nodeTable = new NodeTable(this);
		nodeTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if(nodeTable.getSelectedRow()>=0) {
					networkPanel.getNetworkLayer().getSelectedEdges().clear();
					networkPanel.getNetworkLayer().getSelectedNodes().clear();
					for(int row : nodeTable.getSelectedRows()) {
						networkPanel.getNetworkLayer().getSelectedNodes().add((Integer)nodeTable.getModel().getValueAt(row, 2));
					}
					networkPanel.repaint();
					
					edgeTable.clearSelection();
					updateInfoPanel();
				}
			}
		});
		JScrollPane nodeScroll = new JScrollPane(nodeTable);
		nodeScroll.setBackground(Color.WHITE);
		nodeScroll.getViewport().setBackground(Color.WHITE);
		nodeScroll.setPreferredSize(new Dimension(150,150));
		controlPanel.add(nodeScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel nodeButtonPanel = new JPanel();
		nodeButtonPanel.setOpaque(false);
		nodeButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllNodesButton = new JButton("Select All");
		selectAllNodesButton.setToolTipText("Select All Nodes");
		selectAllNodesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				nodeTable.selectAll();
			}
		});
		nodeButtonPanel.add(selectAllNodesButton);
		JButton deselectAllNodesButton = new JButton("Deselect All");
		deselectAllNodesButton.setToolTipText("Deselect All Nodes");
		deselectAllNodesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				nodeTable.deselectAll();
			}
		});
		nodeButtonPanel.add(deselectAllNodesButton);
		controlPanel.add(nodeButtonPanel, c);
		c.gridy++;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		controlPanel.add(new JLabel("Edges: "), c);
		c.gridy++;
		JPanel edgeSearchPanel = new JPanel();
		edgeSearchPanel.setLayout(new BoxLayout(edgeSearchPanel, BoxLayout.LINE_AXIS));
		edgeSearchText = new SearchTextField("Enter Search Term");
		edgeSearchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent e) {
				edgeTable.updateView();
			}
		});
		edgeSearchPanel.add(edgeSearchText);
		edgeTypeCombo = new JComboBox();
		edgeTypeCombo.addItem("All");
		for(EdgeType type : EdgeType.values()) {
			if(type!=EdgeType.TIME_DEPENDENT)
				edgeTypeCombo.addItem(type);
		}
		edgeTypeCombo.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				if(value instanceof EdgeType) setIcon(((EdgeType)value).getIcon());
				return this;
			}
		});
		edgeTypeCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				edgeTable.updateView();
			}
		});
		edgeSearchPanel.add(edgeTypeCombo);
		controlPanel.add(edgeSearchPanel, c);
		c.gridy++;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.LINE_START;
		edgeTable = new EdgeTable(this);
		edgeTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if(edgeTable.getSelectedRow()>=0) {
					networkPanel.getNetworkLayer().getSelectedEdges().clear();
					networkPanel.getNetworkLayer().getSelectedNodes().clear();
					for(int row : edgeTable.getSelectedRows()) {
						networkPanel.getNetworkLayer().getSelectedEdges().add(((Edge)edgeTable.getModel().getValueAt(row, 1)).getTid());
					}
					networkPanel.repaint();
					
					nodeTable.clearSelection();
					updateInfoPanel();
				}
			}
		});
		JScrollPane edgeScroll = new JScrollPane(edgeTable);
		edgeScroll.setBackground(Color.WHITE);
		edgeScroll.getViewport().setBackground(Color.WHITE);
		edgeScroll.setPreferredSize(new Dimension(150,150));
		controlPanel.add(edgeScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel edgeButtonPanel = new JPanel();
		edgeButtonPanel.setOpaque(false);
		edgeButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JButton selectAllEdgesButton = new JButton("Select All");
		selectAllEdgesButton.setToolTipText("Select All Edges");
		selectAllEdgesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				edgeTable.selectAll();
			}
		});
		edgeButtonPanel.add(selectAllEdgesButton);
		JButton deselectAllEdgesButton = new JButton("Deselect All");
		deselectAllEdgesButton.setToolTipText("Deselect All Edges");
		deselectAllEdgesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				edgeTable.deselectAll();
			}
		});
		edgeButtonPanel.add(deselectAllEdgesButton);
		controlPanel.add(edgeButtonPanel, c);
		setLeftComponent(new JScrollPane(controlPanel));
		
		JSplitPane rightPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		networkPanel = new NetworkPanel(getScenario().getNetwork());
		networkPanel.setPreferredSize(new Dimension(600,350));
		rightPanel.setTopComponent(new JScrollPane(networkPanel));
		
		buildInfoPanel();
		
		rightPanel.setBottomComponent(infoPanel);
		rightPanel.setDividerLocation(400);
		rightPanel.setResizeWeight(1);
		rightPanel.setBorder(BorderFactory.createEmptyBorder());
		
		setRightComponent(rightPanel);
		setResizeWeight(0);
		setDividerLocation(210);
		setBorder(BorderFactory.createEmptyBorder());
		
		updateView();
	}
	private void buildInfoPanel() {
		infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.LINE_AXIS));
		nodePanel = new JPanel();
		edgePanel = new JPanel();
		infoPanel.add(nodePanel);
		infoPanel.add(edgePanel);
		
		int xLabel = 74;
		int yLabel = 12;
		
		// node panel
		nodePanel.setVisible(true);
		nodePanel.setPreferredSize(new Dimension(400, 140));
		nodePanel.setBorder(BorderFactory.createTitledBorder("Node Information"));
		nodePanel.setLayout(new GridBagLayout());
		
		nodeTypeLabel = new JLabel("--");
		nodeDescriptionLabel = new JLabel("--");
		nodeBodyLabel = new JLabel("--");
		nodeLatitudeLabel = new JLabel("--");
		nodeLongitudeLabel = new JLabel("--");
		nodePeriapsisLabel = new JLabel("--");
		nodeApoapsisLabel = new JLabel("--");
		nodeInclinationLabel = new JLabel("--");
		
		GridBagConstraints cnode = new GridBagConstraints();
		cnode.insets = new Insets(2,2,2,2);
		
		cnode.gridy = 0;
		cnode.gridx = 0;
		cnode.gridwidth = 2;
		cnode.gridheight = 1;
		cnode.weightx = 20;
		cnode.weighty = 100;
		cnode.fill = GridBagConstraints.NONE;
		cnode.anchor = GridBagConstraints.EAST;
		nodePanel.add(new JLabel("Description: "), cnode);
		cnode.gridx = 2;
		cnode.gridwidth = 8;
		cnode.weightx = 80;
		cnode.anchor = GridBagConstraints.WEST;
		nodeDescriptionLabel.setPreferredSize(new Dimension(4*xLabel,yLabel));
		nodePanel.add(nodeDescriptionLabel, cnode);
		
		cnode.gridy++;
		cnode.gridx = 0;
		cnode.gridwidth = 2;
		cnode.weightx = 20;
		cnode.anchor = GridBagConstraints.EAST;
		nodePanel.add(new JLabel("Type: "), cnode);
		cnode.gridx = 2;
		cnode.gridwidth = 8;
		cnode.weightx = 80;
		cnode.anchor = GridBagConstraints.WEST;
		nodeTypeLabel.setPreferredSize(new Dimension(2*xLabel,yLabel));
		nodePanel.add(nodeTypeLabel, cnode);
		
		cnode.gridy++;
		cnode.gridx = 0;
		cnode.gridwidth = 2;
		cnode.gridheight = 1;
		cnode.weightx = 20;
		cnode.weighty = 100;
		cnode.fill = GridBagConstraints.NONE;
		cnode.anchor = GridBagConstraints.EAST;
		nodePanel.add(new JLabel("Body: "), cnode);
		cnode.gridx = 2;
		cnode.gridwidth = 8;
		cnode.weightx = 80;
		cnode.anchor = GridBagConstraints.WEST;
		nodeBodyLabel.setPreferredSize(new Dimension(2*xLabel,yLabel));
		nodePanel.add(nodeBodyLabel, cnode);
		
		cnode.gridy++;
		cnode.gridx = 0;
		cnode.gridwidth = 2;
		cnode.weightx = 20;
		cnode.anchor = GridBagConstraints.EAST;
		nodePanel.add(new JLabel("Latitude: "), cnode);
		cnode.gridx = 2;
		cnode.gridwidth = 3;
		cnode.weightx = 30;
		cnode.anchor = GridBagConstraints.WEST;
		nodeLatitudeLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		nodePanel.add(nodeLatitudeLabel, cnode);
		cnode.gridx = 5;
		cnode.gridwidth = 2;
		cnode.weightx = 20;
		cnode.anchor = GridBagConstraints.EAST;
		nodePanel.add(new JLabel("Periapsis: "), cnode);
		cnode.gridx = 7;
		cnode.gridwidth = 3;
		cnode.weightx = 30;
		cnode.anchor = GridBagConstraints.WEST;
		nodePeriapsisLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		nodePanel.add(nodePeriapsisLabel, cnode);
		
		cnode.gridy++;
		cnode.gridx = 0;
		cnode.gridwidth = 2;
		cnode.weightx = 20;
		cnode.anchor = GridBagConstraints.EAST;
		nodePanel.add(new JLabel("Longitude: "), cnode);
		cnode.gridx = 2;
		cnode.gridwidth = 3;
		cnode.weightx = 30;
		cnode.anchor = GridBagConstraints.WEST;
		nodeLongitudeLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		nodePanel.add(nodeLongitudeLabel, cnode);
		cnode.gridx = 5;
		cnode.gridwidth = 2;
		cnode.weightx = 20;
		cnode.anchor = GridBagConstraints.EAST;
		nodePanel.add(new JLabel("Apoapsis: "), cnode);
		cnode.gridx = 7;
		cnode.gridwidth = 3;
		cnode.weightx = 30;
		cnode.anchor = GridBagConstraints.WEST;
		nodeApoapsisLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		nodePanel.add(nodeApoapsisLabel, cnode);
		
		cnode.gridy++;
		cnode.gridx = 5;
		cnode.gridwidth = 2;
		cnode.weightx = 20;
		cnode.fill = GridBagConstraints.NONE;
		cnode.anchor = GridBagConstraints.EAST;
		nodePanel.add(new JLabel("Inclination: "), cnode);
		cnode.gridx = 7;
		cnode.gridwidth = 3;
		cnode.weightx = 30;
		cnode.anchor = GridBagConstraints.WEST;
		nodeInclinationLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		nodePanel.add(nodeInclinationLabel, cnode);
		
		// edge panel
		edgePanel.setVisible(true);
		edgePanel.setPreferredSize(new Dimension(400, 140));
		edgePanel.setBorder(BorderFactory.createTitledBorder("Edge Information"));
		edgePanel.setLayout(new GridBagLayout());
		
		edgeTypeLabel = new JLabel("--");
		edgeDescriptionLabel = new JLabel("--");
		edgeOriginLabel = new JLabel("--");
		edgeDestinationLabel = new JLabel("--");
		edgeDurationLabel = new JLabel("--");
		edgeDistanceLabel = new JLabel("--");
		edgeMaxCrewSizeLabel = new JLabel("--");
		edgeMaxCargoMassLabel = new JLabel("--");
		
		GridBagConstraints cedge = new GridBagConstraints();
		cedge.insets = new Insets(2,2,2,2);
		
		cedge.gridy = 0;
		cedge.gridx = 0;
		cedge.gridwidth = 2;
		cedge.gridheight = 1;
		cedge.weightx = 20;
		cedge.weighty = 100;
		cedge.fill = GridBagConstraints.NONE;
		cedge.anchor = GridBagConstraints.EAST;
		edgePanel.add(new JLabel("Description: "), cedge);
		cedge.gridx = 2;
		cedge.gridwidth = 8;
		cedge.weightx = 80;
		cedge.anchor = GridBagConstraints.WEST;
		edgeDescriptionLabel.setPreferredSize(new Dimension(4*xLabel,yLabel));
		edgePanel.add(edgeDescriptionLabel, cedge);
		
		cedge.gridy++;
		cedge.gridx = 0;
		cedge.gridwidth = 2;
		cedge.weightx = 20;
		cedge.anchor = GridBagConstraints.EAST;
		edgePanel.add(new JLabel("Type: "), cedge);
		cedge.gridx = 2;
		cedge.gridwidth = 8;
		cedge.weightx = 80;
		cedge.anchor = GridBagConstraints.WEST;
		edgeTypeLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		edgePanel.add(edgeTypeLabel, cedge);
		
		cedge.gridy++;
		cedge.gridx = 0;
		cedge.gridwidth = 2;
		cedge.weightx = 20;
		cedge.anchor = GridBagConstraints.EAST;
		edgePanel.add(new JLabel("Origin: "), cedge);
		cedge.gridx = 2;
		cedge.gridwidth = 3;
		cedge.weightx = 30;
		cedge.anchor = GridBagConstraints.WEST;
		edgeOriginLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		edgePanel.add(edgeOriginLabel, cedge);
		cedge.gridx = 5;
		cedge.gridwidth = 2;
		cedge.weightx = 20;
		cedge.anchor = GridBagConstraints.EAST;
		edgePanel.add(new JLabel("Destination: "), cedge);
		cedge.gridx = 7;
		cedge.gridwidth = 3;
		cedge.weightx = 30;
		cedge.anchor = GridBagConstraints.WEST;
		edgeDestinationLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		edgePanel.add(edgeDestinationLabel, cedge);
		
		cedge.gridy++;
		cedge.gridx = 0;
		cedge.gridwidth = 2;
		cedge.gridheight = 1;
		cedge.weightx = 0;
		cedge.weighty = 20;
		cedge.fill = GridBagConstraints.NONE;
		cedge.anchor = GridBagConstraints.EAST;
		edgePanel.add(new JLabel("Distance: "), cedge);
		cedge.gridx = 2;
		cedge.gridwidth = 8;
		cedge.weightx = 80;
		cedge.anchor = GridBagConstraints.WEST;
		edgeDistanceLabel.setPreferredSize(new Dimension(2*xLabel,yLabel));
		edgePanel.add(edgeDistanceLabel, cedge);
		
		cedge.gridy++;
		cedge.gridx = 0;
		cedge.gridwidth = 2;
		cedge.weightx = 20;
		cedge.anchor = GridBagConstraints.EAST;
		edgePanel.add(new JLabel("Duration: "), cedge);
		cedge.gridx = 2;
		cedge.gridwidth = 3;
		cedge.weightx = 30;
		cedge.anchor = GridBagConstraints.WEST;
		edgeDurationLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		edgePanel.add(edgeDurationLabel, cedge);
		cedge.gridx = 5;
		cedge.gridwidth = 2;
		cedge.weightx = 20;
		cedge.anchor = GridBagConstraints.EAST;
		edgePanel.add(new JLabel("Max Crew Size: "), cedge);
		cedge.gridx = 7;
		cedge.gridwidth = 3;
		cedge.weightx = 30;
		cedge.anchor = GridBagConstraints.WEST;
		edgeMaxCrewSizeLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		edgePanel.add(edgeMaxCrewSizeLabel, cedge);
		
		cedge.gridy++;
		cedge.gridx = 5;
		cedge.gridwidth = 2;
		cedge.gridheight = 1;
		cedge.weightx = 0;
		cedge.weighty = 20;
		cedge.fill = GridBagConstraints.NONE;
		cedge.anchor = GridBagConstraints.EAST;
		edgePanel.add(new JLabel("Max Cargo Mass: "), cedge);
		cedge.gridx = 7;
		cedge.gridwidth = 3;
		cedge.weightx = 30;
		cedge.anchor = GridBagConstraints.WEST;
		edgeMaxCargoMassLabel.setPreferredSize(new Dimension(xLabel,yLabel));
		edgePanel.add(edgeMaxCargoMassLabel, cedge);
	}
	
	/**
	 * Update info panel.
	 */
	public void updateInfoPanel() {
		if(nodeTable.getSelectedRow()>=0) {
			Node node = (Node)nodeTable.getValueAt(nodeTable.getSelectedRow(), 1);
			
			nodeTypeLabel.setText(node.getNodeType().getName());
			nodeDescriptionLabel.setText(node.getDescription());
			nodeBodyLabel.setText("" + node.getBody());
			nodeLatitudeLabel.setText("--");
			nodeLongitudeLabel.setText("--");
			nodePeriapsisLabel.setText("--");
			nodeApoapsisLabel.setText("--");
			nodeInclinationLabel.setText("--");
			if(node.getNodeType() == NodeType.LAGRANGE) {
				nodeBodyLabel.setText(node.getBody() + " & " + ((LagrangeNode)node).getMinorBody());
			} else if(node.getNodeType() == NodeType.ORBITAL) {
				nodePeriapsisLabel.setText(((OrbitalNode)node).getPeriapsis() + " km");
				nodeApoapsisLabel.setText(((OrbitalNode)node).getApoapsis() + " km");
				nodeInclinationLabel.setText(((OrbitalNode)node).getInclination() + " deg");
			} else if(node.getNodeType() == NodeType.SURFACE) {
				nodeLatitudeLabel.setText(((SurfaceNode)node).getLatitude() + " deg");
				nodeLongitudeLabel.setText(((SurfaceNode)node).getLongitude() + " deg");
			}
			
			edgeTypeLabel.setText("--");
			edgeDescriptionLabel.setText("--");
			edgeOriginLabel.setText("--");
			edgeDestinationLabel.setText("--");
			edgeDurationLabel.setText("--");
			edgeDistanceLabel.setText("--");
			edgeMaxCrewSizeLabel.setText("--");
			edgeMaxCargoMassLabel.setText("--");
		} else if(edgeTable.getSelectedRow()>=0) {
			Edge edge = (Edge)edgeTable.getValueAt(edgeTable.getSelectedRow(), 1);
			
			edgeTypeLabel.setText(edge.getEdgeType().getName());
			edgeDescriptionLabel.setText(edge.getDescription());
			edgeOriginLabel.setText("" + edge.getOrigin());
			edgeDestinationLabel.setText("" + edge.getDestination());
			edgeDurationLabel.setText("--");
			edgeDistanceLabel.setText("--");
			edgeMaxCrewSizeLabel.setText("--");
			edgeMaxCargoMassLabel.setText("--");
			if(edge.getEdgeType() == EdgeType.FLIGHT) {
				edgeDurationLabel.setText(((FlightEdge)edge).getDuration() + " days");
				edgeMaxCrewSizeLabel.setText(((FlightEdge)edge).getMaxCrewSize() + "");
				edgeMaxCargoMassLabel.setText(((FlightEdge)edge).getMaxCargoMass() + " kg");				
			} else if(edge.getEdgeType() == EdgeType.SPACE) {
				edgeDurationLabel.setText(((SpaceEdge)edge).getDuration() + " days");
			} else if(edge.getEdgeType() == EdgeType.SURFACE) {
				edgeDistanceLabel.setText(((SurfaceEdge)edge).getDistance() + " km");
			}
			
			nodeTypeLabel.setText("--");
			nodeDescriptionLabel.setText("--");
			nodeBodyLabel.setText("--");
			nodeLatitudeLabel.setText("--");
			nodeLongitudeLabel.setText("--");
			nodePeriapsisLabel.setText("--");
			nodeApoapsisLabel.setText("--");
			nodeInclinationLabel.setText("--");	
		}
	}
	
	/**
	 * Gets the scenario.
	 * 
	 * @return the scenario
	 */
	public Scenario getScenario() {
		return scenarioPanel.getScenario();
	}
	
	/**
	 * Gets the scenario panel.
	 * 
	 * @return the scenario panel
	 */
	public ScenarioPanel getScenarioPanel() {
		return scenarioPanel;
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		nodeTable.updateView();
		edgeTable.updateView();
		networkPanel.repaint();
	}
	
	/**
	 * Gets the node search text.
	 * 
	 * @return the node search text
	 */
	public String getNodeSearchText() {
		if(nodeSearchText.getText().equals(nodeSearchText.getDefaultText())) 
			return "";
		else return nodeSearchText.getText();
	}
	
	/**
	 * Gets the node type filter.
	 * 
	 * @return the node type filter
	 */
	public NodeType getNodeTypeFilter() {
		if(nodeTypeCombo.getSelectedItem() instanceof NodeType) {
			return (NodeType)nodeTypeCombo.getSelectedItem();
		} else return null;
	}
	
	/**
	 * Gets the edge search text.
	 * 
	 * @return the edge search text
	 */
	public String getEdgeSearchText() {
		if(edgeSearchText.getText().equals(edgeSearchText.getDefaultText())) 
			return "";
		else return edgeSearchText.getText();
	}
	
	/**
	 * Gets the edge type filter.
	 * 
	 * @return the edge type filter
	 */
	public EdgeType getEdgeTypeFilter() {
		if(edgeTypeCombo.getSelectedItem() instanceof EdgeType) {
			return (EdgeType)edgeTypeCombo.getSelectedItem();
		} else return null;
	}
}
