/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.simulation;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.JToggleButton;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import org.jdesktop.swingworker.SwingWorker;

import edu.mit.spacenet.domain.element.I_Carrier;
import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.element.PropulsiveVehicle;
import edu.mit.spacenet.domain.element.SurfaceVehicle;
import edu.mit.spacenet.gui.component.ElementTree;
import edu.mit.spacenet.gui.visualization.NetworkPanel;
import edu.mit.spacenet.scenario.Scenario;
import edu.mit.spacenet.util.DateFunctions;
import edu.mit.spacenet.util.GlobalParameters;

/**
 * Visualization of the network with options to run animation over the course
 * of the simulation.
 * 
 * @author Paul Grogan
 */
public class NetworkHistoryVisualization extends JSplitPane {
	private static final long serialVersionUID = -6405124067606236705L;
	private static int numLabels = 5;
	private static double stepsPerDay = 4;
	
	private SimulationTab tab;
	private DecimalFormat decimalFormat = new DecimalFormat("0.0");
	
	private JSlider timeSlider;
	private JToggleButton playButton;
	private Animation animation;
	private JButton startButton, rewindButton, stopButton,
		fastforwardButton, endButton;
	private JSpinner rateSpinner;
	private JCheckBox constantTimeCheck;
	
	private NetworkPanel networkPanel;
	private JLabel dateLabel, relativeTimeLabel;
	private ElementTree elementTree;
	private JLabel stateLabel, massLabel, volumeLabel, cargoMassLabel, cargoVolumeLabel, crewLabel, omsLabel, rcsLabel, fuelLabel;
	private JProgressBar cargoMassCapacity, cargoVolumeCapacity, crewCapacity, omsCapacity, rcsCapacity, fuelCapacity;
	
	/**
	 * Instantiates a new network history visualization.
	 * 
	 * @param tab the simulation tab
	 */
	public NetworkHistoryVisualization(SimulationTab tab) {
		this.tab = tab;
		
		JSplitPane visualizationPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		JPanel sliderPanel = new JPanel();
		sliderPanel.setLayout(new BoxLayout(sliderPanel, BoxLayout.PAGE_AXIS));
		sliderPanel.setBackground(Color.WHITE);
		timeSlider = new JSlider();
		timeSlider.setOpaque(false);
		timeSlider.setEnabled(false);
		timeSlider.setFocusable(false);
		timeSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				updateElements();
			}
		});
		sliderPanel.add(timeSlider);
		visualizationPanel.setTopComponent(sliderPanel);
		
		networkPanel = new NetworkPanel(getScenario().getNetwork());
		networkPanel.setBackground(Color.BLACK);
		networkPanel.setPreferredSize(new Dimension(600,400));
		visualizationPanel.setBottomComponent(networkPanel);
		
		visualizationPanel.setDividerSize(0);
		visualizationPanel.setDividerLocation(75);
		visualizationPanel.setBorder(BorderFactory.createEmptyBorder());
		setLeftComponent(visualizationPanel);
		
		JPanel controlPanel = new JPanel();
		controlPanel.setBackground(Color.WHITE);
		controlPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		controlPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		c.gridwidth = 2;
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setOpaque(false);
		startButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("icons/control_start.png")));
		startButton.setMargin(new Insets(0,0,0,0));
		startButton.setEnabled(false);
		startButton.setFocusable(false);
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(playButton.isSelected()) {
					playButton.setSelected(false);
				}
				timeSlider.setValue(timeSlider.getMinimum());
			}
		});
		buttonPanel.add(startButton);
		rewindButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("icons/control_rewind.png")));
		rewindButton.setMargin(new Insets(0,0,0,0));
		rewindButton.setEnabled(false);
		rewindButton.setFocusable(false);
		rewindButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(playButton.isSelected()) {
					playButton.setSelected(false);
				}
				timeSlider.setValue(Math.max(timeSlider.getMinimum(), timeSlider.getValue()-1));
			}
		});
		buttonPanel.add(rewindButton);
		stopButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("icons/control_stop.png")));
		stopButton.setMargin(new Insets(0,0,0,0));
		stopButton.setEnabled(false);
		stopButton.setFocusable(false);
		stopButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(playButton.isSelected()) {
					playButton.setSelected(false);
				}
			}
		});
		buttonPanel.add(stopButton);
		playButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("icons/control_play.png")));
		playButton.setSelectedIcon(new ImageIcon(getClass().getClassLoader().getResource("icons/control_pause.png")));
		playButton.setMargin(new Insets(0,0,0,0));
		playButton.setEnabled(false);
		playButton.setFocusable(false);
		playButton.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED) {
					if(timeSlider.getValue()==timeSlider.getMaximum())
						timeSlider.setValue(timeSlider.getMinimum());
					animation = new Animation();
					animation.execute();
				} else if(e.getStateChange()==ItemEvent.DESELECTED) {
					animation.cancel(true);
				}
			}
		});
		buttonPanel.add(playButton);
		fastforwardButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("icons/control_fastforward.png")));
		fastforwardButton.setMargin(new Insets(0,0,0,0));
		fastforwardButton.setEnabled(false);
		fastforwardButton.setFocusable(false);
		fastforwardButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(playButton.isSelected()) {
					playButton.setSelected(false);
				}
				timeSlider.setValue(Math.min(timeSlider.getMaximum(), timeSlider.getValue()+1));
			}
		});
		buttonPanel.add(fastforwardButton);
		endButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("icons/control_end.png")));
		endButton.setMargin(new Insets(0,0,0,0));
		endButton.setEnabled(false);
		endButton.setFocusable(false);
		endButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(playButton.isSelected()) {
					playButton.setSelected(false);
				}
				timeSlider.setValue(timeSlider.getMaximum());
			}
		});
		buttonPanel.add(endButton);
		controlPanel.add(buttonPanel, c);
		c.gridy++;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		JLabel rateLabel = new JLabel("Frames per second: ");
		rateLabel.setOpaque(false);
		controlPanel.add(rateLabel, c);
		c.gridx++;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		rateSpinner = new JSpinner(new SpinnerNumberModel(10, 1, 50, 1));
		controlPanel.add(rateSpinner, c);
		c.gridx--;
		c.gridy++;
		c.gridwidth = 2;
		constantTimeCheck = new JCheckBox("Constant Time per Frame");
		constantTimeCheck.setOpaque(false);
		controlPanel.add(constantTimeCheck, c);
		c.gridy++;
		c.weighty = 1;
		JPanel detailsPanel = new JPanel();
		controlPanel.add(detailsPanel, c);
		detailsPanel.setBackground(Color.WHITE);
		detailsPanel.setLayout(new GridBagLayout());
		detailsPanel.setBorder(BorderFactory.createTitledBorder("Element Details"));
		c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		c.gridwidth = 2;
		c.weightx = 1;
		dateLabel = new JLabel();
		detailsPanel.add(dateLabel, c);
		c.gridy++;
		relativeTimeLabel = new JLabel();
		detailsPanel.add(relativeTimeLabel, c);
		c.gridy++;
		detailsPanel.add(new JLabel("Elements: "), c);
		c.gridy++;
		c.weighty = 1;
		elementTree = new ElementTree();
		elementTree.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				updateCapacities();
			}
		});
		JScrollPane elementScroll = new JScrollPane(elementTree);
		elementScroll.setBackground(Color.WHITE);
		elementScroll.getViewport().setBackground(Color.WHITE);
		elementScroll.setPreferredSize(new Dimension(150,200));
		detailsPanel.add(elementScroll, c);
		c.gridy++;
		c.weighty = 0;
		c.weightx = 0;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.LINE_END;
		detailsPanel.add(new JLabel("State: "), c);
		c.gridy++;
		detailsPanel.add(new JLabel("Mass: "), c);
		c.gridy++;
		detailsPanel.add(new JLabel("Volume: "), c);
		c.gridy++;
		cargoMassLabel = new JLabel("Cargo Mass: ");
		cargoMassLabel.setVisible(false);
		detailsPanel.add(cargoMassLabel, c);
		c.gridy++;
		cargoVolumeLabel = new JLabel("Cargo Volume: ");
		cargoVolumeLabel.setVisible(false);
		detailsPanel.add(cargoVolumeLabel, c);
		c.gridy++;
		crewLabel = new JLabel("Crew: ");
		crewLabel.setVisible(false);
		detailsPanel.add(crewLabel, c);
		c.gridy++;
		omsLabel = new JLabel("OMS Propellant: ");
		omsLabel.setVisible(false);
		detailsPanel.add(omsLabel, c);
		c.gridy++;
		rcsLabel = new JLabel("RCS Propellant: ");
		rcsLabel.setVisible(false);
		detailsPanel.add(rcsLabel, c);
		c.gridy++;
		fuelLabel = new JLabel("Fuel: ");
		fuelLabel.setVisible(false);
		detailsPanel.add(fuelLabel, c);
		c.gridx++;
		c.gridy-=8;
		c.weightx = 1;
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.LINE_START;
		stateLabel = new JLabel();
		detailsPanel.add(stateLabel, c);
		c.gridy++;
		massLabel = new JLabel();
		detailsPanel.add(massLabel, c);
		c.gridy++;
		volumeLabel = new JLabel();
		detailsPanel.add(volumeLabel, c);
		c.gridy++;
		cargoMassCapacity = new JProgressBar(0,100);
		cargoMassCapacity.setVisible(false);
		cargoMassCapacity.setStringPainted(true);
		detailsPanel.add(cargoMassCapacity, c);
		c.gridy++;
		cargoVolumeCapacity = new JProgressBar(0,100);
		cargoVolumeCapacity.setVisible(false);
		cargoVolumeCapacity.setStringPainted(true);
		detailsPanel.add(cargoVolumeCapacity, c);
		c.gridy++;
		crewCapacity = new JProgressBar(0,100);
		crewCapacity.setVisible(false);
		crewCapacity.setStringPainted(true);
		detailsPanel.add(crewCapacity, c);
		c.gridy++;
		omsCapacity = new JProgressBar(0,100);
		omsCapacity.setVisible(false);
		omsCapacity.setStringPainted(true);
		omsCapacity.setForeground(new Color(0, 153, 0));
		detailsPanel.add(omsCapacity, c);
		c.gridy++;
		rcsCapacity = new JProgressBar(0,100);
		rcsCapacity.setVisible(false);
		rcsCapacity.setStringPainted(true);
		rcsCapacity.setForeground(new Color(0, 153, 0));
		detailsPanel.add(rcsCapacity, c);
		c.gridy++;
		fuelCapacity = new JProgressBar(0,100);
		fuelCapacity.setVisible(false);
		fuelCapacity.setStringPainted(true);
		fuelCapacity.setForeground(new Color(0, 153, 0));
		detailsPanel.add(fuelCapacity, c);
		controlPanel.setMinimumSize(new Dimension(150,50));
		setRightComponent(controlPanel);

		setName("Network History");
		setOneTouchExpandable(true);
		setDividerSize(10);
		setBorder(BorderFactory.createEmptyBorder());
		setResizeWeight(1);
		setDividerLocation(700);
	}
	
	/**
	 * Gets the scenario.
	 * 
	 * @return the scenario
	 */
	public Scenario getScenario() {
		return tab.getScenario();
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		tab.getScenarioPanel().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		if(tab.getSimulator().getNetworkHistory().size() > 0) {
			timeSlider.setEnabled(true);
			timeSlider.setMinimum(0);
			double maxTime = tab.getSimulator().getNetworkHistory().get(tab.getSimulator().getNetworkHistory().size()-1).getTime();
			int maxVal = (int)Math.ceil(maxTime*stepsPerDay);
			while(maxVal%numLabels!=0) maxVal++;
			timeSlider.setMaximum(maxVal);
			Hashtable<Integer, JLabel> labelDictionary = new Hashtable<Integer, JLabel>();
			SimpleDateFormat dateFormat = new SimpleDateFormat("MMM d, yyyy");
			for(int i=0; i<=numLabels; i++) {
				Date date = DateFunctions.getDate(getScenario().getStartDate(), maxVal*i/numLabels/stepsPerDay);
				labelDictionary.put(maxVal*i/numLabels, new JLabel(dateFormat.format(date)));
			}
			timeSlider.setLabelTable(labelDictionary);
			timeSlider.setMajorTickSpacing(maxVal/numLabels);
			timeSlider.setMinorTickSpacing(0);
			timeSlider.setPaintTicks(true);
			timeSlider.setPaintLabels(true);
			timeSlider.setValue(timeSlider.getMinimum());
			
			startButton.setEnabled(true);
			rewindButton.setEnabled(true);
			playButton.setEnabled(true);
			stopButton.setEnabled(true);
			fastforwardButton.setEnabled(true);
			endButton.setEnabled(true);
			updateElements();
		} else {
			timeSlider.setEnabled(false);
			startButton.setEnabled(false);
			rewindButton.setEnabled(false);
			playButton.setEnabled(false);
			stopButton.setEnabled(false);
			fastforwardButton.setEnabled(false);
			endButton.setEnabled(false);
		}
		tab.getScenarioPanel().setCursor(Cursor.getDefaultCursor());
	}
	private void updateElements() {
		int i=0;
		while(i < tab.getSimulator().getNetworkHistory().size()
				&& tab.getSimulator().getNetworkHistory().get(i).getTime() < timeSlider.getValue()/stepsPerDay) {
			i++;
		}
		SimpleDateFormat dateFormat = new SimpleDateFormat("MMM d, yyyy kk:mm");
		dateLabel.setText("Date: " + dateFormat.format(DateFunctions.getDate(getScenario().getStartDate(), timeSlider.getValue()/stepsPerDay))); 
		relativeTimeLabel.setText("Relative Time: " + GlobalParameters.getRoundedTime(timeSlider.getValue()/stepsPerDay));
		elementTree.setRoot(tab.getSimulator().getNetworkHistory().get(i).getNetwork());
		networkPanel.setNetwork(tab.getSimulator().getNetworkHistory().get(i).getNetwork());
		networkPanel.repaint();
	}
	private void updateCapacities() {
		if(elementTree.getSelection() instanceof I_Element) {
			I_Element element = (I_Element)elementTree.getSelection();
			if(element.getCurrentState()==null) {
				stateLabel.setText(null);
				stateLabel.setIcon(null);
			} else {
				stateLabel.setText(element.getCurrentState().toString());
				stateLabel.setIcon(element.getCurrentState().getStateType().getIcon());
			}
			massLabel.setText(decimalFormat.format(element.getTotalMass()) + " kg");
			volumeLabel.setText(decimalFormat.format(element.getTotalVolume()) + " m^3");
		} else {
			stateLabel.setText(null);
			stateLabel.setIcon(null);
			massLabel.setText(null);
			volumeLabel.setText(null);
		}
		
		if(elementTree.getSelection() instanceof I_Carrier) {
			I_Carrier carrier = (I_Carrier)elementTree.getSelection();
			
			if(carrier.getCargoMass() > carrier.getMaxCargoMass()) cargoMassCapacity.setForeground(new Color(153, 0, 0));
			else cargoMassCapacity.setForeground(new Color(0, 153, 0));
			if(carrier.getMaxCargoMass()==0) cargoMassCapacity.setValue(100);
			else cargoMassCapacity.setValue((int)(100*carrier.getCargoMass()/carrier.getMaxCargoMass()));
			cargoMassCapacity.setString(decimalFormat.format(carrier.getCargoMass()) + " kg");
			
			if(carrier.getCargoVolume() > carrier.getMaxCargoVolume()) cargoVolumeCapacity.setForeground(new Color(153, 0, 0));
			else cargoVolumeCapacity.setForeground(new Color(0, 153, 0));
			if(carrier.getMaxCargoVolume()==0) cargoVolumeCapacity.setValue(100);
			else cargoVolumeCapacity.setValue((int)(100*carrier.getCargoVolume()/carrier.getMaxCargoVolume()));
			cargoVolumeCapacity.setString(decimalFormat.format(carrier.getCargoVolume()) + " m^3");
			
			if(carrier.getCrewSize() > carrier.getMaxCrewSize()) crewCapacity.setForeground(new Color(153, 0, 0));
			else crewCapacity.setForeground(new Color(0, 153, 0));
			if(carrier.getMaxCrewSize()==0) crewCapacity.setValue(100);
			else crewCapacity.setValue((int)(100*carrier.getCrewSize()/carrier.getMaxCrewSize()));
			crewCapacity.setString(carrier.getCrewSize() + " crew");
			
			cargoMassLabel.setVisible(true);
			cargoVolumeLabel.setVisible(true);
			crewLabel.setVisible(true);
			cargoMassCapacity.setVisible(true);
			cargoVolumeCapacity.setVisible(true);
			crewCapacity.setVisible(true);
		} else {
			cargoMassLabel.setVisible(false);
			cargoVolumeLabel.setVisible(false);
			crewLabel.setVisible(false);
			cargoMassCapacity.setVisible(false);
			cargoVolumeCapacity.setVisible(false);
			crewCapacity.setVisible(false);
		}
		if(elementTree.getSelection() instanceof PropulsiveVehicle) {
			PropulsiveVehicle vehicle = (PropulsiveVehicle)elementTree.getSelection();
			if(vehicle.getOmsFuelTank()!=null) {
				omsLabel.setVisible(true);
				omsCapacity.setVisible(true);
				omsCapacity.setValue((int)(100*vehicle.getOmsFuelTank().getAmount()/vehicle.getOmsFuelTank().getMaxAmount()));
				omsCapacity.setString(decimalFormat.format(vehicle.getOmsFuelTank().getAmount()) 
						+ " " + vehicle.getOmsFuelTank().getResource().getUnits());
			} else {
				omsLabel.setVisible(false);
				omsCapacity.setVisible(false);
			}
			if(vehicle.getRcsFuelTank() != null 
					&& vehicle.getRcsFuelTank()!=vehicle.getOmsFuelTank()) {
				rcsLabel.setVisible(true);
				rcsCapacity.setVisible(true);
				rcsCapacity.setValue((int)(100*vehicle.getRcsFuelTank().getAmount()/vehicle.getRcsFuelTank().getMaxAmount()));
				rcsCapacity.setString(decimalFormat.format(vehicle.getRcsFuelTank().getAmount()) 
						+ " " + vehicle.getRcsFuelTank().getResource().getUnits());
			} else {
				rcsLabel.setVisible(false);
				rcsCapacity.setVisible(false);
			}
			fuelLabel.setVisible(false);
			fuelCapacity.setVisible(false);
		} else if(elementTree.getSelection() instanceof SurfaceVehicle) {
			SurfaceVehicle vehicle = (SurfaceVehicle)elementTree.getSelection();
			fuelLabel.setVisible(true);
			fuelCapacity.setVisible(true);
			fuelCapacity.setValue((int)(100*vehicle.getFuelTank().getAmount()/vehicle.getFuelTank().getMaxAmount()));
			fuelCapacity.setString(decimalFormat.format(vehicle.getFuelTank().getAmount()) 
					+ " " + vehicle.getFuelTank().getResource().getUnits());
			omsLabel.setVisible(false);
			rcsLabel.setVisible(false);
			omsCapacity.setVisible(false);
			rcsCapacity.setVisible(false);
		} else {
			omsLabel.setVisible(false);
			rcsLabel.setVisible(false);
			fuelLabel.setVisible(false);
			omsCapacity.setVisible(false);
			rcsCapacity.setVisible(false);
			fuelCapacity.setVisible(false);
		}
	}
	
	/**
	 * The Class Animation.
	 */
	class Animation extends SwingWorker<Void, Void> {
		
		/* (non-Javadoc)
		 * @see org.jdesktop.swingworker.SwingWorker#doInBackground()
		 */
		@Override 
		public Void doInBackground() {
			if(constantTimeCheck.isSelected()) {
				while(timeSlider.getValue() < timeSlider.getMaximum() 
						&& !isCancelled()) {
					timeSlider.setValue(timeSlider.getValue() + 1);
					try {
						Thread.sleep((1000/(Integer)rateSpinner.getValue()));
					} catch (InterruptedException e) { }
				}
			} else {
				int i = 0;
				while(i < tab.getSimulator().getNetworkHistory().size() 
						&& timeSlider.getValue()/stepsPerDay > tab.getSimulator().getNetworkHistory().get(i).getTime()) {
					i++;
				}
				while(i < tab.getSimulator().getNetworkHistory().size()
						&& !isCancelled()) {
					timeSlider.setValue((int)(tab.getSimulator().getNetworkHistory().get(i).getTime()*stepsPerDay));
					i++;
					try {
						Thread.sleep((1000/(Integer)rateSpinner.getValue()));
					} catch (InterruptedException e) { }
				}
			}
			return null;
		}
		
		/* (non-Javadoc)
		 * @see org.jdesktop.swingworker.SwingWorker#done()
		 */
		public void done() {
			playButton.setSelected(false);
		}
	}
}
