/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.scenario.data;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.model.I_DemandModel;
import edu.mit.spacenet.domain.network.edge.Edge;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.domain.resource.GenericResource;
import edu.mit.spacenet.domain.resource.I_Item;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.domain.resource.Resource;

/**
 * An abstract implementation of the data source interface.
 * 
 * @author Paul Grogan
 */
public abstract class AbstractDataSource implements I_DataSource {
	@Deprecated private DataSourceType dataSourceType;
	protected List<Node> nodeLibrary;
	protected List<Edge> edgeLibrary;
	protected List<I_Resource> resourceTypeLibrary;
	protected List<ElementPreview> elementPreviewLibrary;
	protected List<I_DemandModel> demandModelLibrary;
	private Date lastLoadDate;
	
	/**
	 * Instantiates a new abstract data source.
	 */
	public AbstractDataSource() {
		nodeLibrary = new ArrayList<Node>();
		edgeLibrary = new ArrayList<Edge>();
		resourceTypeLibrary = new ArrayList<I_Resource>();
		elementPreviewLibrary = new ArrayList<ElementPreview>();
		demandModelLibrary = new ArrayList<I_DemandModel>();
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getNodeLibrary()
	 */
	public List<Node> getNodeLibrary() {
		return nodeLibrary;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getEdgeLibrary()
	 */
	public List<Edge> getEdgeLibrary() {
		return edgeLibrary;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getResourceLibrary()
	 */
	public List<I_Resource> getResourceLibrary() {
		return resourceTypeLibrary;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getContinuousResourceLibrary()
	 */
	public List<Resource> getContinuousResourceLibrary() {
		ArrayList<Resource> list = new ArrayList<Resource>();
		for(I_Resource r : resourceTypeLibrary) {
			if(r.getClass().equals(Resource.class)) {
				list.add((Resource)r);
			}
		}
		return list;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getItemLibrary()
	 */
	public List<I_Item> getItemLibrary() {
		ArrayList<I_Item> list = new ArrayList<I_Item>();
		for(I_Resource r : resourceTypeLibrary) {
			if(r instanceof I_Item) {
				list.add((I_Item)r);
			}
		}
		return list;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getElementPreviewLibrary()
	 */
	public List<ElementPreview> getElementPreviewLibrary() {
		return elementPreviewLibrary;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getDemandModelLibrary()
	 */
	public List<I_DemandModel> getDemandModelLibrary() {
		return demandModelLibrary;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#print()
	 */
	public void print() {
		System.out.println("Node Library:");
		for(Node n : nodeLibrary) {
			System.out.println("\t" + n.getTid() + ": " + n.getName() + " (" + n.getClass().getSimpleName() + ") ");
		}
		System.out.println("Edge Library:");
		for(Edge e : edgeLibrary) {
			System.out.println("\t" + e.getTid() + ": " + e.getName() + " (" + e.getClass().getSimpleName() + ") ");
		}
		System.out.println("Resource Library:");
		for(I_Resource r : resourceTypeLibrary) {
			System.out.println("\t" + r.getTid() + ": " + r.getName() + " (COS " + r.getClassOfSupply().getId() + ") ");
		}
		System.out.println("Element Library:");
		for(ElementPreview o : elementPreviewLibrary) {
			System.out.println("\t" + o.ID + ": " + o.NAME + " (" + o.TYPE + ") ");
		}
		System.out.println("Demand Model Library:");
		for(I_DemandModel o : demandModelLibrary) {
			System.out.println("\t" + o.getTid() + ": " + o.getName() + " (" + o.getDemandModelType() + ") ");
		}
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#loadLibraries(boolean)
	 */
	public void loadLibraries(boolean updateNodes, boolean updateEdges, boolean updateResources) throws IOException {
		lastLoadDate = new Date();
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#loadLibraries()
	 */
	public void loadLibraries() throws IOException  {
		loadLibraries(true, true, true);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#loadResource(int)
	 */
	public I_Resource loadResource(int tid) {
		if(tid < 0) {
			return new GenericResource(ClassOfSupply.getInstance(-tid));
		}
		for(I_Resource resource : resourceTypeLibrary) {
			if(resource.getTid() == tid) return resource;
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getLastLoadDate()
	 */
	public Date getLastLoadDate() {
		return lastLoadDate;
	}
}
