/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.scenario.data;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.model.I_DemandModel;
import edu.mit.spacenet.domain.network.edge.Edge;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.domain.resource.I_Item;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.domain.resource.Resource;

/**
 * The data source interface allows for the abstracted loading of libraries and
 * elements from a general data source.
 * 
 * @author Paul Grogan
 */
public interface I_DataSource {
	
	/**
	 * Runs a validation script on the data source, checking for any errors.
	 * 
	 * @return a list of error messages
	 * 
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	public List<String> validateData() throws IOException;
	
	/**
	 * Gets a list of nodes loaded from the data source.
	 * 
	 * @return the list of nodes
	 */
	public List<Node> getNodeLibrary();
	
	/**
	 * Gets a list of edges loaded from the data source.
	 * 
	 * @return the list of edges
	 */
	public List<Edge> getEdgeLibrary();
	
	/**
	 * Gets a list of resources loaded from the data source.
	 * 
	 * @return the list of resources
	 */
	public List<I_Resource> getResourceLibrary();
	
	/**
	 * Gets a list of continuous resources loaded from the data source.
	 * 
	 * @return the list of continuous resources
	 */
	public List<Resource> getContinuousResourceLibrary();
	
	/**
	 * Gets a list of discrete resources loaded from the data source.
	 * 
	 * @return the list of discrete resources
	 */
	public List<I_Item> getItemLibrary();
	
	/**
	 * Gets a list of element previews loaded from the data source.
	 * 
	 * @return the list of element previews
	 */
	public List<ElementPreview> getElementPreviewLibrary();
	
	/**
	 * Gets a list of the demand models loaded from the data source.
	 * 
	 * @return the list of demand models
	 */
	public List<I_DemandModel> getDemandModelLibrary();
	
	/**
	 * Prints the.
	 */
	public void print();
	
	/**
	 * Loads the libraries with an option to overwrite existing items.
	 * 
	 * @param updateNodes whether to update the existing nodes
	 * @param updateEdges whether to update the existing edges
	 * @param updateResources whether to update the existing resources
	 * 
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	public void loadLibraries(boolean updateNodes, boolean updateEdges, boolean updateResources) throws IOException;
	
	/**
	 * Loads the libraries while overwriting existing items.
	 * 
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	public void loadLibraries() throws IOException;
	
	/**
	 * Loads an element from the data source by its type id.
	 * 
	 * @param tid the element's type identification number
	 * 
	 * @return the element
	 */
	public I_Element loadElement(int tid);
	
	/**
	 * Loads a resource from the data source by its type id.
	 * 
	 * @param tid the resource's type identification number
	 * 
	 * @return the resource
	 */
	public I_Resource loadResource(int tid);
	
	/**
	 * Gets the type of the data source.
	 * 
	 * @return the data source type
	 */
	public DataSourceType getDataSourceType();
	
	/**
	 * Gets the date that the data source was last loaded.
	 * 
	 * @return the last load date
	 */
	public Date getLastLoadDate();
	
	/**
	 * Gets the name of the data source.
	 * 
	 * @return the data source name
	 */
	public String getName();
}
