/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.scenario.data;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.Environment;
import edu.mit.spacenet.domain.element.Carrier;
import edu.mit.spacenet.domain.element.CrewMember;
import edu.mit.spacenet.domain.element.Element;
import edu.mit.spacenet.domain.element.ElementType;
import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.element.PartApplication;
import edu.mit.spacenet.domain.element.PropulsiveVehicle;
import edu.mit.spacenet.domain.element.ResourceContainer;
import edu.mit.spacenet.domain.element.ResourceTank;
import edu.mit.spacenet.domain.element.State;
import edu.mit.spacenet.domain.element.StateType;
import edu.mit.spacenet.domain.element.SurfaceVehicle;
import edu.mit.spacenet.domain.model.BoilOffDemandModel;
import edu.mit.spacenet.domain.model.IsruProductionModel;
import edu.mit.spacenet.domain.model.RatedDemandModel;
import edu.mit.spacenet.domain.model.SparingByMassDemandModel;
import edu.mit.spacenet.domain.model.TimedImpulseDemandModel;
import edu.mit.spacenet.domain.network.edge.Burn;
import edu.mit.spacenet.domain.network.edge.BurnType;
import edu.mit.spacenet.domain.network.edge.Edge;
import edu.mit.spacenet.domain.network.edge.FlightEdge;
import edu.mit.spacenet.domain.network.edge.SpaceEdge;
import edu.mit.spacenet.domain.network.edge.SurfaceEdge;
import edu.mit.spacenet.domain.network.node.Body;
import edu.mit.spacenet.domain.network.node.LagrangeNode;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.domain.network.node.OrbitalNode;
import edu.mit.spacenet.domain.network.node.SurfaceNode;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.domain.resource.Item;
import edu.mit.spacenet.domain.resource.Resource;

/**
 * A spreadsheet format compatible with the features introduced in SpaceNet 2.5
 * 
 * @author Paul Grogan
 */
public class Spreadsheet_2_5 extends AbstractDataSource {
	private String filePath;
	
	/**
	 * Instantiates a new spreadsheet_2_5.
	 * 
	 * @param filePath the file path
	 */
	public Spreadsheet_2_5(String filePath) {
		this.filePath = filePath;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#validateData()
	 */
	public List<String> validateData() throws IOException {
		POIFSFileSystem fs = new POIFSFileSystem(new FileInputStream(filePath));
    	HSSFWorkbook wb = new HSSFWorkbook(fs);
		return checkWorkbook(wb);
	}
	private List<String> checkWorkbook(HSSFWorkbook workbook) {
		ArrayList<String> errors = new ArrayList<String>();
		errors.addAll(checkNodes(workbook.getSheetAt(0)));
		errors.addAll(checkEdges(workbook.getSheetAt(1)));
		errors.addAll(checkBurns(workbook.getSheetAt(2)));
		errors.addAll(checkResources(workbook.getSheetAt(3)));
		errors.addAll(checkElements(workbook.getSheetAt(4)));
		errors.addAll(checkPartApplications(workbook.getSheetAt(5)));
		errors.addAll(checkStates(workbook.getSheetAt(6)));
		errors.addAll(checkModels(workbook.getSheetAt(7)));
		errors.addAll(checkDemands(workbook.getSheetAt(8)));
		return errors;
	}
	private List<String> checkNodes(HSSFSheet sheet) {
    	ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
    		String type = "";
    		
    		if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Node in row " + (i+1) + " has missing or invalid id.");
    		}
    		if(row.getCell(1)==null || row.getCell(1).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
    			errors.add("Node in row " + (i+1) + " has missing or invalid type.");
    		} else {
    			type = row.getCell(1).getStringCellValue();
    		}
    		if(row.getCell(2)==null || row.getCell(2).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
    			errors.add("Node in row " + (i+1) + " has missing or invalid name.");
    		}
    		if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
    			errors.add("Node in row " + (i+1) + " has missing or invalid body.");
    		}
    		if(type.toLowerCase().equals("surface")) {
    			if(row.getCell(4)==null || row.getCell(4).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    				errors.add("Node in row " + (i+1) + " has missing or invalid latitude.");
    			}
    			if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    				errors.add("Node in row " + (i+1) + " has missing or invalid longitude.");
    			}
    		} else if(type.toLowerCase().equals("orbital")) {
    			if(row.getCell(6)==null || row.getCell(6).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    				errors.add("Node in row " + (i+1) + " has missing or invalid apoapsis.");
    			}
    			if(row.getCell(7)==null || row.getCell(7).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    				errors.add("Node in row " + (i+1) + " has missing or invalid periapsis.");
    			}
    			if(row.getCell(8)==null || row.getCell(8).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    				errors.add("Node in row " + (i+1) + " has missing or invalid inclination.");
    			}
    		} else if(type.toLowerCase().equals("lagrange")) {
    			if(row.getCell(9)==null || row.getCell(9).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
    				errors.add("Node in row " + (i+1) + " has missing or invalid minor body.");
    			}
    			if(row.getCell(10)==null || row.getCell(10).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    				errors.add("Node in row " + (i+1) + " has missing or invalid lagrange number.");
    			}
    		} else if(!type.equals("")) {
    			errors.add("Node in row " + (i+1) + " has missing or invalid type.");
    		}
    	}
		return errors;
	}
	private List<String> checkEdges(HSSFSheet sheet) {
		ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
    		String type = "";
    		if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Edge in row " + (i+1) + " has missing or invalid id.");
			}
			if(row.getCell(1)==null || row.getCell(1).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
				errors.add("Edge in row " + (i+1) + " has missing or invalid type.");
			} else {
    			type = row.getCell(1).getStringCellValue();
    		}
			if(row.getCell(2)==null || row.getCell(2).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
				errors.add("Edge in row " + (i+1) + " has missing or invalid name.");
			}
			if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Edge in row " + (i+1) + " has missing or invalid origin node.");
			}
			if(row.getCell(4)==null || row.getCell(4).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Edge in row " + (i+1) + " has missing or invalid destination node.");
			}
			if(type.toLowerCase().equals("space")) {
				if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Edge in row " + (i+1) + " has missing or invalid duration.");
    			}
			} else if(type.toLowerCase().equals("surface")) {
				if(row.getCell(6)==null || row.getCell(6).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Edge in row " + (i+1) + " has missing or invalid distance.");
    			}
			} else if(type.toLowerCase().equals("flight")) {
				if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Edge in row " + (i+1) + " has missing or invalid duration.");
    			}
    			if(row.getCell(7)==null || row.getCell(7).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    				errors.add("Edge in row " + (i+1) + " has missing or invalid max crew size.");
    			}
    			if(row.getCell(8)==null || row.getCell(8).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    				errors.add("Edge in row " + (i+1) + " has missing or invalid max cargo mass.");
    			}
			} else if(!type.equals("")) {
				errors.add("Edge in row " + (i+1) + " has missing or invalid type.");
			}
		}
		return errors;
	}
	private List<String> checkBurns(HSSFSheet sheet) {
		ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
			if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Burn in row " + (i+1) + " has missing or invalid id.");
			}
    		if(row.getCell(2)==null || row.getCell(2).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Burn in row " + (i+1) + " has missing or invalid time.");
			}
    		if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Burn in row " + (i+1) + " has missing or invalid order.");
			}
    		if(row.getCell(4)==null || row.getCell(4).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
    			errors.add("Burn in row " + (i+1) + " has missing or invalid type.");
			}
    		if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Burn in row " + (i+1) + " has missing or invalid delta-v.");
			}
		}
		return errors;
	}
	private List<String> checkResources(HSSFSheet sheet) {
		ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
    		String type = "";
    		if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Resource in row " + (i+1) + " has missing or invalid id.");
			}
			if(row.getCell(1)==null || row.getCell(1).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
				errors.add("Resource in row " + (i+1) + " has missing or invalid type.");
			} else {
    			type = row.getCell(1).getStringCellValue();
    		}
			if(row.getCell(2)==null || row.getCell(2).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
				errors.add("Resource in row " + (i+1) + " has missing or invalid name.");
			}
			if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC
					|| ClassOfSupply.getInstance((int)row.getCell(3).getNumericCellValue()) == null) {
				errors.add("Resource in row " + (i+1) + " has missing or invalid class of supply.");
			}
			if(row.getCell(4)==null || row.getCell(4).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Resource in row " + (i+1) + " has missing or invalid unit mass.");
			}
			if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Resource in row " + (i+1) + " has missing or invalid unit volume.");
			}
			if(row.getCell(6)==null || row.getCell(6).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Resource in row " + (i+1) + " has missing or invalid packing factor.");
			}
			if(row.getCell(7)==null || row.getCell(7).getCellType()!=HSSFCell.CELL_TYPE_STRING 
					|| (!row.getCell(7).getStringCellValue().toLowerCase().equals("pressurized")
							&& !row.getCell(7).getStringCellValue().toLowerCase().equals("unpressurized"))) {
				errors.add("Resource in row " + (i+1) + " has missing or invalid environment.");
			}
			if(type.toLowerCase().equals("continuous")) {
				
			} else if(type.toLowerCase().equals("discrete")) {
				
			} else if(!type.equals("")) {
				errors.add("Resource in row " + (i+1) + " has missing or invalid type.");
			}
		}
		return errors;
	}
	private List<String> checkElements(HSSFSheet sheet) {
		ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
    		String type = "";
    		if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Element in row " + (i+1) + " has missing or invalid id.");
			}
			if(row.getCell(1)==null || row.getCell(1).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
				errors.add("Element in row " + (i+1) + " has missing or invalid type.");
			} else {
    			type = row.getCell(1).getStringCellValue();
    		}
			if(row.getCell(2)==null || row.getCell(2).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
				errors.add("Element in row " + (i+1) + " has missing or invalid name.");
			}
			if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC
					|| ClassOfSupply.getInstance((int)row.getCell(3).getNumericCellValue()) == null) {
				errors.add("Element in row " + (i+1) + " has missing or invalid class of supply.");
			}
			if(row.getCell(4)==null || row.getCell(4).getCellType()!=HSSFCell.CELL_TYPE_STRING 
					|| (!row.getCell(4).getStringCellValue().toLowerCase().equals("pressurized")
							&& !row.getCell(4).getStringCellValue().toLowerCase().equals("unpressurized"))) {
				errors.add("Element in row " + (i+1) + " has missing or invalid environment.");
			}
			if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Element in row " + (i+1) + " has missing or invalid accommodation mass.");
			}
			if(row.getCell(6)==null || row.getCell(6).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Element in row " + (i+1) + " has missing or invalid mass.");
			}
			if(row.getCell(7)==null || row.getCell(7).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
				errors.add("Element in row " + (i+1) + " has missing or invalid volume.");
			}
			if(type.toLowerCase().equals("element")) {
				
			} else if(type.toLowerCase().equals("container")) {
				if(row.getCell(9)==null || row.getCell(9).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Container in row " + (i+1) + " has missing or invalid max cargo mass.");
				}
				if(row.getCell(10)==null || row.getCell(10).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Container in row " + (i+1) + " has missing or invalid max cargo volume.");
				}
				if(row.getCell(11)==null || row.getCell(11).getCellType()!=HSSFCell.CELL_TYPE_STRING 
						|| (!row.getCell(11).getStringCellValue().toLowerCase().equals("pressurized")
								&& !row.getCell(11).getStringCellValue().toLowerCase().equals("unpressurized"))) {
					errors.add("Container in row " + (i+1) + " has missing or invalid cargo environment.");
				}
			} else if(type.toLowerCase().equals("crew member")) {
				if(row.getCell(12)==null || row.getCell(12).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Crew Member in row " + (i+1) + " has missing or invalid active time fraction.");
				}
			} else if(type.toLowerCase().equals("carrier")
					|| type.toLowerCase().equals("propulsive")
					|| type.toLowerCase().equals("surface")) {
				if(row.getCell(8)==null || row.getCell(8).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Carrier in row " + (i+1) + " has missing or invalid max crew.");
				}
				if(row.getCell(9)==null || row.getCell(9).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Carrier in row " + (i+1) + " has missing or invalid max cargo mass.");
				}
				if(row.getCell(10)==null || row.getCell(10).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Container in row " + (i+1) + " has missing or invalid max cargo volume.");
				}
				if(row.getCell(11)==null || row.getCell(11).getCellType()!=HSSFCell.CELL_TYPE_STRING 
						|| (!row.getCell(11).getStringCellValue().toLowerCase().equals("pressurized")
								&& !row.getCell(11).getStringCellValue().toLowerCase().equals("unpressurized"))) {
					errors.add("Container in row " + (i+1) + " has missing or invalid cargo environment.");
				}
			} else if(type.toLowerCase().equals("propulsive")) {
				if(row.getCell(13)==null || row.getCell(13).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Propulsive Vehicle in row " + (i+1) + " has missing or invalid OMS Isp.");
				}
				if(row.getCell(15)==null || row.getCell(15).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Propulsive Vehicle in row " + (i+1) + " has missing or invalid max OMS fuel.");
				}
				if(row.getCell(16)==null || row.getCell(16).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Propulsive Vehicle in row " + (i+1) + " has missing or invalid RCS Isp.");
				}
				if(row.getCell(18)==null || row.getCell(18).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Propulsive Vehicle in row " + (i+1) + " has missing or invalid max RCS fuel.");
				}
			} else if(type.toLowerCase().equals("surface")) {
				if(row.getCell(19)==null || row.getCell(13).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Surface Vehicle in row " + (i+1) + " has missing or invalid max speed.");
				}
				if(row.getCell(20)==null || row.getCell(14).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Surface Vehicle in row " + (i+1) + " has missing or invalid max fuel.");
				}
				if(row.getCell(21)==null || row.getCell(15).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
					errors.add("Surface Vehicle in row " + (i+1) + " has missing or invalid fuel id.");
				}
			} else if(!type.equals("")) {
				errors.add("Element in row " + (i+1) + " has missing or invalid type.");
			}
		}
		return errors;
	}
	private List<String> checkPartApplications(HSSFSheet sheet) {
		ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
    		if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Part Application in row " + (i+1) + " has missing or invalid id.");
			}
    		if(row.getCell(1)==null || row.getCell(1).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			// TODO: check for resource type validity
    			errors.add("Part Application in row " + (i+1) + " has missing or invalid part id.");
			}
    		if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Part Application in row " + (i+1) + " has missing or invalid quantity.");
			}
    		if(row.getCell(4)==null || row.getCell(4).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Part Application in row " + (i+1) + " has missing or invalid duty cycle.");
			}
    		if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Part Application in row " + (i+1) + " has missing or invalid mean time to failure.");
			}
    		if(row.getCell(6)==null || row.getCell(6).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Part Application in row " + (i+1) + " has missing or invalid mean time to repair.");
			}
    		if(row.getCell(7)==null || row.getCell(7).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Part Application in row " + (i+1) + " has missing or invalid mass to repair.");
			}
		}
		return errors;
	}
	private List<String> checkStates(HSSFSheet sheet) {
		ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
    		if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("State in row " + (i+1) + " has missing or invalid id.");
			}
    		if(row.getCell(2)==null || row.getCell(2).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
    			errors.add("State in row " + (i+1) + " has missing or invalid name.");
			}
    		if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_STRING
    				|| !(row.getCell(3).getStringCellValue().toLowerCase().equals("active") 
    						|| row.getCell(3).getStringCellValue().toLowerCase().equals("quiescent")
    						|| row.getCell(3).getStringCellValue().toLowerCase().equals("dormant")
    						|| row.getCell(3).getStringCellValue().toLowerCase().equals("decommissioned"))) {
    			errors.add("State in row " + (i+1) + " has missing or invalid type.");
			}
    		if(row.getCell(4)==null || row.getCell(4).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("State in row " + (i+1) + " has missing or invalid duty cycle.");
			}
    		if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("State in row " + (i+1) + " has missing or invalid initial state flag.");
			}
		}
		return errors;
	}
	private List<String> checkModels(HSSFSheet sheet) {
		ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
    		String type = "";
    		if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Model in row " + (i+1) + " has missing or invalid id.");
			}
    		if(row.getCell(1)==null || row.getCell(1).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
    			errors.add("Model in row " + (i+1) + " has missing or invalid type.");
			} else {
				type = row.getCell(1).getStringCellValue();
			}
    		if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_STRING) {
    			errors.add("Model in row " + (i+1) + " has missing or invalid name.");
			}
    		if(type.toLowerCase().equals("impulse")) {
    			// nothing to check
    		} else if(type.toLowerCase().equals("timed")) { 
    			if(row.getCell(8)==null || !HSSFDateUtil.isCellDateFormatted(row.getCell(8))) {
    				errors.add("Timed Impulse Model in row " + (i+1) + " has missing or invalid trigger date.");
    			}
			} else if(type.toLowerCase().equals("rated")) {
    			// nothing to check
			} else if(type.toLowerCase().equals("boiloff")) {
				if(row.getCell(4)==null || row.getCell(4).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
        			errors.add("Boiloff Model in row " + (i+1) + " has missing or invalid boiloff rate.");
    			}
    		} else if(type.toLowerCase().equals("sparing by mass")) {
    			if(row.getCell(5)==null || row.getCell(5).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
        			errors.add("Sparing by Mass Model in row " + (i+1) + " has missing or invalid unpressurized spares rate.");
    			}
    			if(row.getCell(6)==null || row.getCell(6).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
        			errors.add("Sparing by Mass Model in row " + (i+1) + " has missing or invalid pressurized spares rate.");
    			}
    		} else if(type.toLowerCase().equals("isru")) {
    			if(row.getCell(7)==null || row.getCell(7).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
        			errors.add("ISRU Model in row " + (i+1) + " has missing or invalid resource id.");
    			}
    		} else if(!type.equals("")) {
    			errors.add("Model in row " + (i+1) + " has missing or invalid type.");
    		}
		}
		return errors;
	}
	private List<String> checkDemands(HSSFSheet sheet) {
		ArrayList<String> errors = new ArrayList<String>();
		for(int i = 1; i <= sheet.getLastRowNum(); i++) {
			HSSFRow row = sheet.getRow(i);
			if(row.getCell(0)==null || row.getCell(0).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Demand in row " + (i+1) + " has missing or invalid id.");
			}
			if(row.getCell(2)==null || row.getCell(2).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Demand in row " + (i+1) + " has missing or invalid resource id.");
			}
			if(row.getCell(3)==null || row.getCell(3).getCellType()!=HSSFCell.CELL_TYPE_NUMERIC) {
    			errors.add("Demand in row " + (i+1) + " has missing or invalid amount.");
			}
		}
		return errors;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.AbstractDataSource#loadLibraries(boolean)
	 */
	@Override
	public void loadLibraries(boolean updateNodes, boolean updateEdges, boolean updateResources) throws IOException {
		super.loadLibraries(updateNodes, updateEdges, updateResources);
    	POIFSFileSystem fs = new POIFSFileSystem(new FileInputStream(filePath));
    	HSSFWorkbook wb = new HSSFWorkbook(fs);
    	
    	HSSFSheet nodeSheet = wb.getSheetAt(0);
    	for(int i = 1; i <= nodeSheet.getLastRowNum(); i++) {
    		HSSFRow nodeRow = nodeSheet.getRow(i);
			String nodeType = nodeRow.getCell(1).toString().toLowerCase();
    		Node n = null;
    		if(nodeType.equals("surface")) {
    			n = new SurfaceNode();
				((SurfaceNode)n).setLatitude(nodeRow.getCell(4).getNumericCellValue());
				((SurfaceNode)n).setLongitude(nodeRow.getCell(5).getNumericCellValue());
    		} else if(nodeType.equals("orbital")) {
    			n = new OrbitalNode();
				((OrbitalNode)n).setApoapsis(nodeRow.getCell(6).getNumericCellValue());
				((OrbitalNode)n).setPeriapsis(nodeRow.getCell(7).getNumericCellValue());
				((OrbitalNode)n).setInclination(nodeRow.getCell(8).getNumericCellValue());
    		} else if(nodeType.equals("lagrange")) {
    			n = new LagrangeNode();
				((LagrangeNode)n).setMinorBody(Body.getInstance(nodeRow.getCell(9).toString()));
				((LagrangeNode)n).setNumber((int)nodeRow.getCell(10).getNumericCellValue());
    		}
    		
			n.setTid((int)nodeRow.getCell(0).getNumericCellValue());
			n.setName(nodeRow.getCell(2).toString());
			n.setBody(Body.getInstance(nodeRow.getCell(3).toString()));
			if(nodeRow.getCell(11)!=null) n.setDescription(nodeRow.getCell(11).getStringCellValue());
			if(!nodeLibrary.contains(n)) {
				nodeLibrary.add(n);
			} else if(updateNodes) {
				int index = nodeLibrary.indexOf(n);
				nodeLibrary.get(index).setName(n.getName());
				nodeLibrary.get(index).setDescription(n.getDescription());
				nodeLibrary.get(index).setBody(n.getBody());
				if(nodeLibrary.get(index).getClass().equals(n.getClass())) {
					if(nodeType.equals("surface")) {
						((SurfaceNode)nodeLibrary.get(index)).setLatitude(((SurfaceNode)n).getLatitude());
						((SurfaceNode)nodeLibrary.get(index)).setLongitude(((SurfaceNode)n).getLongitude());
					} else if(nodeType.equals("orbital")) {
						((OrbitalNode)nodeLibrary.get(index)).setApoapsis(((OrbitalNode)n).getApoapsis());
						((OrbitalNode)nodeLibrary.get(index)).setPeriapsis(((OrbitalNode)n).getPeriapsis());
						((OrbitalNode)nodeLibrary.get(index)).setInclination(((OrbitalNode)n).getInclination());
					} else if(nodeType.equals("lagrange")) {
						((LagrangeNode)nodeLibrary.get(index)).setMinorBody(((LagrangeNode)n).getMinorBody());
						((LagrangeNode)nodeLibrary.get(index)).setNumber(((LagrangeNode)n).getNumber());
					}
				}
			}
    	}
    	
    	HSSFSheet edgeSheet = wb.getSheetAt(1);
    	for(int i = 1; i <= edgeSheet.getLastRowNum(); i++) {
    		HSSFRow edgeRow = edgeSheet.getRow(i);
			String edgeType = edgeRow.getCell(1).toString().toLowerCase();
			Edge e = null;
    		if(edgeType.equals("space")) {
    			e = new SpaceEdge();
				((SpaceEdge)e).setDuration(edgeRow.getCell(5).getNumericCellValue());
    			HSSFSheet burnSheet = wb.getSheetAt(2);
		    	for(int j = 1; j <= burnSheet.getLastRowNum(); j++) {
		    		HSSFRow burnRow = burnSheet.getRow(j);
		    		if(burnRow.getCell(1).getCellType()==HSSFCell.CELL_TYPE_NUMERIC
		    				&& (int)edgeRow.getCell(0).getNumericCellValue() == (int)burnRow.getCell(1).getNumericCellValue()) {
    		    		Burn b = new Burn();
        				b.setTid((int)burnRow.getCell(0).getNumericCellValue());
        				b.setTime(burnRow.getCell(2).getNumericCellValue());
        				b.setBurnType(BurnType.getInstance(burnRow.getCell(4).toString()));
        				b.setDeltaV(burnRow.getCell(5).getNumericCellValue());
        				((SpaceEdge)e).getBurns().add((int)burnRow.getCell(3).getNumericCellValue(), b);
		    		}
		    	}
    		} else if(edgeType.equals("surface")) {
    			e = new SurfaceEdge();
				((SurfaceEdge)e).setDistance(edgeRow.getCell(6).getNumericCellValue());
    		} else if(edgeType.equals("flight")) {
    			e = new FlightEdge();
				((FlightEdge)e).setDuration(edgeRow.getCell(5).getNumericCellValue());
				((FlightEdge)e).setMaxCrewSize((int)edgeRow.getCell(7).getNumericCellValue());
				((FlightEdge)e).setMaxCargoMass(edgeRow.getCell(8).getNumericCellValue());
    		}
			e.setTid((int)edgeRow.getCell(0).getNumericCellValue());
			e.setName(edgeRow.getCell(2).toString());
			if(edgeRow.getCell(9)!=null) e.setDescription(edgeRow.getCell(9).getStringCellValue());

			for(Node n : nodeLibrary) {
				if(n.getTid() == (int)edgeRow.getCell(3).getNumericCellValue()) {
					e.setOrigin(n);
				} else if(n.getTid() == (int)edgeRow.getCell(4).getNumericCellValue()) {
					e.setDestination(n);
				}
			}
			if(!edgeLibrary.contains(e)) {
				edgeLibrary.add(e);
			} else if(updateEdges) {
				int index = edgeLibrary.indexOf(e);
				edgeLibrary.get(index).setName(e.getName());
				edgeLibrary.get(index).setDescription(e.getDescription());
				edgeLibrary.get(index).setOrigin(e.getOrigin());
				edgeLibrary.get(index).setDestination(e.getDestination());
				if(edgeLibrary.get(index).getClass().equals(e.getClass())) {
					if(edgeType.equals("surface")) {
						((SurfaceEdge)edgeLibrary.get(index)).setDistance(((SurfaceEdge)e).getDistance());
					} else if(edgeType.equals("space")) {
						((SpaceEdge)edgeLibrary.get(index)).setDuration(((SpaceEdge)e).getDuration());
						((SpaceEdge)edgeLibrary.get(index)).getBurns().clear();
						for(Burn burn : ((SpaceEdge)e).getBurns()) {
							((SpaceEdge)edgeLibrary.get(index)).getBurns().add(burn);
						}
					} else if(edgeType.equals("flight")) {
						((FlightEdge)edgeLibrary.get(index)).setDuration(((FlightEdge)e).getDuration());
						((FlightEdge)edgeLibrary.get(index)).setMaxCrewSize(((FlightEdge)e).getMaxCrewSize());
						((FlightEdge)edgeLibrary.get(index)).setMaxCargoMass(((FlightEdge)e).getMaxCargoMass());
					}
				}
			}
    	}

    	HSSFSheet resourceSheet = wb.getSheetAt(3);
    	for(int i = 1; i <= resourceSheet.getLastRowNum(); i++) {
    		HSSFRow resourceRow = resourceSheet.getRow(i);
    		String resourceType = resourceRow.getCell(1).toString().toLowerCase();
    		Resource r = null;
    		if(resourceType.equals("continuous")) {
    			r = new Resource();
    		} else if(resourceType.equals("discrete")) {
    			r = new Item();
    		} else {
    			System.out.println("Unrecognized Resource Type");
    		}
    		if(r != null) {
    			r.setTid((int)resourceRow.getCell(0).getNumericCellValue());
	    		r.setName(resourceRow.getCell(2).toString());
	    		r.setClassOfSupply(ClassOfSupply.getInstance((int)resourceRow.getCell(3).getNumericCellValue()));
	    		r.setUnitMass(resourceRow.getCell(4).getNumericCellValue());
	    		r.setUnitVolume(resourceRow.getCell(5).getNumericCellValue());
	    		r.setPackingFactor(resourceRow.getCell(6).getNumericCellValue());
    			r.setEnvironment(Environment.getInstance(resourceRow.getCell(7).getStringCellValue()));
	    		if(resourceRow.getCell(8) != null)  r.setDescription(resourceRow.getCell(8).getStringCellValue());
	    		if(!resourceTypeLibrary.contains(r)) {
	    			resourceTypeLibrary.add(r);
	    		} else if(updateResources) {
	    			int index = resourceTypeLibrary.indexOf(r);
	    			resourceTypeLibrary.get(index).setName(r.getName());
	    			resourceTypeLibrary.get(index).setClassOfSupply(r.getClassOfSupply());
	    			resourceTypeLibrary.get(index).setUnitMass(r.getUnitMass());
	    			resourceTypeLibrary.get(index).setUnitVolume(r.getUnitVolume());
	    			resourceTypeLibrary.get(index).setPackingFactor(r.getPackingFactor());
	    			resourceTypeLibrary.get(index).setEnvironment(r.getEnvironment());
	    			resourceTypeLibrary.get(index).setDescription(r.getDescription());
	    		}
    		}
    	}

    	HSSFSheet elementSheet = wb.getSheetAt(4);
    	for(int i = 1; i <= elementSheet.getLastRowNum(); i++) {
    		HSSFRow elementRow = elementSheet.getRow(i);
    		String elementType = elementRow.getCell(1).toString().toLowerCase();
    		Class<?> clazz = null;
    		ElementType type = null;
    		if(elementType.equals("element")) {
    			clazz = Element.class;
    			type = ElementType.ELEMENT;
    		} else if(elementType.equals("container")) {
    			clazz = ResourceContainer.class;
    			type = ElementType.RESOURCE_CONTAINER;
    		} else if(elementType.equals("tank")) {
    			clazz = ResourceTank.class;
    			type = ElementType.RESOURCE_TANK;
    		} else if(elementType.equals("carrier")) {
    			clazz = Carrier.class;
    			type = ElementType.CARRIER;
    		} else if(elementType.equals("propulsive")) {
    			clazz = PropulsiveVehicle.class;
    			type = ElementType.PROPULSIVE_VEHICLE;
    		} else if(elementType.equals("surface")) {
    			clazz = SurfaceVehicle.class;
    			type = ElementType.SURFACE_VEHICLE;
    		} else if(elementType.equals("crew member")) {
    			clazz = CrewMember.class;
    			type = ElementType.CREW_MEMBER;
    		} else {
    			System.out.println("Unrecognized Element Type");
    		}
    		if(clazz != null) {
    			ElementPreview e = new ElementPreview(
	    				(int)elementRow.getCell(0).getNumericCellValue(),
	    				elementRow.getCell(2).toString(), 
	    				type);
	    		if(!elementPreviewLibrary.contains(e)) {
	    			elementPreviewLibrary.add(e);
	    		} else if(true) {
	    			int index = elementPreviewLibrary.indexOf(e);
	    			elementPreviewLibrary.remove(index);
	    			elementPreviewLibrary.add(index, e);
	    		}
    		}
    	}
	}
	private void loadParts(HSSFWorkbook wb, I_Element element) {
		HSSFSheet partSheet = wb.getSheetAt(5);
		for(int j = 1; j <= partSheet.getLastRowNum(); j++) {
			HSSFRow partRow = partSheet.getRow(j);
			if((int)partRow.getCell(2).getNumericCellValue() == element.getTid()) {
				PartApplication partApp = new PartApplication();
				partApp.setPart((Item)loadResource((int)partRow.getCell(1).getNumericCellValue()));
				partApp.setQuantity((int)partRow.getCell(3).getNumericCellValue());
				partApp.setDutyCycle(partRow.getCell(4).getNumericCellValue());
				partApp.setMeanTimeToFailure(partRow.getCell(5).getNumericCellValue());
				partApp.setMeanTimeToRepair(partRow.getCell(6).getNumericCellValue());
				partApp.setMassToRepair(partRow.getCell(7).getNumericCellValue());
				element.getParts().add(partApp);
			}
		}
	}
	private void loadStates(HSSFWorkbook wb, I_Element element) {
		HSSFSheet stateSheet = wb.getSheetAt(6);
		for(int j = 1; j <= stateSheet.getLastRowNum(); j++) {
			HSSFRow stateRow = stateSheet.getRow(j);
			if((int)stateRow.getCell(1).getNumericCellValue() == element.getTid()) {
				State state = new State();
				state.setTid((int)stateRow.getCell(0).getNumericCellValue());
				state.setName(stateRow.getCell(2).getStringCellValue());
				state.setStateType(StateType.getInstance(stateRow.getCell(3).getStringCellValue()));
				state.setDutyCycle(stateRow.getCell(4).getNumericCellValue());	
				loadDemandModels(wb, element, state);
				element.getStates().add(state);
				if((int)stateRow.getCell(5).getNumericCellValue() == 1) 
					element.setCurrentState(state);
			}
		}
	}
	private void loadDemandModels(HSSFWorkbook wb, I_Element element, State state) {
		HSSFSheet modelSheet = wb.getSheetAt(7);
		for(int k = 1; k <= modelSheet.getLastRowNum(); k++) {
			HSSFRow modelRow = modelSheet.getRow(k);
			String modelType = modelRow.getCell(1).getStringCellValue().toLowerCase();
			if((int)modelRow.getCell(2).getNumericCellValue() == state.getTid()) {
				if(modelType.equals("timed")) {
					TimedImpulseDemandModel model = new TimedImpulseDemandModel();
					model.setTid((int)modelRow.getCell(0).getNumericCellValue());
					model.setName(modelRow.getCell(3).getStringCellValue());
					model.setTriggerDate(modelRow.getCell(8).getDateCellValue());
					HSSFSheet demandSheet = wb.getSheetAt(8);
					for(int l = 1; l <= demandSheet.getLastRowNum(); l++) {
						HSSFRow demandRow = demandSheet.getRow(l);
						if(demandRow.getCell(1)!=null 
								&& (int)demandRow.getCell(1).getNumericCellValue() == (int)modelRow.getCell(0).getNumericCellValue()) {
							Demand demand = new Demand();
							demand.setResource(loadResource((int)demandRow.getCell(2).getNumericCellValue()));
							demand.setAmount(demandRow.getCell(3).getNumericCellValue());
							model.getDemands().add(demand);
						}
					}
					state.getDemandModels().add(model);
				} else if(modelType.equals("rated")) {
					RatedDemandModel model = new RatedDemandModel();
					model.setTid((int)modelRow.getCell(0).getNumericCellValue());
					model.setName(modelRow.getCell(3).getStringCellValue());
					HSSFSheet demandSheet = wb.getSheetAt(8);
					for(int l = 1; l <= demandSheet.getLastRowNum(); l++) {
						HSSFRow demandRow = demandSheet.getRow(l);
						if(demandRow.getCell(1)!=null 
								&& (int)demandRow.getCell(1).getNumericCellValue() == (int)modelRow.getCell(0).getNumericCellValue()) {
							Demand demand = new Demand();
							demand.setResource(loadResource((int)demandRow.getCell(2).getNumericCellValue()));
							demand.setAmount(demandRow.getCell(3).getNumericCellValue());
							model.getDemandRates().add(demand);
						}
					}
					state.getDemandModels().add(model);
				} else if(modelType.equals("boiloff")) {
					BoilOffDemandModel model = new BoilOffDemandModel();
					model.setTid((int)modelRow.getCell(0).getNumericCellValue());
					model.setName(modelRow.getCell(3).getStringCellValue());
					model.setRate(modelRow.getCell(4).getNumericCellValue());
					if(element instanceof ResourceTank) {
						model.setTank((ResourceTank)element);
					} else if(element instanceof PropulsiveVehicle) {
						if(((PropulsiveVehicle)element).getOmsFuelTank() != null) {
							model.setTank(((PropulsiveVehicle)element).getOmsFuelTank());
						} else if(((PropulsiveVehicle)element).getRcsFuelTank() != null) {
							model.setTank(((PropulsiveVehicle)element).getRcsFuelTank());
						} else {
							System.out.println("Invalid Container");
						}
					} else if(element instanceof SurfaceVehicle) {
						if(((SurfaceVehicle)element).getFuelTank() != null) {
							model.setTank(((SurfaceVehicle)element).getFuelTank());
						} else {
							System.out.println("Invalid Container");
						}
					} else {
						System.out.println("Invalid Container");
					}
					state.getDemandModels().add(model);
				} else if(modelType.equals("sparing by mass")) {
					SparingByMassDemandModel model = new SparingByMassDemandModel(element);
					model.setTid((int)modelRow.getCell(0).getNumericCellValue());
					model.setName(modelRow.getCell(3).getStringCellValue());
					model.setElement(element);
					model.setUnpressurizedSparesRate(modelRow.getCell(5).getNumericCellValue());
					model.setPressurizedSparesRate(modelRow.getCell(6).getNumericCellValue());
					state.getDemandModels().add(model);
				} else if(modelType.equals("isru")) {
					IsruProductionModel model = new IsruProductionModel(element);
					model.setTid((int)modelRow.getCell(0).getNumericCellValue());
					model.setName(modelRow.getCell(3).getStringCellValue());
					model.setElement(element);
					model.setResource(loadResource((int)modelRow.getCell(7).getNumericCellValue()));
					state.getDemandModels().add(model);
				}
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#loadElement(int)
	 */
	public I_Element loadElement(int tid) {
		I_Element e = null;
		try {
	    	POIFSFileSystem fs = new POIFSFileSystem(new FileInputStream(filePath));
	    	HSSFWorkbook wb = new HSSFWorkbook(fs);
			HSSFSheet elementSheet = wb.getSheetAt(4);
	    	for(int i = 1; i <= elementSheet.getLastRowNum(); i++) {
	    		HSSFRow elementRow = elementSheet.getRow(i);
	    		if((int)elementRow.getCell(0).getNumericCellValue() == tid) {
	    			String elementType = elementRow.getCell(1).toString().toLowerCase();
	    			if(elementType.equals("element")) {
	    				e = new Element();
		    		} else if(elementType.equals("container")) {
		    			e = new ResourceContainer();
		    			((ResourceContainer)e).setMaxCargoMass(elementRow.getCell(9).getNumericCellValue());
		    			((ResourceContainer)e).setMaxCargoVolume(elementRow.getCell(10).getNumericCellValue());
		    			((ResourceContainer)e).setCargoEnvironment(Environment.getInstance(elementRow.getCell(11).getStringCellValue()));
		    			HSSFSheet demandSheet = wb.getSheetAt(8);
						for(int l = 1; l <= demandSheet.getLastRowNum(); l++) {
							HSSFRow demandRow = demandSheet.getRow(l);
							if(demandRow.getCell(4)!=null 
									&& (int)demandRow.getCell(4).getNumericCellValue() == (int)elementRow.getCell(0).getNumericCellValue()) {
								Demand demand = new Demand();
								demand.setResource(loadResource((int)demandRow.getCell(2).getNumericCellValue()));
								demand.setAmount(demandRow.getCell(3).getNumericCellValue());
								((ResourceContainer)e).getContents().put(demand.getResource(), demand.getAmount());
							}
						}
		    		} else if(elementType.equals("crew member")) {
		    			e = new CrewMember();
		    			((CrewMember)e).setAvailableTimeFraction(elementRow.getCell(12).getNumericCellValue());
		    		} else if(elementType.equals("carrier")) {
		    			e = new Carrier();
		    			((Carrier)e).setMaxCrewSize((int)elementRow.getCell(8).getNumericCellValue());
		    			((Carrier)e).setMaxCargoMass(elementRow.getCell(9).getNumericCellValue());
		    			((Carrier)e).setMaxCargoVolume(elementRow.getCell(10).getNumericCellValue());
		    			((Carrier)e).setCargoEnvironment(Environment.getInstance(elementRow.getCell(11).getStringCellValue()));
		    		} else if(elementType.equals("propulsive")) {
		    			e = new PropulsiveVehicle();
		    			((PropulsiveVehicle)e).setMaxCrewSize((int)elementRow.getCell(8).getNumericCellValue());
		    			((PropulsiveVehicle)e).setMaxCargoMass(elementRow.getCell(9).getNumericCellValue());
		    			((PropulsiveVehicle)e).setMaxCargoVolume(elementRow.getCell(10).getNumericCellValue());
		    			((PropulsiveVehicle)e).setCargoEnvironment(Environment.getInstance(elementRow.getCell(11).getStringCellValue()));
		    			((PropulsiveVehicle)e).setOmsIsp(elementRow.getCell(13).getNumericCellValue());
		    			((PropulsiveVehicle)e).setRcsIsp(elementRow.getCell(16).getNumericCellValue());
	    				if(((PropulsiveVehicle)e).getOmsIsp() > 0) {
	    					((PropulsiveVehicle)e).getOmsFuelTank().setMaxAmount(elementRow.getCell(14).getNumericCellValue());
	    					((PropulsiveVehicle)e).getOmsFuelTank().setAmount(elementRow.getCell(14).getNumericCellValue());
	    					((PropulsiveVehicle)e).getOmsFuelTank().setResource(loadResource((int)elementRow.getCell(15).getNumericCellValue()));
	    				} else {
	    					((PropulsiveVehicle)e).setOmsFuelTank(null);
	    				}
	    				if(((PropulsiveVehicle)e).getRcsIsp() > 0 && elementRow.getCell(17).getNumericCellValue() > 0) {
	    					((PropulsiveVehicle)e).getRcsFuelTank().setMaxAmount(elementRow.getCell(17).getNumericCellValue());
	    					((PropulsiveVehicle)e).getRcsFuelTank().setAmount(elementRow.getCell(17).getNumericCellValue());
	    					((PropulsiveVehicle)e).getRcsFuelTank().setResource(loadResource((int)elementRow.getCell(18).getNumericCellValue()));
	    				} else if(((PropulsiveVehicle)e).getRcsIsp() > 0) {
	    					((PropulsiveVehicle)e).getContents().remove(((PropulsiveVehicle)e).getRcsFuelTank());
	    					((PropulsiveVehicle)e).setRcsFuelTank(((PropulsiveVehicle)e).getOmsFuelTank());
	    				} else {
	    					((PropulsiveVehicle)e).setRcsFuelTank(null);
	    				}
		    		} else if(elementType.equals("surface")) {
		    			e = new SurfaceVehicle();
		    			((SurfaceVehicle)e).setMaxCrewSize((int)elementRow.getCell(8).getNumericCellValue());
		    			((SurfaceVehicle)e).setMaxCargoMass(elementRow.getCell(9).getNumericCellValue());
		    			((SurfaceVehicle)e).setMaxCargoVolume(elementRow.getCell(10).getNumericCellValue());
		    			((SurfaceVehicle)e).setCargoEnvironment(Environment.getInstance(elementRow.getCell(11).getStringCellValue()));
		    			((SurfaceVehicle)e).setMaxSpeed(elementRow.getCell(19).getNumericCellValue());
		    			((SurfaceVehicle)e).getFuelTank().setMaxAmount(elementRow.getCell(20).getNumericCellValue());
		    			((SurfaceVehicle)e).getFuelTank().setAmount(elementRow.getCell(20).getNumericCellValue());
		    			((SurfaceVehicle)e).getFuelTank().setResource(loadResource((int)elementRow.getCell(21).getNumericCellValue()));
		    		} else {
		    			System.out.println("Unrecognized Element Type");
		    		}
	    			e.setTid((int)elementRow.getCell(0).getNumericCellValue());
    				e.setName(elementRow.getCell(2).getStringCellValue());
    				e.setClassOfSupply(ClassOfSupply.getInstance(
    						(int)elementRow.getCell(3).getNumericCellValue()));
	    			e.setEnvironment(Environment.getInstance(elementRow.getCell(4).getStringCellValue()));
	    			e.setAccommodationMass(elementRow.getCell(5).getNumericCellValue());
    				e.setMass(elementRow.getCell(6).getNumericCellValue());
    				e.setVolume(elementRow.getCell(7).getNumericCellValue());
    				if(elementRow.getCell(22)!=null) e.setDescription(elementRow.getCell(22).getStringCellValue());
    				loadParts(wb, e);
    				loadStates(wb, e);
	    		}
	    	}
    	} catch(Exception ex) {
    		System.err.println("Error opening Excel document: " + ex.getMessage());
    		ex.printStackTrace();
    	}
    	return e;
	}
	
	/**
	 * Gets the file path.
	 * 
	 * @return the file path
	 */
	public String getFilePath() {
		return filePath;
	}
	
	/**
	 * Sets the file path.
	 * 
	 * @param filePath the new file path
	 */
	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getDataSourceType()
	 */
	public DataSourceType getDataSourceType() {
		return DataSourceType.EXCEL_2_5;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.scenario.data.I_DataSource#getName()
	 */
	public String getName() {
		if(filePath==null||filePath.length()==0) return getDataSourceType().getName();
		int lastIndex = Math.max(filePath.lastIndexOf("/"), filePath.lastIndexOf("\\"));
		return filePath.substring(lastIndex+1, filePath.length());
	}
}
